@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: aac_common.cpp 2802 2005-02-20 18:40:37Z mosu $
+   $Id: aac_common.cpp 2898 2005-04-28 14:50:43Z mosu $
 
    helper function for AAC data
 
@@ -217,7 +217,10 @@ parse_aac_data(unsigned char *data,
     mxverb(4, "%02x ", data[i]);
   mxverb(4, "\n");
 
-  profile = (data[0] >> 3) - 1;
+  profile = data[0] >> 3;
+  if (0 == profile)
+    return false;
+  --profile;
   sample_rate = aac_sampling_freq[((data[0] & 0x07) << 1) | (data[1] >> 7)];
   channels = (data[1] & 0x7f) >> 3;
   if (size == 5) {
