@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: chapters.cpp 2872 2005-04-07 08:46:58Z mosu $
+   \version $Id: chapters.cpp 2979 2005-07-07 14:09:27Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
 */
@@ -58,12 +58,11 @@ string default_chapter_country;
                           isseparator(s + 18) && \
                           isthreedigits(s + 19))
 /** Does \c s point to a valid OGM style chapter name entry? */
-#define ischapternameline(s) ((strlen(s) >= 15) && \
+#define ischapternameline(s) ((strlen(s) >= 14) && \
                           ischapter(s) && \
                           istwodigits(s + 7) && \
                           isname(s + 9) && \
-                          isequal(s + 13) && \
-                          !isblanktab(*(s + 14)))
+                          isequal(s + 13))
 
 /** \brief Format an error message and throw an exception.
 
@@ -187,7 +186,7 @@ parse_simple_chapters(mm_text_io_c *in,
   KaxChapterAtom *atom;
   KaxChapterDisplay *display;
   int64_t start, hour, minute, second, msecs;
-  string name, line, use_language;
+  string name, line, use_language, s_timecode;
   int mode, num, cc_utf8;
   bool do_convert;
   UTFstring wchar_string;
@@ -247,11 +246,14 @@ parse_simple_chapters(mm_text_io_c *in,
         start = msecs + second * 1000 + minute * 1000 * 60 +
           hour * 1000 * 60 * 60;
         mode = 1;
+        s_timecode = line.substr(10);
 
       } else {
         if (!ischapternameline(line.c_str()))
           chapter_error("'%s' is not a CHAPTERxxNAME=... line.", line.c_str());
         name = line.substr(14);
+        if (name == "")
+          name = s_timecode;
         mode = 0;
 
         if ((start >= min_tc) && ((start <= max_tc) || (max_tc == -1))) {
@@ -363,7 +365,7 @@ parse_chapters(const string &file_name,
     if (exception_on_error)
       throw;
     mxerror("Could not parse the chapters in '%s': %s\n", file_name.c_str(),
-            e.get_error());
+            e.get_error().c_str());
   } catch (...) {
     if (in != NULL)
       delete in;
@@ -446,7 +448,7 @@ parse_chapters(mm_text_io_c *in,
   } catch (error_c e) {
     if (exception_on_error)
       throw e;
-    mxerror("%s", e.get_error());
+    mxerror("%s", e.get_error().c_str());
   }
 
   return NULL;
