@@ -7,7 +7,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.cpp 2887 2005-04-11 10:47:01Z mosu $
+   $Id: common.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    helper functions, common variables
 
@@ -57,9 +57,12 @@ using namespace libebml;
 
 #include "base64.h"
 #include "common.h"
+#include "common_memory.h"
 #include "error.h"
 #include "hacks.h"
 #include "mm_io.h"
+#include "random.h"
+#include "smart_pointers.h"
 
 int verbose = 1;
 bool suppress_warnings = false;
@@ -117,7 +120,7 @@ bitvalue_c::bitvalue_c(string s,
 
     // Input too long?
     if ((allowed_bitlength > 0) && ((s2.length() * 4) >= allowed_bitlength))
-      throw error_c(mxsprintf("input too long: %d > %d", s2.length() * 4,
+      throw error_c(mxsprintf("input too long: %d > %d", (int)s2.length() * 4,
                               allowed_bitlength));
 
     // Store the value.
@@ -185,10 +188,7 @@ bitvalue_c::data()
 
 void
 bitvalue_c::generate_random() {
-  int i;
-
-  for (i = 0; i < bitsize / 8; i++)
-    value[i] = (unsigned char)(255.0 * rand() / RAND_MAX);
+  random_c::generate_bytes(value, bitsize / 8);
 }
 
 /*
@@ -430,29 +430,30 @@ put_uint64_be(void *buf,
    Character map conversion stuff
 */
 
-typedef struct {
+struct kax_conv_t {
   iconv_t ict_from_utf8, ict_to_utf8;
-  char *charset;
-} kax_conv_t;
+  string charset;
+
+  kax_conv_t(iconv_t n_ict_from_utf8, iconv_t n_ict_to_utf8,
+             const string &n_charset):
+    ict_from_utf8(n_ict_from_utf8), ict_to_utf8(n_ict_to_utf8),
+    charset(n_charset) { }
+};
 
 static vector<kax_conv_t> kax_convs;
 int cc_local_utf8 = -1;
 
 int
-add_kax_conv(const char *charset,
+add_kax_conv(const string &charset,
              iconv_t ict_from,
              iconv_t ict_to) {
-  kax_conv_t c;
   int i;
 
   for (i = 0; i < kax_convs.size(); i++)
-    if (!strcmp(kax_convs[i].charset, charset))
+    if (kax_convs[i].charset == charset)
       return i;
 
-  c.charset = safestrdup(charset);
-  c.ict_from_utf8 = ict_from;
-  c.ict_to_utf8 = ict_to;
-  kax_convs.push_back(c);
+  kax_convs.push_back(kax_conv_t(ict_from, ict_to, charset));
 
   return kax_convs.size() - 1;
 }
@@ -465,6 +466,8 @@ get_local_charset() {
 #if defined(COMP_MINGW) || defined(COMP_MSC)
   lc_charset = "CP" + to_string(GetACP());
 #elif defined(SYS_SOLARIS)
+  int i;
+
   lc_charset = nl_langinfo(CODESET);
   if (parse_int(lc_charset, i))
     lc_charset = string("ISO") + lc_charset + string("-US");
@@ -523,7 +526,6 @@ utf8_done() {
       iconv_close(kax_convs[i].ict_from_utf8);
     if (kax_convs[i].ict_to_utf8 != (iconv_t)(-1))
       iconv_close(kax_convs[i].ict_to_utf8);
-    safefree(kax_convs[i].charset);
   }
   kax_convs.clear();
 }
@@ -566,8 +568,8 @@ to_utf8(int handle,
     return local;
 
   if (handle >= kax_convs.size())
-    die("common.cpp/to_utf8(): Invalid conversion handle %d (num: %d).",
-        handle, kax_convs.size());
+    die("common.cpp/to_utf8(): Invalid conversion handle %d (num: %u).",
+        handle, (unsigned int)kax_convs.size());
 
   return convert_charset(kax_convs[handle].ict_to_utf8, local);
 }
@@ -581,15 +583,24 @@ from_utf8(int handle,
     return utf8;
 
   if (handle >= kax_convs.size())
-    die("common.cpp/from_utf8(): Invalid conversion handle %d (num: %d).",
-        handle, kax_convs.size());
+    die("common.cpp/from_utf8(): Invalid conversion handle %d (num: %u).",
+        handle, (unsigned int)kax_convs.size());
 
   return convert_charset(kax_convs[handle].ict_from_utf8, utf8);
 }
 
+static int cc_stdio = -1;
+string stdio_charset;
+
 void
 init_cc_stdio() {
-  cc_stdio = utf8_init(get_local_charset());
+  set_cc_stdio(get_local_charset());
+}
+
+void
+set_cc_stdio(const string &charset) {
+  stdio_charset = charset;
+  cc_stdio = utf8_init(charset);
 }
 
 /*
@@ -597,7 +608,6 @@ init_cc_stdio() {
 */
 
 static vector<uint32_t> ru_numbers[4];
-static bool random_seeded = false;
 
 void
 clear_list_of_unique_uint32(unique_id_category_e category) {
@@ -661,7 +671,7 @@ remove_unique_uint32(uint32_t number,
 
 uint32_t
 create_unique_uint32(unique_id_category_e category) {
-  uint32_t rnumber, half;
+  uint32_t rnumber;
 
   assert((category >= UNIQUE_TRACK_IDS) &&
          (category <= UNIQUE_ATTACHMENT_IDS));
@@ -671,16 +681,8 @@ create_unique_uint32(unique_id_category_e category) {
     return ru_numbers[category].size();
   }
 
-  if (!random_seeded) {
-    srand(time(NULL));
-    random_seeded = true;
-  }
-
   do {
-    half = (uint32_t)(65535.0 * rand() / RAND_MAX);
-    rnumber = half;
-    half = (uint32_t)(65535.0 * rand() / RAND_MAX);
-    rnumber |= (half << 16);
+    rnumber = random_c::generate_32bits();
   } while ((rnumber == 0) || !is_unique_uint32(rnumber, category));
   add_unique_uint32(rnumber, category);
 
@@ -709,7 +711,8 @@ _safememdup(const void *s,
   copy = malloc(size);
   if (copy == NULL)
     die("common.cpp/safememdup() called from file %s, line %d: malloc() "
-        "returned NULL for a size of %d bytes.", file, line, size);
+        "returned NULL for a size of %u bytes.", file, line,
+        (unsigned int)size);
   memcpy(copy, s, size);
 
   return copy;
@@ -726,7 +729,8 @@ _safemalloc(size_t size,
   mem = malloc(size);
   if (mem == NULL)
     die("common.cpp/safemalloc() called from file %s, line %d: malloc() "
-        "returned NULL for a size of %d bytes.", file, line, size);
+        "returned NULL for a size of %u bytes.", file, line,
+        (unsigned int)size);
 
   return mem;
 }
@@ -739,7 +743,8 @@ _saferealloc(void *mem,
   mem = realloc(mem, size);
   if (mem == NULL)
     die("common.cpp/saferealloc() called from file %s, line %d: realloc() "
-        "returned NULL for a size of %d bytes.", file, line, size);
+        "returned NULL for a size of %u bytes.", file, line,
+        (unsigned int)size);
 
   return mem;
 }
@@ -896,7 +901,8 @@ unescape(const string &source) {
 }
 
 string
-escape_xml(const string &source) {
+escape_xml(const string &source,
+           bool escape_quotes) {
   string dst;
   string::const_iterator src;
 
@@ -908,6 +914,8 @@ escape_xml(const string &source) {
       dst += "&gt;";
     else if (*src == '<')
       dst += "&lt;";
+    else if (escape_quotes && (*src == '"'))
+      dst += "&quot;";
     else
       dst += *src;
     src++;
@@ -916,6 +924,21 @@ escape_xml(const string &source) {
   return dst;
 }
 
+string
+create_xml_node_name(const char *name,
+                     const char **atts) {
+  int i;
+  string node_name;
+
+  node_name = string("<") + name;
+  for (i = 0; (NULL != atts[i]) && (NULL != atts[i + 1]); i += 2)
+    node_name += string(" ") + atts[i] + "=\"" +
+      escape_xml(atts[i + 1], true) + "\"";
+  node_name += ">";
+
+  return node_name;
+}
+
 bool
 starts_with(const string &s,
             const char *start,
@@ -1234,8 +1257,6 @@ fix_format(const char *fmt,
 #endif
 }
 
-int cc_stdio = -1;
-
 void
 mxprint(void *stream,
         const char *fmt,
@@ -1263,13 +1284,32 @@ mxprints(char *dst,
   va_end(ap);
 }
 
+counted_ptr<mm_io_c> mm_stdio;
+static bool mm_stdio_redirected = false;
+
+void
+init_stdio() {
+  mm_stdio = counted_ptr<mm_io_c>(new mm_stdio_c());
+  mm_stdio_redirected = false;
+}
+
+void
+redirect_stdio(mm_io_c *stdio) {
+  mm_stdio = counted_ptr<mm_io_c>(stdio);
+  mm_stdio_redirected = true;
+}
+
+bool
+stdio_redirected() {
+  return mm_stdio_redirected;
+}
+
 static void
 mxmsg(int level,
       const char *fmt,
       va_list ap) {
   string new_fmt, output;
   bool nl;
-  FILE *stream;
   char *prefix;
 
   fix_format(fmt, new_fmt);
@@ -1280,7 +1320,6 @@ mxmsg(int level,
   } else
     nl = false;
 
-  stream = stdout;
   prefix = NULL;
 
   if (level == MXMSG_ERROR)
@@ -1291,14 +1330,14 @@ mxmsg(int level,
     prefix = "DBG> ";
 
   if (nl)
-    fprintf(stream, "\n");
+    mm_stdio->puts("\n");
 
   if (prefix != NULL)
-    fprintf(stream, prefix);
+    mm_stdio->puts(prefix);
 
   output = from_utf8(cc_stdio, mxvsprintf(new_fmt.c_str(), ap));
-  fputs(output.c_str(), stream);
-  fflush(stream);
+  mm_stdio->puts(output);
+  mm_stdio->flush();
 }
 
 void
@@ -1525,46 +1564,131 @@ mxhexdump(int level,
   mxinfo("%s\n", output);
 }
 
-const char *timecode_parser_error = NULL;
+string timecode_parser_error;
 
 static bool
-set_tcp_error(const char *error) {
+set_tcp_error(const string &error) {
   timecode_parser_error = error;
   return false;
 }
 
 bool
-parse_timecode(const char *src,
-               int64_t *timecode) {
+parse_timecode(const string &src,
+               int64_t &timecode) {
   // Recognized format:
-  // HH:MM:SS and HH:MM:SS.nnn with up to nine 'n' for ns precision
-  //              012345678901...
-  int h, m, s, n, i;
-  char format[10];
+  // 1. XXXXXXXs   with XXXXXX being a number of seconds followed
+  //    by the letter 's',
+  // 2. HH:MM:SS.nnnnnnnnn  with up to nine digits 'n' for ns precision;
+  // HH: is optional; HH, MM and SS can be either one or two digits.
+  int h, m, s, n, i, values[4], num_values, num_digits, num_colons;
+  bool decimal_point_found;
 
-  if ((strlen(src) < 8) || (strlen(src) > 18) || (src[2] != ':') ||
-      (src[5] != ':'))
-    return set_tcp_error("Invalid format");
-  if (sscanf(src, "%02d:%02d:%02d", &h, &m, &s) != 3)
-    return set_tcp_error("Invalid format (non-numbers encountered)");
-  if (m > 59)
-    return set_tcp_error("Invalid minute");
-  if (s > 59)
-    return set_tcp_error("Invalid second");
-  if (strlen(src) > 9) {
-    if (src[8] != '.')
-      return set_tcp_error("Invalid format (expected a dot '.' after the "
-                           "seconds)");
-    sprintf(format, "%%0%dd", strlen(src) - 8);
-    if (sscanf(&src[9], format, &n) != 1)
-      return set_tcp_error("Invalid format (non-numbers encountered)");
-    for (i = strlen(src); i < 18; i++)
+  if (src.empty())
+    return false;
+
+  // Number of seconds postfixed by 's'.
+  if ('s' == src[src.length() - 1]) {
+    string tmp = src;
+
+    tmp.erase(tmp.length() - 1);
+    if (!parse_int(tmp, timecode))
+      return set_tcp_error("Non-digits found but expected a nunber of seconds "
+                           "before the 's' at the end");
+
+    timecode *= 1000000000ll;
+    return true;
+  }
+
+  num_digits = 0;
+  num_colons = 0;
+  num_values = 1;
+  decimal_point_found = false;
+  memset(&values, 0, sizeof(int) * 4);
+
+  for (i = 0; src.length() > i; ++i) {
+    if (isdigit(src[i])) {
+      if (decimal_point_found && (9 == num_digits))
+        return set_tcp_error("Invalid format: More than nine nano-second "
+                             "digits");
+      values[num_values - 1] = values[num_values - 1] * 10 + src[i] - '0';
+      ++num_digits;
+
+    } else if (':' == src[i]) {
+      if (decimal_point_found)
+        return set_tcp_error("Invalid format: Colon inside nano-second part");
+      if (2 == num_colons)
+        return set_tcp_error("Invalid format: More than two colons");
+      if (0 == num_digits)
+        return set_tcp_error("Invalid format: No digits before colon");
+      ++num_colons;
+      ++num_values;
+      num_digits = 0;
+
+    } else if ('.' == src[i]) {
+      if (decimal_point_found)
+        return set_tcp_error("Invalid format: Second decimal point after "
+                             "first decimal point");
+
+      if (0 == num_digits)
+        return set_tcp_error("Invalid format: No digits before decimal point");
+      ++num_values;
+      num_digits = 0;
+      decimal_point_found = true;
+
+    } else
+      return set_tcp_error(mxsprintf("Invalid format: unknown character '%c' "
+                                     "found", src[i]));
+  }
+
+  if (1 > num_colons)
+    return set_tcp_error("Invalid format: At least minutes and seconds "
+                         "have to be given, but no colon was found");
+
+  if ((':' == src[src.length() - 1]) || ('.' == src[src.length() - 1]))
+    return set_tcp_error("Invalid format: The last character is a colon or a "
+                         "decimal point instead of a digit");
+
+  // No error has been found. Now find out whoich parts have been
+  // set and which haven't.
+
+  if (4 == num_values) {
+    h = values[0];
+    m = values[1];
+    s = values[2];
+    n = values[3];
+    for (; 9 > num_digits; ++num_digits)
       n *= 10;
-  } else
+
+  } else if (2 == num_values) {
+    h = 0;
+    m = values[0];
+    s = values[1];
     n = 0;
-  if (timecode != NULL)
-    *timecode = ((int64_t)h * 60 * 60 + (int64_t)m * 60 + (int64_t)s) *
-      1000000000 + n;
+
+  } else if (decimal_point_found) {
+    h = 0;
+    m = values[0];
+    s = values[1];
+    n = values[2];
+    for (; 9 > num_digits; ++num_digits)
+      n *= 10;
+
+  } else {
+    h = values[0];
+    m = values[1];
+    s = values[2];
+    n = 0;
+
+  }
+
+  if (m > 59)
+    return set_tcp_error(mxsprintf("Invalid number of minutes: %d > 59", m));
+  if (s > 59)
+    return set_tcp_error(mxsprintf("Invalid number of seconds: %d > 59", s));
+
+  timecode = ((int64_t)h * 60 * 60 + (int64_t)m * 60 + (int64_t)s) *
+    1000000000ll + n;
+
   timecode_parser_error = "no error";
   return true;
 }
@@ -1713,3 +1837,92 @@ command_line_utf8(int,
   return args;
 }
 #endif // !defined(SYS_WINDOWS)
+
+string usage_text, version_info;
+
+/** Handle command line arguments common to all programs
+
+   Iterates over the list of command line arguments and handles the ones
+   that are common to all programs. These include --output-charset,
+   --redirect-output, --help, --version and --verbose along with their
+   short counterparts.
+
+   \param args A vector of strings containing the command line arguments.
+     The ones that have been handled are removed from the vector.
+   \param redirect_output_short The name of the short option that is
+     recognized for --redirect-output. If left empty then no short
+     version is accepted.
+*/
+void
+handle_common_cli_args(vector<string> &args,
+                       const string &redirect_output_short) {
+  int i;
+
+  // First see if there's an output charset given.
+  i = 0;
+  while (args.size() > i) {
+    if (args[i] == "--output-charset") {
+      if ((i + 1) == args.size())
+        mxerror("Missing argument for '--output-charset'.\n");
+      set_cc_stdio(args[i + 1]);
+      args.erase(args.begin() + i, args.begin() + i + 2);
+    } else
+      ++i;
+  }
+
+  // Now let's see if the user wants the output redirected.
+  i = 0;
+  while (args.size() > i) {
+    if ((args[i] == "--redirect-output") || (args[i] == "-r") ||
+        ((redirect_output_short != "") &&
+         (args[i] == redirect_output_short))) {
+      if ((i + 1) == args.size())
+        mxerror("'%s' is missing the file name.\n", args[i].c_str());
+      try {
+        if (!stdio_redirected()) {
+          mm_file_io_c *file = new mm_file_io_c(args[i + 1], MODE_CREATE);
+          file->write_bom(stdio_charset);
+          redirect_stdio(file);
+        }
+        args.erase(args.begin() + i, args.begin() + i + 2);
+      } catch(mm_io_open_error_c &e) {
+        mxerror("Could not open the file '%s' for directing the output.\n",
+                args[i + 1].c_str());
+      }
+    } else
+      ++i;
+  }
+
+  // Last find the --help and --version arguments.
+  i = 0;
+  while (args.size() > i) {
+    if ((args[i] == "-V") || (args[i] == "--version")) {
+      mxinfo("%s built on %s %s\n", version_info.c_str(), __DATE__, __TIME__);
+      mxexit(0);
+
+    } else if ((args[i] == "-v") || (args[i] == "--verbose")) {
+      ++verbose;
+      args.erase(args.begin() + i, args.begin() + i + 1);
+
+    } else if ((args[i] == "-h") || (args[i] == "-?") ||
+             (args[i] == "--help"))
+      usage();
+
+    else
+      ++i;
+  }
+}
+
+void
+usage(int exit_code) {
+  mxinfo("%s\n", usage_text.c_str());
+  mxexit(exit_code);
+}
+
+buffer_t::buffer_t():
+  buffer(NULL), size(0) {
+}
+
+buffer_t::~buffer_t() {
+  safefree(buffer);
+}
