@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.h 2885 2005-04-09 17:36:20Z mosu $
+   $Id: common.h 3033 2005-08-21 13:06:50Z mosu $
 
    definitions used in all programs, helper functions
 
@@ -55,7 +55,7 @@ namespace libebml {
 
 using namespace libebml;
 
-#define VERSIONNAME "Jimi Thing"
+#define VERSIONNAME "Another White Dash"
 #define VERSIONINFO "mkvmerge v" VERSION " ('" VERSIONNAME "')"
 #define BUGMSG _("This should not have happened. Please contact the author " \
                  "Moritz Bunkus <moritz@bunkus.org> with this error/warning " \
@@ -93,8 +93,8 @@ using namespace std;
                             (int32_t)(((t) / 1000000000) % 60), \
                             (int32_t)((t) % 1000000000)
 #define ARG_TIMECODEN(t) ARG_TIMECODENINT((int64_t)(t))
-extern const char * MTX_DLL_API timecode_parser_error;
-extern bool MTX_DLL_API parse_timecode(const char *src, int64_t *timecode);
+extern string MTX_DLL_API timecode_parser_error;
+extern bool MTX_DLL_API parse_timecode(const string &s, int64_t &timecode);
 
 extern bool MTX_DLL_API suppress_warnings;
 void MTX_DLL_API fix_format(const char *fmt, string &new_fmt);
@@ -139,6 +139,12 @@ void MTX_DLL_API _trace(const char *func, const char *file, int line);
 
 void MTX_DLL_API mxhexdump(int level, const unsigned char *buffer, int lenth);
 
+class mm_io_c;
+
+void MTX_DLL_API init_stdio();
+void MTX_DLL_API redirect_stdio(mm_io_c *new_stdio);
+bool MTX_DLL_API stdio_redirected();
+
 #define get_fourcc(b) get_uint32_be(b)
 uint16_t MTX_DLL_API get_uint16_le(const void *buf);
 uint32_t MTX_DLL_API get_uint24_le(const void *buf);
@@ -155,8 +161,10 @@ void MTX_DLL_API put_uint16_be(void *buf, uint16_t value);
 void MTX_DLL_API put_uint32_be(void *buf, uint32_t value);
 void MTX_DLL_API put_uint64_be(void *buf, uint64_t value);
 
-extern int MTX_DLL_API cc_local_utf8, cc_stdio;
+extern int MTX_DLL_API cc_local_utf8;
 void MTX_DLL_API init_cc_stdio();
+void MTX_DLL_API set_cc_stdio(const string &charset);
+extern string MTX_DLL_API stdio_charset;
 string MTX_DLL_API get_local_charset();
 int MTX_DLL_API utf8_init(const string &charset);
 void MTX_DLL_API utf8_done();
@@ -164,6 +172,10 @@ string MTX_DLL_API to_utf8(int handle, const string &local);
 string MTX_DLL_API from_utf8(int handle, const string &utf8);
 
 vector<string> MTX_DLL_API command_line_utf8(int argc, char **argv);
+extern string MTX_DLL_API usage_text, version_info;
+void MTX_DLL_API usage(int exit_code = 0);
+void MTX_DLL_API handle_common_cli_args(vector<string> &args,
+                                        const string &redirect_output_short);
 
 enum unique_id_category_e {
   UNIQUE_ALL_IDS = -1,
@@ -225,8 +237,9 @@ string MTX_DLL_API join(const char *pattern, vector<string> &strings);
 void MTX_DLL_API strip(string &s, bool newlines = false);
 void MTX_DLL_API strip(vector<string> &v, bool newlines = false);
 string MTX_DLL_API escape(const string &src);
-string MTX_DLL_API escape_xml(const string &src);
 string MTX_DLL_API unescape(const string &src);
+string MTX_DLL_API escape_xml(const string &src, bool escape_quotes = false);
+string MTX_DLL_API create_xml_node_name(const char *name, const char **atts);
 bool MTX_DLL_API starts_with(const string &s, const char *start,
                              int maxlen = -1);
 inline bool
