@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: commonebml.cpp 2789 2005-02-11 20:38:00Z mosu $
+   $Id: commonebml.cpp 2840 2005-03-01 14:52:36Z mosu $
 
    helper functions that need libebml/libmatroska
 
@@ -16,9 +16,11 @@
 #include "os.h"
 
 #if defined(COMP_MSC)
-#include <libcharset.h>
-#elif !defined(COMP_MINGW)
-#include <langinfo.h>
+# include <libcharset.h>
+#elif HAVE_NL_LANGINFO
+# include <langinfo.h>
+#elif HAVE_LOCALE_CHARSET
+# include <libcharset.h>
 #endif
 #if defined(SYS_WINDOWS)
 #include <windows.h>
