@@ -1,21 +1,16 @@
 /*
-  mkvmerge -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge -- utility for splicing together matroska files
+   from component media subtypes
 
-  iso639.cpp
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
+   $Id: iso639.cpp 2719 2005-01-17 13:58:24Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   ISO639 language definitions, lookup functions
 
-/*!
-    \file
-    \version $Id: iso639.cpp 1513 2004-02-23 16:04:30Z mosu $
-    \brief ISO639 language definitions, lookup functions
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #include <stdio.h>
@@ -151,8 +146,8 @@ const iso639_language_t iso639_languages[] =
    {"Dogrib", "dgr", NULL},
    {"Dravidian (Other)", "dra", NULL},
    {"Duala", "dua", NULL},
-   {"Dutch; Flemish", "dut", "nl"},
    {"Dutch", "nld", "nl"},
+   {"Dutch; Flemish", "dut", "nl"},
    {"Dutch, Middle (ca. 1050-1350)", "dum", NULL},
    {"Dyula", "dyu", NULL},
    {"Dzongkha", "dzo", "dz"},
@@ -214,6 +209,7 @@ const iso639_language_t iso639_languages[] =
    {"Hausa", "hau", "ha"},
    {"Hawaiian", "haw", NULL},
    {"Hebrew", "heb", "he"},
+   {"Hebrew", "heb", "iw"},
    {"Herero", "her", "hz"},
    {"Hiligaynon", "hil", NULL},
    {"Himachali", "him", NULL},
@@ -270,6 +266,7 @@ const iso639_language_t iso639_languages[] =
    {"Kimbundu", "kmb", NULL},
    {"Kinyarwanda", "kin", "rw"},
    {"Kirghiz", "kir", "ky"},
+   {"Klingon; tlhlngan-Hol", "tlh", NULL},
    {"Komi", "kom", "kv"},
    {"Kongo", "kon", "kg"},
    {"Konkani", "kok", NULL},
@@ -548,18 +545,33 @@ char *get_iso639_english_name(const char *iso639_2_code) {
   return NULL;
 }
 
-int
+bool
 is_valid_iso639_2_code(const char *iso639_2_code) {
   int i;
 
   i = 0;
   while (iso639_languages[i].iso639_2_code != NULL) {
     if (!strcmp(iso639_languages[i].iso639_2_code, iso639_2_code))
-      return i;
+      return true;
     i++;
   }
 
-  return 0;
+  return false;
+}
+
+bool
+is_valid_iso639_1_code(const char *iso639_1_code) {
+  int i;
+
+  i = 0;
+  while (iso639_languages[i].iso639_2_code != NULL) {
+    if ((iso639_languages[i].iso639_1_code != NULL) &&
+        !strcmp(iso639_languages[i].iso639_1_code, iso639_1_code))
+      return true;
+    i++;
+  }
+
+  return false;
 }
 
 void
@@ -584,12 +596,20 @@ list_iso639_languages() {
 
 const char *
 map_iso639_1_to_iso639_2(const char *iso639_1_code) {
-  uint32_t i;
+  int i, ndx;
 
+  ndx = -1;
   for (i = 0; iso639_languages[i].iso639_2_code != NULL; i++)
     if ((iso639_languages[i].iso639_1_code != NULL) &&
-        !strcmp(iso639_1_code, iso639_languages[i].iso639_1_code))
-      return iso639_languages[i].iso639_2_code;
+        !strcmp(iso639_1_code, iso639_languages[i].iso639_1_code)) {
+      if (is_popular_language_code(iso639_languages[i].iso639_2_code))
+        return iso639_languages[i].iso639_2_code;
+      else if (ndx == -1)
+        ndx = i;
+    }
+
+  if (ndx >= 0)
+    return iso639_languages[ndx].iso639_2_code;
 
   return NULL;
 }
@@ -620,3 +640,39 @@ map_english_name_to_iso639_2(const char *name) {
 
   return NULL;
 }
+
+bool
+is_popular_language(const char *lang) {
+  return
+    !strcmp(lang, "Chinese") ||
+    !strcmp(lang, "Dutch") ||
+    !strcmp(lang, "English") ||
+    !strcmp(lang, "Finnish") ||
+    !strcmp(lang, "French") ||
+    !strcmp(lang, "German") ||
+    !strcmp(lang, "Italian") ||
+    !strcmp(lang, "Japanese") ||
+    !strcmp(lang, "Norwegian") ||
+    !strcmp(lang, "Portuguese") ||
+    !strcmp(lang, "Russian") ||
+    !strcmp(lang, "Spanish") ||
+    !strcmp(lang, "Swedish");
+}
+
+bool
+is_popular_language_code(const char *code) {
+  return
+    !strcmp(code, "zho") || // Chinese
+    !strcmp(code, "dut") || // Dutch
+    !strcmp(code, "eng") || // English
+    !strcmp(code, "fin") || // Finnish
+    !strcmp(code, "fre") || // French
+    !strcmp(code, "ger") || // German
+    !strcmp(code, "ita") || // Italian
+    !strcmp(code, "jpn") || // Japanese
+    !strcmp(code, "nor") || // Norwegian
+    !strcmp(code, "por") || // Portuguese
+    !strcmp(code, "rus") || // Russian
+    !strcmp(code, "spa") || // Spanish
+    !strcmp(code, "swe");   // Swedish
+}
