@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mm_io.cpp 2886 2005-04-10 09:07:45Z mosu $
+   $Id: mm_io.cpp 3029 2005-08-17 15:28:47Z mosu $
 
    IO callback class implementation
 
@@ -91,14 +91,14 @@ mm_file_io_c::mm_file_io_c(const string &path,
 # endif
       break;
     default:
-      throw error_c("Unknown open mode");
+      throw mm_io_error_c("Unknown open mode");
   }
 
   local_path = from_utf8(cc_local_utf8, path);
   file = (FILE *)fopen(local_path.c_str(), cmode);
 
   if (file == NULL)
-    throw error_c(mxsprintf("Error opening file %s", path.c_str()));
+    throw mm_io_open_error_c();
 
 # if HAVE_POSIX_FADVISE
   if (use_posix_fadvise && use_posix_fadvise_here &&
@@ -123,7 +123,7 @@ mm_file_io_c::setFilePointer(int64 offset,
     whence = SEEK_CUR;
 
   if (fseeko((FILE *)file, offset, whence) != 0)
-    throw error_c("seeking failed");
+    throw mm_io_seek_error_c();
 
   if (mode == seek_beginning)
     m_current_position = offset;
@@ -197,8 +197,10 @@ mm_file_io_c::read(void *buffer,
 
 void
 mm_file_io_c::close() {
-  if (file != NULL)
+  if (NULL != file) {
     fclose((FILE *)file);
+    file = NULL;
+  }
 }
 
 bool
@@ -273,14 +275,14 @@ mm_file_io_c::mm_file_io_c(const string &path,
       disposition = CREATE_ALWAYS;
       break;
     default:
-      throw error_c("Unknown open mode");
+      throw mm_io_error_c("Unknown open mode");
   }
 
   file = (void *)CreateFileUtf8(path.c_str(), access_mode, share_mode, NULL,
                                 disposition, 0, NULL);
   _eof = false;
   if ((HANDLE)file == (HANDLE)0xFFFFFFFF)
-    throw error_c(mxsprintf("Error opening file %s", path.c_str()));
+    throw mm_io_open_error_c();
 
   file_name = path;
   dos_style_newlines = true;
@@ -288,8 +290,10 @@ mm_file_io_c::mm_file_io_c(const string &path,
 
 void
 mm_file_io_c::close() {
-  if (file != NULL)
+  if (NULL != file) {
     CloseHandle((HANDLE)file);
+    file = NULL;
+  }
   file_name = "";
 }
 
@@ -331,7 +335,7 @@ mm_file_io_c::setFilePointer(int64 offset,
                        method);
 
   if ((INVALID_SET_FILE_POINTER == low) && (GetLastError() != NO_ERROR))
-    throw error_c("seeking failed");
+    throw mm_io_seek_error_c();
 
   m_current_position = (int64_t)low + ((int64_t)high << 32);
 }
@@ -443,7 +447,7 @@ mm_io_c::getline() {
   string s;
 
   if (eof())
-    throw error_c("end-of-file");
+    throw mm_io_eof_error_c();
 
   while (read(&c, 1) == 1) {
     if (c == '\r')
@@ -478,18 +482,23 @@ mm_io_c::setFilePointer2(int64 offset, seek_mode mode) {
 }
 
 size_t
-mm_io_c::puts_unl(const string &s) {
+mm_io_c::puts(const string &s) {
   int i;
-  size_t bytes_written;
   const char *cs;
+  string output, h;
 
   cs = s.c_str();
-  bytes_written = 0;
   for (i = 0; cs[i] != 0; i++)
-    if (cs[i] != '\r')
-      bytes_written += write(&cs[i], 1);
+    if (cs[i] != '\r') {
+#if defined(SYS_WINDOWS)
+      if ('\n' == cs[i])
+        output += "\r";
+#endif
+      output += cs[i];
+    } else if ('\n' != cs[i + 1])
+      output += "\r";
 
-  return bytes_written;
+  return write(output.c_str(), output.length());
 }
 
 uint32_t
@@ -965,37 +974,38 @@ mm_mem_io_c::eof() {
 
 mm_text_io_c::mm_text_io_c(mm_io_c *_in,
                            bool _delete_in):
-  mm_proxy_io_c(_in, _delete_in) {
+  mm_proxy_io_c(_in, _delete_in),
+  byte_order(BO_NONE),
+  bom_len(0) {
   unsigned char buffer[4];
+  int num_read;
 
   _in->setFilePointer(0, seek_beginning);
-  if (_in->read(buffer, 4) != 4) {
-    _in->close();
-    if (_delete_in)
-      delete _in;
-    throw error_c("end-of-file reading the BOM");
+
+  num_read = _in->read(buffer, 4);
+  if (2 > num_read) {
+    _in->setFilePointer(0, seek_beginning);
+    return;
   }
 
-  if ((buffer[0] == 0xef) && (buffer[1] == 0xbb) && (buffer[2] == 0xbf)) {
+  if ((3 <= num_read) &&
+      (buffer[0] == 0xef) && (buffer[1] == 0xbb) && (buffer[2] == 0xbf)) {
     byte_order = BO_UTF8;
     bom_len = 3;
-  } else if ((buffer[0] == 0xff) && (buffer[1] == 0xfe) &&
+  } else if ((4 <= num_read) && (buffer[0] == 0xff) && (buffer[1] == 0xfe) &&
              (buffer[2] == 0x00) && (buffer[3] == 0x00)) {
     byte_order = BO_UTF32_LE;
     bom_len = 4;
-  } else if ((buffer[0] == 0x00) && (buffer[1] == 0x00) &&
+  } else if ((4 <= num_read) && (buffer[0] == 0x00) && (buffer[1] == 0x00) &&
              (buffer[2] == 0xfe) && (buffer[3] == 0xff)) {
     byte_order = BO_UTF32_BE;
     bom_len = 4;
-  } else if ((buffer[0] == 0xff) && (buffer[1] == 0xfe)) {
+  } else if ((2 <= num_read) && (buffer[0] == 0xff) && (buffer[1] == 0xfe)) {
     byte_order = BO_UTF16_LE;
     bom_len = 2;
-  } else if ((buffer[0] == 0xfe) && (buffer[1] == 0xff)) {
+  } else if ((2 <= num_read) && (buffer[0] == 0xfe) && (buffer[1] == 0xff)) {
     byte_order = BO_UTF16_BE;
     bom_len = 2;
-  } else {
-    byte_order = BO_NONE;
-    bom_len = 0;
   }
 
   _in->setFilePointer(bom_len, seek_beginning);
@@ -1150,9 +1160,31 @@ mm_stdio_c::read(void *buffer,
 size_t
 mm_stdio_c::write(const void *buffer,
                   size_t size) {
+#if defined(SYS_WINDOWS)
+  int i, bytes_written;
+  const char *s;
+
+  bytes_written = 0;
+  s = (const char *)buffer;
+  for (i = 0; i < size; ++i)
+    if (('\r' != s[i]) || ((i + 1) == size) || ('\n' != s[i + 1]))
+      bytes_written += fwrite(&s[i], 1, 1, stdout);
+
+  fflush(stdout);
+
+  return bytes_written;
+
+#else  // defined(SYS_WINDOWS)
+
   return fwrite(buffer, 1, size, stdout);
+#endif // defined(SYS_WINDOWS)
 }
 
 void
 mm_stdio_c::close() {
 }
+
+void
+mm_stdio_c::flush() {
+  fflush(stdout);
+}
