@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mm_io.h 2886 2005-04-10 09:07:45Z mosu $
+   $Id: mm_io.h 3004 2005-08-03 10:26:08Z mosu $
 
    IO callback class definitions
 
@@ -23,9 +23,31 @@
 
 #include <ebml/IOCallback.h>
 
+#include "error.h"
+
 using namespace std;
 using namespace libebml;
 
+class MTX_DLL_API mm_io_error_c: public error_c {
+public:
+  mm_io_error_c(const string &_error): error_c(_error) { }
+};
+
+class MTX_DLL_API mm_io_eof_error_c: public mm_io_error_c {
+public:
+  mm_io_eof_error_c(): mm_io_error_c("end-of-file") { }
+};
+
+class MTX_DLL_API mm_io_seek_error_c: public mm_io_error_c {
+public:
+  mm_io_seek_error_c(): mm_io_error_c("seeking failed") { }
+};
+
+class MTX_DLL_API mm_io_open_error_c: public mm_io_error_c {
+public:
+  mm_io_open_error_c(): mm_io_error_c("opening the file failed") { }
+};
+
 class MTX_DLL_API mm_io_c: public IOCallback {
 protected:
   bool dos_style_newlines;
@@ -64,12 +86,14 @@ public:
   virtual void skip(int64 numbytes);
   virtual size_t write(const void *buffer, size_t size) = 0;
   virtual bool eof() = 0;
+  virtual void flush() {
+  }
 
   virtual string get_file_name() const = 0;
 
   virtual string getline();
   virtual bool getline2(string &s);
-  virtual size_t puts_unl(const string &s);
+  virtual size_t puts(const string &s);
   virtual bool write_bom(const string &charset);
   virtual int getch();
 
@@ -213,6 +237,9 @@ public:
   virtual string getline();
   virtual int read_next_char(char *buffer);
   virtual byte_order_e get_byte_order();
+  virtual void set_byte_order(byte_order_e new_byte_order) {
+    byte_order = new_byte_order;
+  }
 };
 
 class MTX_DLL_API mm_stdio_c: public mm_io_c {
@@ -230,6 +257,7 @@ public:
   virtual string get_file_name() const {
     return "";
   }
+  virtual void flush();
 };
 
 #endif // __MM_IO_H
