@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xml_element_parser.cpp 2859 2005-03-19 11:03:12Z mosu $
+   $Id: xml_element_parser.cpp 2989 2005-07-19 18:19:28Z mosu $
 
    XML element parser
 
@@ -116,12 +116,14 @@ el_get_time(parser_data_t *pdata,
   const char *errmsg = "Expected a time in the following format: HH:MM:SS.nnn"
     " (HH = hour, MM = minute, SS = second, nnn = millisecond up to "
     "nanosecond. You may use up to nine digits for 'n' which would mean "
-    "nanosecond precision). Found '%s' instead. Additional error message: %s";
+    "nanosecond precision). You may omit the hour as well. "
+    "Found '%s' instead. Additional error message: %s";
   int64_t usec;
 
   strip(*pdata->bin);
-  if (!parse_timecode(pdata->bin->c_str(), &usec))
-    xmlp_error(pdata, errmsg, pdata->bin->c_str(), timecode_parser_error);
+  if (!parse_timecode(*pdata->bin, usec))
+    xmlp_error(pdata, errmsg, pdata->bin->c_str(),
+               timecode_parser_error.c_str());
 
   *(static_cast<EbmlUInteger *>(el)) = usec;
 }
@@ -532,7 +534,7 @@ parse_xml_elements(const char *parser_name,
     }
 
   } catch (error_c e) {
-    error = (const char *)e;
+    error = e.get_error();
   }
 
   root_element = pdata->root_element;
@@ -550,3 +552,185 @@ parse_xml_elements(const char *parser_name,
 
   return root_element;
 }
+
+// -------------------------------------------------------------------
+
+static void
+xml_parser_start_element_cb(void *user_data,
+                            const char *name,
+                            const char **atts) {
+  xml_parser_c *parser = ((xml_parser_c *)user_data);
+
+  try {
+    parser->start_element_cb(name, atts);
+  } catch (xml_parser_error_c &e) {
+    parser->throw_error(e);
+  }
+}
+
+static void
+xml_parser_end_element_cb(void *user_data,
+                          const char *name) {
+  xml_parser_c *parser = ((xml_parser_c *)user_data);
+
+  try {
+    parser->end_element_cb(name);
+  } catch (xml_parser_error_c &e) {
+    parser->throw_error(e);
+  }
+}
+
+static void
+xml_parser_add_data_cb(void *user_data,
+                       const XML_Char *s,
+                       int len) {
+  xml_parser_c *parser = ((xml_parser_c *)user_data);
+
+  try {
+    parser->add_data_cb(s, len);
+  } catch (xml_parser_error_c &e) {
+    parser->throw_error(e);
+  }
+}
+
+xml_parser_c::xml_parser_c(mm_text_io_c *xml_source):
+  m_xml_parser_state(XMLP_STATE_INITIAL),
+  m_xml_source(xml_source),
+  m_xml_parser(NULL) {
+}
+
+xml_parser_c::xml_parser_c():
+  m_xml_parser_state(XMLP_STATE_INITIAL),
+  m_xml_parser(NULL) {
+}
+
+void
+xml_parser_c::setup_xml_parser() {
+  if (NULL != m_xml_parser)
+    XML_ParserFree(m_xml_parser);
+
+  m_xml_parser = XML_ParserCreate(NULL);
+  XML_SetUserData(m_xml_parser, this);
+  XML_SetElementHandler(m_xml_parser, xml_parser_start_element_cb,
+                        xml_parser_end_element_cb);
+  XML_SetCharacterDataHandler(m_xml_parser, xml_parser_add_data_cb);
+}
+
+xml_parser_c::~xml_parser_c() {
+  if (NULL != m_xml_parser)
+    XML_ParserFree(m_xml_parser);
+}
+
+void
+xml_parser_c::parse_xml_file() {
+  m_xml_source->setFilePointer(0);
+
+  while (parse_one_xml_line())
+    ;
+}
+
+bool
+xml_parser_c::parse_one_xml_line() {
+  string line;
+
+  if (NULL == m_xml_parser)
+    setup_xml_parser();
+
+  if (setjmp(m_parser_error_jmp_buf) == 1)
+    throw m_saved_parser_error;
+
+  if (!m_xml_source->getline2(line))
+    return false;
+
+  handle_xml_encoding(line);
+
+  line += "\n";
+  if (XML_Parse(m_xml_parser, line.c_str(), line.length(), false) == 0) {
+    string error;
+    XML_Error xerror;
+
+    xerror = XML_GetErrorCode(m_xml_parser);
+    error = XML_ErrorString(xerror);
+    if (xerror == XML_ERROR_INVALID_TOKEN)
+      error += "Remember that special characters like &, <, > and \" "
+        "must be escaped in the usual HTML way: &amp; for '&', "
+        "&lt; for '<', &gt; for '>' and &quot; for '\"'.";
+    throw xml_parser_error_c(error, m_xml_parser);
+  }
+
+  return true;
+}
+
+void
+xml_parser_c::throw_error(const xml_parser_error_c &error) {
+  m_saved_parser_error = error;
+  longjmp(m_parser_error_jmp_buf, 1);
+}
+
+void
+xml_parser_c::handle_xml_encoding(string &line) {
+  int pos;
+  string new_line;
+
+  if ((XMLP_STATE_AFTER_HEADER == m_xml_parser_state) ||
+      (BO_NONE == m_xml_source->get_byte_order()))
+    return;
+
+  pos = 0;
+
+  if (XMLP_STATE_INITIAL == m_xml_parser_state) {
+    pos = line.find("<?xml");
+    if (0 > pos)
+      return;
+    m_xml_parser_state = XMLP_STATE_ATTRIBUTE_NAME;
+    pos += 5;
+    new_line = line.substr(0, pos);
+  }
+
+  while ((line.length() > pos) &&
+         (XMLP_STATE_AFTER_HEADER != m_xml_parser_state)) {
+    char cur_char = line[pos];
+    ++pos;
+
+    if (XMLP_STATE_ATTRIBUTE_NAME == m_xml_parser_state) {
+      if (('?' == cur_char) && (line.length() > pos) &&
+          ('>' == line[pos])) {
+        new_line += "?>" + line.substr(pos + 1, line.length() - pos - 1);
+        m_xml_parser_state = XMLP_STATE_AFTER_HEADER;
+
+      } else if ('"' == cur_char)
+        m_xml_parser_state = XMLP_STATE_ATTRIBUTE_VALUE;
+
+      else if ((' ' != cur_char) && ('=' != cur_char))
+        m_xml_attribute_name += cur_char;
+
+    } else {
+      // XMLP_STATE_ATTRIBUTE_VALUE
+      if ('"' == cur_char) {
+        m_xml_parser_state = XMLP_STATE_ATTRIBUTE_NAME;
+        strip(m_xml_attribute_name);
+        strip(m_xml_attribute_value);
+        if (m_xml_attribute_name == "encoding") {
+          m_xml_attribute_value = downcase(m_xml_attribute_value);
+          if ((m_xml_source->get_byte_order() == BO_NONE) &&
+              ((m_xml_attribute_value == "utf-8") ||
+               (m_xml_attribute_value == "utf8")))
+            m_xml_source->set_byte_order(BO_UTF8);
+
+          else if (starts_with_case(m_xml_attribute_value, "utf"))
+            m_xml_attribute_value = "UTF-8";
+        }
+
+        new_line += " " + m_xml_attribute_name + "=\"" +
+          m_xml_attribute_value + "\"";
+        m_xml_attribute_name = "";
+        m_xml_attribute_value = "";
+
+      } else
+        m_xml_attribute_value += cur_char;
+    }
+  }
+
+  line = new_line;
+}
+
