@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mkvextract.cpp 2883 2005-04-07 17:31:52Z mosu $
+   $Id: mkvextract.cpp 3012 2005-08-03 17:21:31Z mosu $
 
    command line parsing, setup
 
@@ -70,22 +70,24 @@ using namespace std;
 
 #define NAME "mkvextract"
 
-#define MODE_TRACKS      0
-#define MODE_TAGS        1
-#define MODE_ATTACHMENTS 2
-#define MODE_CHAPTERS    3
-#define MODE_CUESHEET    4
+#define MODE_TRACKS       0
+#define MODE_TAGS         1
+#define MODE_ATTACHMENTS  2
+#define MODE_CHAPTERS     3
+#define MODE_CUESHEET     4
+#define MODE_TIMECODES_V2 5
 
 bool no_variable_data = false;
 
 void
-usage() {
-  const char *usage_infos = N_(
+set_usage() {
+  usage_text = N_(
 "Usage: mkvextract tracks <inname> [options] [TID1:out1 [TID2:out2 ...]]\n"
 "   or  mkvextract tags <inname> [options]\n"
 "   or  mkvextract attachments <inname> [options] [AID1:out1 [AID2:out2 ...]]"
 "\n   or  mkvextract chapters <inname> [options]\n"
 "   or  mkvextract cuesheet <inname> [options]\n"
+"   or  mkvextract timecodes_v2 <inname> [TID1:out1 [TID2:out2 ...]]\n"
 "   or  mkvextract <-h|-V>\n"
 "\n"
 " The first word tells mkvextract what to extract. The second must be the\n"
@@ -102,7 +104,7 @@ usage() {
 "                   (default: keep all levels)\n"
 "  --raw            Extract the data to a raw file.\n"
 "  --fullraw        Extract the data to a raw file including the CodecPrivate"
-"                   as a header.\n"
+"\n                   as a header.\n"
 "  TID:out          Write track with the ID TID to the file 'out'.\n"
 "\n"
 " Example:\n"
@@ -137,13 +139,19 @@ usage() {
 " Example:\n"
 " mkvextract cuesheet \"audiofile.mka\" > audiofile.cue\n"
 "\n"
+" The sixth mode finds the timecodes of all blocks for a track and outpus\n"
+" a timecode v2 file with these timecodes.\n"
+"\n"
+" Example:\n"
+" mkvextract timecodes_v2 \"a movie.mkv\" 1:timecodes_track1.txt\n"
+"\n"
 " These options can be used instead of the mode keyword to obtain\n"
 " further information:\n"
 "  -v, --verbose  Increase verbosity.\n"
 "  -h, --help     Show this help.\n"
 "  -V, --version  Show version information.\n");
 
-  mxinfo(_(usage_infos));
+  version_info = "mkvextract v" VERSION " ('" VERSIONNAME "')";
 }
 
 static bool chapter_format_simple = false;
@@ -167,31 +175,23 @@ parse_args(vector<string> args,
   sub_charset = NULL;
   verbose = 0;
 
-  if (args.size() < 1) {
+  handle_common_cli_args(args, "-o");
+
+  if (args.size() < 1)
     usage();
-    mxexit(0);
-  }
 
-  if ((args[0] == "-V") || (args[0] == "--version")) {
-    mxinfo("mkvextract v" VERSION " ('" VERSIONNAME "')\n");
-    mxexit(0);
-
-  } else if ((args[0] == "-h") || (args[0] == "-?") ||
-             (args[0] == "--help")) {
-    usage();
-    mxexit(0);
-  }
-
-  if ((args[0] == "tracks"))
+  if (args[0] == "tracks")
     mode = MODE_TRACKS;
-  else if ((args[0] == "tags"))
+  else if (args[0] == "tags")
     mode = MODE_TAGS;
-  else if ((args[0] == "attachments"))
+  else if (args[0] == "attachments")
     mode = MODE_ATTACHMENTS;
-  else if ((args[0] == "chapters"))
+  else if (args[0] == "chapters")
     mode = MODE_CHAPTERS;
-  else if ((args[0] == "cuesheet"))
+  else if (args[0] == "cuesheet")
     mode = MODE_CUESHEET;
+  else if (args[0] == "timecodes_v2")
+    mode = MODE_TIMECODES_V2;
   else
     mxerror(_("Unknown mode '%s'.\n"), args[0].c_str());
 
@@ -209,10 +209,7 @@ parse_args(vector<string> args,
 
   // Now process all the other options.
   for (i = 2; i < args.size(); i++)
-    if ((args[i] == "-v") || (args[i] == "--verbose"))
-      verbose++;
-
-    else if ((args[i] == "--no-variable-data"))
+    if ((args[i] == "--no-variable-data"))
       no_variable_data = true;
 
     else if ((args[i] == "-f") || (args[i] == "--parse-fully"))
@@ -276,11 +273,12 @@ parse_args(vector<string> args,
 
       chapter_format_simple = true;
 
-    } else if ((mode == MODE_TRACKS) || (mode == MODE_ATTACHMENTS)) {
+    } else if ((mode == MODE_TRACKS) || (mode == MODE_ATTACHMENTS) ||
+               (MODE_TIMECODES_V2 == mode)) {
       copy = safestrdup(args[i].c_str());
       colon = strchr(copy, ':');
       if (colon == NULL)
-        mxerror(mode == MODE_TRACKS ?
+        mxerror((MODE_TRACKS == mode) || (MODE_TIMECODES_V2 == mode) ?
                 _("Missing track ID in argument '%s'.\n") :
                 _("Missing attachment ID in argument '%s'.\n"),
                 args[i].c_str());
@@ -288,7 +286,7 @@ parse_args(vector<string> args,
       *colon = 0;
       if (!parse_int(copy, tid) || (tid < 0))
         mxerror("Invalid %s ID in argument '%s'.\n",
-                mode == MODE_TRACKS ?
+                (MODE_TRACKS == mode) || (MODE_TIMECODES_V2 == mode) ?
                 _("Invalid track ID in argument '%s'.\n") :
                 _("Invalid attachment ID in argument '%s'.\n"),
                 args[i].c_str());
@@ -325,7 +323,6 @@ parse_args(vector<string> args,
   if (tracks.size() == 0) {
     mxinfo(_("Nothing to do.\n\n"));
     usage();
-    mxexit(0);
   }
 }
 
@@ -384,6 +381,9 @@ main(int argc,
   int mode;
   vector<track_spec_t> tracks;
 
+  init_stdio();
+  set_usage();
+
 #if defined(SYS_UNIX)
   nice(2);
 #endif
@@ -421,6 +421,9 @@ main(int argc,
   else if (mode == MODE_CUESHEET)
     extract_cuesheet(input_file.c_str(), parse_fully);
 
+  else if (mode == MODE_TIMECODES_V2)
+    extract_timecodes(input_file, tracks, 2);
+
   else
     die("mkvextract: Unknown mode!?");
 
