@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_base.cpp 2883 2005-04-07 17:31:52Z mosu $
+   $Id: xtr_base.cpp 3000 2005-08-02 08:20:01Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -24,6 +24,7 @@
 #include "matroska.h"
 
 #include "xtr_aac.h"
+#include "xtr_avc.h"
 #include "xtr_avi.h"
 #include "xtr_base.h"
 #include "xtr_ogg.h"
@@ -83,6 +84,10 @@ xtr_base_c::handle_block(KaxBlock &block,
   }
 }
 
+void
+xtr_base_c::finish_track() {
+}
+
 void
 xtr_base_c::finish_file() {
 }
@@ -126,6 +131,8 @@ xtr_base_c::create_extractor(const string &new_codec_id,
   // Video formats
   else if (new_codec_id == MKV_V_MSCOMP)
     return new xtr_avi_c(new_codec_id, new_tid, tspec);
+  else if (new_codec_id == MKV_V_MPEG4_AVC)
+    return new xtr_avc_c(new_codec_id, new_tid, tspec);
   else if (starts_with_case(new_codec_id, "V_REAL/"))
     return new xtr_rmff_c(new_codec_id, new_tid, tspec);
 
@@ -140,6 +147,8 @@ xtr_base_c::create_extractor(const string &new_codec_id,
     return new xtr_ssa_c(new_codec_id, new_tid, tspec);
   else if (new_codec_id == MKV_S_VOBSUB)
     return new xtr_vobsub_c(new_codec_id, new_tid, tspec);
+  else if (new_codec_id == MKV_S_TEXTUSF)
+    return new xtr_usf_c(new_codec_id, new_tid, tspec);
 
   return NULL;
 }
