@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_ogg.cpp 2859 2005-03-19 11:03:12Z mosu $
+   $Id: xtr_ogg.cpp 2903 2005-05-02 13:50:23Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -14,6 +14,7 @@
 
 #include "common.h"
 #include "commonebml.h"
+#include "random.h"
 
 #include "xtr_ogg.h"
 
@@ -61,7 +62,7 @@ xtr_oggbase_c::create_file(xtr_base_c *_master,
   if (no_variable_data)
     ogg_stream_init(&os, 1804289383);
   else
-    ogg_stream_init(&os, rand());
+    ogg_stream_init(&os, random_c::generate_31bits());
 }
 
 void
@@ -172,7 +173,7 @@ xtr_oggflac_c::create_file(xtr_base_c *_master,
   if (no_variable_data)
     ogg_stream_init(&os, 1804289383);
   else
-    ogg_stream_init(&os, rand());
+    ogg_stream_init(&os, random_c::generate_31bits());
 
   // Handle the three header packets: Headers, comments, codec
   // setup data.
