@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_rmff.cpp 2765 2005-02-05 17:36:14Z mosu $
+   $Id: xtr_rmff.cpp 2859 2005-03-19 11:03:12Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -49,7 +49,8 @@ xtr_rmff_c::create_file(xtr_base_c *_master,
     mxerror("Memory allocation error: %d (%s).\n",
             rmff_last_error, rmff_last_error_msg);
 
-  rmff_set_type_specific_data(rmtrack, (const unsigned char *)&binary(*priv),
+  rmff_set_type_specific_data(rmtrack,
+                              (const unsigned char *)priv->GetBuffer(),
                               priv->GetSize());
 
   if ('V' == codec_id[0])
