@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_textsubs.cpp 2859 2005-03-19 11:03:12Z mosu $
+   $Id: xtr_textsubs.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -76,7 +76,7 @@ xtr_srt_c::handle_block(KaxBlock &block,
                      end / 1000 / 60 / 60, (end / 1000 / 60) % 60,
                      (end / 1000) % 60, end % 1000,
                      from_utf8(conv, text).c_str());
-  out->puts_unl(buffer);
+  out->puts(buffer);
   delete []text;
 }
 
@@ -164,12 +164,12 @@ xtr_ssa_c::create_file(xtr_base_c *_master,
   if (pos2 < 0)
     pos2 = sconv.length();
   ssa_format = split(sconv.substr(pos1 + 7, pos2 - pos1 - 7), ",");
-  strip(ssa_format);
+  strip(ssa_format, true);
   for (pos1 = 0; pos1 < ssa_format.size(); pos1++)
     ssa_format[pos1] = downcase(ssa_format[pos1]);
 
   sconv = from_utf8(conv, sconv);
-  out->puts_unl(sconv);
+  out->puts(sconv);
 }
 
 void
@@ -207,8 +207,9 @@ xtr_ssa_c::handle_block(KaxBlock &block,
   fields = split(s, ",", 9);
   if (9 < fields.size()) {
     mxwarn("Invalid format for a SSA line ('%s') at timecode " FMT_TIMECODE
-           ": Too many fields found (%d instead of 9). This entry will be "
-           "skipped.\n", s, ARG_TIMECODE_NS(timecode), fields.size());
+           ": Too many fields found (%u instead of 9). This entry will be "
+           "skipped.\n", s, ARG_TIMECODE_NS(timecode),
+           (unsigned int)fields.size());
     return;
   }
   while (9 != fields.size())
@@ -275,6 +276,158 @@ xtr_ssa_c::finish_file() {
   // write them.
   sort(lines.begin(), lines.end());
   for (i = 0; i < lines.size(); i++)
-    out->puts_unl(lines[i].line.c_str());
+    out->puts(lines[i].line.c_str());
 }
 
+// ------------------------------------------------------------------------
+
+xtr_usf_c::xtr_usf_c(const string &_codec_id,
+                     int64_t _tid,
+                     track_spec_t &tspec):
+  xtr_base_c(_codec_id, _tid, tspec),
+  m_sub_charset(tspec.sub_charset) {
+
+  if (m_sub_charset == "")
+    m_sub_charset = "UTF-8";
+}
+
+void
+xtr_usf_c::create_file(xtr_base_c *_master,
+                       KaxTrackEntry &track) {
+  KaxCodecPrivate *priv;
+  KaxTrackLanguage *language;
+  unsigned char *new_priv;
+  uint32_t private_size;
+
+  priv = FINDFIRST(&track, KaxCodecPrivate);
+  if (NULL == priv)
+    mxerror("Track %lld with the CodecID '%s' is missing the \"codec private"
+            "\" element and cannot be extracted.\n", tid, codec_id.c_str());
+
+  if (!content_decoder.initialize(track))
+    mxerror("Tracks with unsupported content encoding schemes (compression "
+            "or encryption) cannot be extracted.\n");
+
+  private_size = priv->GetSize();
+
+  new_priv = const_cast<unsigned char *>(priv->GetBuffer());
+  if (!content_decoder.reverse(new_priv, private_size,
+                               CONTENT_ENCODING_SCOPE_CODECPRIVATE)) {
+    m_codec_private.append((const char *)new_priv, private_size);
+  } else {
+    m_codec_private.append((const char *)new_priv, private_size);
+    safefree(new_priv);
+  }
+
+  language = FINDFIRST(&track, KaxTrackLanguage);
+  if (NULL == language)
+    m_language = "eng";
+  else
+    m_language = string(*language);
+
+  if (NULL != _master) {
+    xtr_usf_c *usf_master;
+
+    usf_master = dynamic_cast<xtr_usf_c *>(_master);
+    if (NULL == usf_master)
+      mxerror("Cannot write track %lld with the CodecID '%s' to the file '%s' "
+              "because track %lld with the CodecID '%s' is already being "
+              "written to the same file.\n", tid, codec_id.c_str(),
+              file_name.c_str(), _master->tid, _master->codec_id.c_str());
+    if (m_codec_private != usf_master->m_codec_private)
+      mxerror("Cannot write track %lld with the CodecID '%s' to the file '%s' "
+              "because track %lld with the CodecID '%s' is already being "
+              "written to the same file, and their CodecPrivate data (the "
+              "USF styles etc) do not match.\n", tid, codec_id.c_str(),
+              file_name.c_str(), _master->tid, _master->codec_id.c_str());
+
+    m_formatter = usf_master->m_formatter;
+    master = usf_master;
+
+  } else {
+    try {
+      int end_tag_pos;
+      string codec_private_mod, end_tag = "</USFSubtitles>";
+
+      codec_private_mod = m_codec_private;
+      end_tag_pos = codec_private_mod.find(end_tag);
+      if (0 <= end_tag_pos)
+        codec_private_mod.erase(end_tag_pos, end_tag.length());
+
+      out = new mm_file_io_c(file_name, MODE_CREATE);
+
+      m_formatter =
+        counted_ptr<xml_formatter_c>(new xml_formatter_c(out, m_sub_charset));
+      m_formatter->set_doctype("USFSubtitles", "USFV100.dtd");
+      m_formatter->set_stylesheet("text/xsl", "USFV100.xsl");
+      m_formatter->write_header();
+      m_formatter->format(codec_private_mod + "\n");
+    } catch (mm_io_error_c &error) {
+      mxerror("Failed to create the file '%s': %d (%s)\n", file_name.c_str(),
+              errno, strerror(errno));
+    } catch (xml_formatter_error_c &error) {
+      mxerror("Failed to parse the USF codec private data for track %lld: "
+              "%s\n", tid, error.get_error().c_str());
+      
+    }
+  }
+}
+
+void
+xtr_usf_c::handle_block(KaxBlock &block,
+                        KaxBlockAdditions *additions,
+                        int64_t timecode,
+                        int64_t duration,
+                        int64_t bref,
+                        int64_t fref) {
+  if (0 == block.NumberFrames())
+    return;
+
+  DataBuffer &data_buffer = block.GetBuffer(0);
+  usf_entry_t entry("", timecode, timecode + duration);
+  uint32_t size = data_buffer.Size();
+  unsigned char *data = data_buffer.Buffer();
+  bool free_mem = content_decoder.reverse(data, size,
+                                          CONTENT_ENCODING_SCOPE_BLOCK);
+  entry.m_text.append((const char *)data, size);
+  if (free_mem)
+    safefree(data);
+  m_entries.push_back(entry);
+}
+
+void
+xtr_usf_c::finish_track() {
+  vector<usf_entry_t>::const_iterator entry;
+
+  try {
+    m_formatter->format(mxsprintf("<subtitles>\n<language code=\"%s\"/>\n",
+                                  m_language.c_str()));
+    foreach(entry, m_entries) {
+      string text = entry->m_text;
+      strip(text, true);
+      m_formatter->format(mxsprintf("<subtitle start=\"" FMT_TIMECODE
+                                    "\" stop=\"" FMT_TIMECODE "\">",
+                                    ARG_TIMECODE_NS(entry->m_start),
+                                    ARG_TIMECODE_NS(entry->m_end)));
+      m_formatter->format_fixed(text);
+      m_formatter->format("</subtitle>\n");
+    }
+    m_formatter->format("</subtitles>\n");
+  } catch (xml_formatter_error_c &error) {
+    mxerror("Failed to parse an USF subtitle entry for track %lld: %s\n", tid,
+            error.get_error().c_str());
+  }
+}
+
+void
+xtr_usf_c::finish_file() {
+  try {
+    if (NULL == master) {
+      m_formatter->format("</USFSubtitles>");
+      out->printf("\n");
+    }
+  } catch (xml_formatter_error_c &error) {
+    mxerror("Failed to parse the USF end tag for track %lld: %s\n", tid,
+            error.get_error().c_str());
+  }
+}
