@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_wav.cpp 2791 2005-02-11 23:29:39Z mosu $
+   $Id: xtr_wav.cpp 2882 2005-04-07 17:15:31Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -86,7 +86,7 @@ xtr_wavpack4_c::create_file(xtr_base_c *_master,
   if ((NULL == priv) || (2 > priv->GetSize()))
     mxerror("Track %lld with the CodecID '%s' is missing the \"codec private"
             "\" element and cannot be extracted.\n", tid, codec_id.c_str());
-  memcpy(version, &binary(*priv), 2);
+  memcpy(version, priv->GetBuffer(), 2);
 
   channels = kt_get_a_channels(track);
 
@@ -211,5 +211,12 @@ xtr_wavpack4_c::handle_block(KaxBlock &block,
 
 void
 xtr_wavpack4_c::finish_file() {
-  delete corr_out;
+  out->setFilePointer(12);
+  out->write_uint32_le(number_of_samples);
+
+  if (NULL != corr_out) {
+    corr_out->setFilePointer(12);
+    corr_out->write_uint32_le(number_of_samples);
+    delete corr_out;
+  }
 }
