@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ac3.cpp 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_ac3.cpp 2932 2005-05-25 21:31:26Z mosu $
 
    AC3 demultiplexer module
 
@@ -86,8 +86,7 @@ ac3_reader_c::read(generic_packetizer_c *,
     return FILE_STATUS_DONE;
   }
 
-  memory_c mem(chunk, nread, false);
-  PTZR0->process(mem);
+  PTZR0->process(new packet_t(new memory_c(chunk, nread, false)));
   bytes_processed += nread;
 
   return FILE_STATUS_MOREDATA;
