@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_avi.cpp 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_avi.cpp 2967 2005-06-26 10:04:02Z mosu $
 
    AVI demultiplexer module
 
@@ -159,11 +159,7 @@ avi_reader_c::create_packetizer(int64_t tid) {
         !strcasecmp(codec, "MPG3") ||
         !strcasecmp(codec, "MP43"))
       is_divx = RAVI_DIVX3;
-    else if (!strcasecmp(codec, "MP42") ||
-             !strcasecmp(codec, "DIV2") ||
-             !strcasecmp(codec, "DIVX") ||
-             !strcasecmp(codec, "XVID") ||
-             !strcasecmp(codec, "DX50"))
+    else if (is_mpeg4_p2_fourcc(codec))
       is_divx = RAVI_MPEG4;
     else
       is_divx = 0;
@@ -181,8 +177,8 @@ avi_reader_c::create_packetizer(int64_t tid) {
                                                        false,
                                                        ti));
       if (verbose)
-        mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module for "
-               "this track.\n", ti.fname.c_str(), (int64_t)0);
+        mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module.\n",
+               ti.fname.c_str(), (int64_t)0);
     } else {
       vptzr = add_packetizer(new video_packetizer_c(this, NULL,
                                                     AVI_frame_rate(avi),
@@ -190,7 +186,7 @@ avi_reader_c::create_packetizer(int64_t tid) {
                                                     AVI_video_height(avi),
                                                     ti));
       if (verbose)
-        mxinfo(FMT_TID "Using the video output module for the video track.\n",
+        mxinfo(FMT_TID "Using the video output module.\n",
                ti.fname.c_str(), (int64_t)0);
     }
   }
@@ -432,9 +428,12 @@ avi_reader_c::read(generic_packetizer_c *ptzr,
         nread = AVI_read_frame(avi, (char *)chunk, &key);
         debug_leave("AVI_read_frame");
         if (nread < 0) {
-          memory_c mem(old_chunk, old_nread, true);
-          PTZR(vptzr)->process(mem, -1, -1,
-                               old_key ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC);
+          PTZR(vptzr)->process(new packet_t(new memory_c(old_chunk, old_nread,
+                                                         true),
+                                            -1, -1,
+                                            old_key ? VFT_IFRAME :
+                                            VFT_PFRAMEAUTOMATIC,
+                                            VFT_NOBFRAME));
           bytes_processed += old_nread;
           old_chunk = NULL;
           mxwarn(PFX "Reading frame number %d resulted in an error. "
@@ -456,9 +455,12 @@ avi_reader_c::read(generic_packetizer_c *ptzr,
       }
       duration = (int64_t)(1000000000.0 * frames_read / fps);
       if (nread > 0) {
-        memory_c mem(old_chunk, old_nread, true);
-        PTZR(vptzr)->process(mem, -1, duration,
-                             old_key ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC);
+        PTZR(vptzr)->process(new packet_t(new memory_c(old_chunk, old_nread,
+                                                       true),
+                                          -1, duration,
+                                          old_key ? VFT_IFRAME :
+                                          VFT_PFRAMEAUTOMATIC,
+                                          VFT_NOBFRAME));
         bytes_processed += old_nread;
         old_chunk = NULL;
         if (! last_frame) {
@@ -469,9 +471,11 @@ avi_reader_c::read(generic_packetizer_c *ptzr,
           old_key = key;
           old_nread = nread;
         } else if (nread > 0) {
-          memory_c mem(chunk, nread, false);
-          PTZR(vptzr)->process(mem, -1, duration,
-                               key ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC);
+          PTZR(vptzr)->process(new packet_t(new memory_c(chunk, nread, false),
+                                            -1, duration,
+                                            key ? VFT_IFRAME :
+                                            VFT_PFRAMEAUTOMATIC,
+                                            VFT_NOBFRAME));
           bytes_processed += nread;
         }
       }
@@ -516,9 +520,9 @@ avi_reader_c::read(generic_packetizer_c *ptzr,
         size = AVI_read_audio_chunk(avi, NULL);
         if (size > 0)
           need_more_data = true;
-        memory_c mem(audio_chunk, nread, true);
         PTZR(demuxer->ptzr)->add_avi_block_size(nread);
-        PTZR(demuxer->ptzr)->process(mem);
+        PTZR(demuxer->ptzr)->process(new packet_t(new memory_c(audio_chunk,
+                                                               nread, true)));
         bytes_processed += nread;
       } else
         safefree(audio_chunk);
