@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_flac.cpp 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_flac.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    OGG media stream reader
 
@@ -222,7 +222,7 @@ flac_reader_c::parse_file() {
   result =
     (int)FLAC__seekable_stream_decoder_process_until_end_of_metadata(decoder);
   mxverb(2, FPFX "extract->metadata, result: %d, mdp: %d, num blocks: %u\n",
-         result, metadata_parsed, blocks.size());
+         result, metadata_parsed, (unsigned int)blocks.size());
 
   if (!metadata_parsed)
     mxerror(FMT_FN "No metadata block found. This file is broken.\n",
@@ -308,9 +308,9 @@ flac_reader_c::read(generic_packetizer_c *,
     PTZR0->flush();
     return FILE_STATUS_DONE;
   }
-  memory_c mem(buf, current_block->len, true);
   samples_here = flac_get_num_samples(buf, current_block->len, stream_info);
-  PTZR0->process(mem, samples * 1000000000 / sample_rate);
+  PTZR0->process(new packet_t(new memory_c(buf, current_block->len, true),
+                              samples * 1000000000 / sample_rate));
   samples += samples_here;
   current_block++;
 
