@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_matroska.cpp 2874 2005-04-07 15:00:22Z mosu $
+   $Id: r_matroska.cpp 3022 2005-08-15 20:47:56Z mosu $
 
    Matroska reader
 
@@ -89,8 +89,10 @@ using namespace libmatroska;
                                   (c) == 'b' ? "buttons" : \
                                   (c) == 'v' ? "video" : "subtitle")
 
-#define in_parent(p) (in->getFilePointer() < \
-                      (p->GetElementPosition() + p->ElementSize()))
+#define in_parent(p) \
+  (!p->IsFiniteSize() || \
+   (in->getFilePointer() < \
+    (p->GetElementPosition() + p->HeadSize() + p->GetSize())))
 
 // }}}
 
@@ -152,9 +154,6 @@ kax_reader_c::~kax_reader_c() {
     if (tracks[i] != NULL)
       delete tracks[i];
 
-  for (i = 0; i < attachments.size(); i++)
-    safefree(attachments[i].data);
-
   if (saved_l1 != NULL)
     delete saved_l1;
   if (in != NULL)
@@ -198,7 +197,7 @@ kax_reader_c::new_kax_track() {
 }
 
 kax_track_t *
-kax_reader_c::find_track_by_num(uint32_t n,
+kax_reader_c::find_track_by_num(uint64_t n,
                                 kax_track_t *c) {
   int i;
 
@@ -211,7 +210,7 @@ kax_reader_c::find_track_by_num(uint32_t n,
 }
 
 kax_track_t *
-kax_reader_c::find_track_by_uid(uint32_t uid,
+kax_reader_c::find_track_by_uid(uint64_t uid,
                                 kax_track_t *c) {
   int i;
 
@@ -264,7 +263,7 @@ kax_reader_c::verify_tracks() {
           if ((t->private_data == NULL) ||
               (t->private_size < sizeof(alBITMAPINFOHEADER))) {
             if (verbose)
-              mxwarn(PFX "CodecID for track %u is '" MKV_V_MSCOMP
+              mxwarn(PFX "CodecID for track %llu is '" MKV_V_MSCOMP
                      "', but there was no BITMAPINFOHEADER struct present. "
                      "Therefore we don't have a FourCC to identify the video "
                      "codec used.\n", t->tnum);
@@ -278,7 +277,7 @@ kax_reader_c::verify_tracks() {
             if (t->v_width != u) {
               if (verbose)
                 mxwarn(PFX "(MS compatibility "
-                       "mode, track %u) Matrosa says video width is %u, but "
+                       "mode, track %llu) Matrosa says video width is %llu, but "
                        "the BITMAPINFOHEADER says %u.\n", t->tnum, t->v_width,
                        u);
               if (t->v_width == 0)
@@ -289,8 +288,9 @@ kax_reader_c::verify_tracks() {
             if (t->v_height != u) {
               if (verbose)
                 mxwarn(PFX "(MS compatibility "
-                       "mode, track %u) Matrosa video height is %u, but the "
-                       "BITMAPINFOHEADER says %u.\n", t->tnum, t->v_height, u);
+                       "mode, track %llu) Matrosa video height is %llu, but "
+                       "the BITMAPINFOHEADER says %u.\n", t->tnum, t->v_height,
+                       u);
               if (t->v_height == 0)
                 t->v_height = u;
             }
@@ -301,12 +301,12 @@ kax_reader_c::verify_tracks() {
 
         if (t->v_width == 0) {
           if (verbose)
-            mxwarn(PFX "The width for track %u was not set.\n", t->tnum);
+            mxwarn(PFX "The width for track %llu was not set.\n", t->tnum);
           continue;
         }
         if (t->v_height == 0) {
           if (verbose)
-            mxwarn(PFX "The height for track %u was not set.\n", t->tnum);
+            mxwarn(PFX "The height for track %llu was not set.\n", t->tnum);
           continue;
         }
 
@@ -322,7 +322,7 @@ kax_reader_c::verify_tracks() {
           if ((t->private_data == NULL) ||
               (t->private_size < sizeof(alWAVEFORMATEX))) {
             if (verbose)
-              mxwarn(PFX "CodecID for track %u is '"
+              mxwarn(PFX "CodecID for track %llu is '"
                      MKV_A_ACM "', but there was no WAVEFORMATEX struct "
                      "present. Therefore we don't have a format ID to "
                      "identify the audio codec used.\n", t->tnum);
@@ -337,9 +337,9 @@ kax_reader_c::verify_tracks() {
             if (((uint32_t)t->a_sfreq) != u) {
               if (verbose)
                 mxwarn(PFX "(MS compatibility mode for "
-                       "track %u) Matroska says that there are %u samples per"
-                       " second, but WAVEFORMATEX says that there are %u.\n",
-                       t->tnum, (uint32_t)t->a_sfreq, u);
+                       "track %llu) Matroska says that there are %u samples "
+                       "per second, but WAVEFORMATEX says that there are "
+                       "%u.\n", t->tnum, (uint32_t)t->a_sfreq, u);
               if (t->a_sfreq == 0.0)
                 t->a_sfreq = (float)u;
             }
@@ -348,9 +348,9 @@ kax_reader_c::verify_tracks() {
             if (t->a_channels != u) {
               if (verbose)
                 mxwarn(PFX "(MS compatibility mode for "
-                       "track %u) Matroska says that there are %u channels, "
-                       "but the WAVEFORMATEX says that there are %u.\n",
-                       t->tnum, t->a_channels, u);
+                       "track %llu) Matroska says that there are %llu "
+                       "channels, but the WAVEFORMATEX says that there are "
+                       "%u.\n", t->tnum, t->a_channels, u);
               if (t->a_channels == 0)
                 t->a_channels = u;
             }
@@ -359,8 +359,8 @@ kax_reader_c::verify_tracks() {
             if (t->a_bps != u) {
               if (verbose && (t->a_formattag == 0x0001))
                 mxwarn(PFX "(MS compatibility mode for "
-                       "track %u) Matroska says that there are %u bits per "
-                       "sample, but the WAVEFORMATEX says that there are %u."
+                       "track %llu) Matroska says that there are %llu bits per"
+                       " sample, but the WAVEFORMATEX says that there are %u."
                        "\n", t->tnum, t->a_bps, u);
               if (t->a_bps == 0)
                 t->a_bps = u;
@@ -378,7 +378,7 @@ kax_reader_c::verify_tracks() {
           else if (t->codec_id == MKV_A_VORBIS) {
             if (t->private_data == NULL) {
               if (verbose)
-                mxwarn(PFX "CodecID for track %u is "
+                mxwarn(PFX "CodecID for track %llu is "
                        "'A_VORBIS', but there are no header packets present.",
                        t->tnum);
               continue;
@@ -471,7 +471,7 @@ kax_reader_c::verify_tracks() {
         if (t->codec_id == MKV_S_VOBSUB) {
           if (t->private_data == NULL) {
             if (verbose)
-              mxwarn(PFX "CodecID for track %u is '%s', but there was no "
+              mxwarn(PFX "CodecID for track %llu is '%s', but there was no "
                      "private data found.\n", t->tnum, t->codec_id.c_str());
             continue;
           }
@@ -482,7 +482,7 @@ kax_reader_c::verify_tracks() {
       case 'b':
         if (t->codec_id != MKV_B_VOBBTN) {
           if (verbose)
-            mxwarn(PFX "CodecID '%s' for track %u is unknown.\n",
+            mxwarn(PFX "CodecID '%s' for track %llu is unknown.\n",
                    t->codec_id.c_str(), t->tnum);
           continue;
         }
@@ -492,12 +492,12 @@ kax_reader_c::verify_tracks() {
       default:                  // unknown track type!? error in demuxer...
         if (verbose)
           mxwarn(PFX "matroska_reader: unknown "
-                 "demuxer type for track %u: '%c'\n", t->tnum, t->type);
+                 "demuxer type for track %llu: '%c'\n", t->tnum, t->type);
         continue;
     }
 
     if (t->ok && (verbose > 1))
-      mxinfo(PFX "Track %u seems to be ok.\n", t->tnum);
+      mxinfo(PFX "Track %llu seems to be ok.\n", t->tnum);
   }
 }
 
@@ -518,7 +518,6 @@ kax_reader_c::handle_attachments(mm_io_c *io,
   int64_t size, id;
   unsigned char *data;
   bool found;
-  kax_attachment_t matt;
 
   found = false;
   for (i = 0; i < handled_attachments.size(); i++)
@@ -567,7 +566,7 @@ kax_reader_c::handle_attachments(mm_io_c *io,
 
           } else if (EbmlId(*l2) == KaxFileUID::ClassInfos.GlobalId) {
             KaxFileUID &fuid = *static_cast<KaxFileUID *>(l2);
-            id = uint32(fuid);
+            id = uint64(fuid);
 
           } else if (EbmlId(*l2) == KaxFileData::ClassInfos.GlobalId) {
             KaxFileData &fdata = *static_cast<KaxFileData *>(l2);
@@ -578,23 +577,16 @@ kax_reader_c::handle_attachments(mm_io_c *io,
 
         if ((id != -1) && (size != -1) && (mime_type.length() > 0) &&
             (name.length() > 0)) {
-          found = false;
+          attachment_t matt;
 
-          for (k = 0; k < attachments.size(); k++)
-            if (attachments[k].id == id) {
-              found = true;
-              break;
-            }
-
-          if (!found) {
-            matt.name = name;
-            matt.mime_type = mime_type;
-            matt.description = description;
-            matt.size = size;
-            matt.id = id;
-            matt.data = (unsigned char *)safememdup(data, size);
-            attachments.push_back(matt);
-          }
+          matt.name = UTFstring_to_cstrutf8(name);
+          matt.mime_type = mime_type;
+          matt.description = UTFstring_to_cstrutf8(description);
+          matt.id = id;
+          matt.data = counted_ptr<buffer_t>(new buffer_t);
+          matt.data->size = size;
+          matt.data->buffer = (unsigned char *)safememdup(data, size);
+          add_attachment(matt);
         }
       }
     }
@@ -705,7 +697,7 @@ kax_reader_c::handle_tags(mm_io_c *io,
           is_global = true;
         else {
           found = false;
-          track = find_track_by_uid(uint32(*tuid));
+          track = find_track_by_uid(uint64(*tuid));
           if (track != NULL) {
             found = true;
             contains_tag = false;
@@ -944,16 +936,16 @@ kax_reader_c::read_headers() {
           track->tnum = uint8(*ktnum);
           if ((find_track_by_num(track->tnum, track) != NULL) && (verbose > 1))
             mxwarn(PFX "|  + There's more than one track with "
-                   "the number %u.\n", track->tnum);
+                   "the number %llu.\n", track->tnum);
 
           ktuid = FINDFIRST(ktentry, KaxTrackUID);
           if (ktuid == NULL)
             mxerror(PFX "A track is missing its track UID.\n");
           if (verbose > 1)
-            mxinfo(PFX "|  + Track UID: %u\n", uint32(*ktuid));
-          track->tuid = uint32(*ktuid);
+            mxinfo(PFX "|  + Track UID: %llu\n", uint64(*ktuid));
+          track->tuid = uint64(*ktuid);
           if ((find_track_by_uid(track->tuid, track) != NULL) && (verbose > 1))
-            mxwarn(PFX "|  + There's more than one track with the UID %u.\n",
+            mxwarn(PFX "|  + There's more than one track with the UID %llu.\n",
                    track->tnum);
 
           kdefdur = FINDFIRST(ktentry, KaxTrackDefaultDuration);
@@ -1022,7 +1014,7 @@ kax_reader_c::read_headers() {
             if (ka_channels != NULL) {
               track->a_channels = uint8(*ka_channels);
               if (verbose > 1)
-                mxinfo(PFX "|   + Channels: %u\n", track->a_channels);
+                mxinfo(PFX "|   + Channels: %llu\n", track->a_channels);
             } else
               track->a_channels = 1;
 
@@ -1031,7 +1023,7 @@ kax_reader_c::read_headers() {
             if (ka_bitdepth != NULL) {
               track->a_bps = uint8(*ka_bitdepth);
               if (verbose > 1)
-                mxinfo(PFX "|   + Bit depth: %u\n", track->a_bps);
+                mxinfo(PFX "|   + Bit depth: %llu\n", track->a_bps);
             }
           }
 
@@ -1049,32 +1041,32 @@ kax_reader_c::read_headers() {
 
             kv_pwidth = FINDFIRST(ktvideo, KaxVideoPixelWidth);
             if (kv_pwidth != NULL) {
-              track->v_width = uint16(*kv_pwidth);
+              track->v_width = uint64(*kv_pwidth);
               if (verbose > 1)
-                mxinfo(PFX "|   + Pixel width: %u\n", track->v_width);
+                mxinfo(PFX "|   + Pixel width: %llu\n", track->v_width);
             } else
               mxerror(PFX "Pixel width is missing.\n");
 
             kv_pheight = FINDFIRST(ktvideo, KaxVideoPixelHeight);
             if (kv_pheight != NULL) {
-              track->v_height = uint16(*kv_pheight);
+              track->v_height = uint64(*kv_pheight);
               if (verbose > 1)
-                mxinfo(PFX "|   + Pixel height: %u\n", track->v_height);
+                mxinfo(PFX "|   + Pixel height: %llu\n", track->v_height);
             } else
               mxerror(PFX "Pixel height is missing.\n");
 
             kv_dwidth = FINDFIRST(ktvideo, KaxVideoDisplayWidth);
             if (kv_dwidth != NULL) {
-              track->v_dwidth = uint16(*kv_dwidth);
+              track->v_dwidth = uint64(*kv_dwidth);
               if (verbose > 1)
-                mxinfo(PFX "|   + Display width: %u\n", track->v_dwidth);
+                mxinfo(PFX "|   + Display width: %llu\n", track->v_dwidth);
             }
 
             kv_dheight = FINDFIRST(ktvideo, KaxVideoDisplayHeight);
             if (kv_dheight != NULL) {
-              track->v_dheight = uint16(*kv_dheight);
+              track->v_dheight = uint64(*kv_dheight);
               if (verbose > 1)
-                mxinfo(PFX "|   + Display height: %u\n", track->v_dheight);
+                mxinfo(PFX "|   + Display height: %llu\n", track->v_dheight);
             }
 
             // For older files.
@@ -1087,30 +1079,31 @@ kax_reader_c::read_headers() {
 
             kv_pcleft = FINDFIRST(ktvideo, KaxVideoPixelCropLeft);
             if (kv_pcleft != NULL) {
-              track->v_pcleft = uint16(*kv_pcleft);
+              track->v_pcleft = uint64(*kv_pcleft);
               if (verbose > 1)
-                mxinfo(PFX "|   + Pixel crop left: %u\n", track->v_pcleft);
+                mxinfo(PFX "|   + Pixel crop left: %llu\n", track->v_pcleft);
             }
 
             kv_pctop = FINDFIRST(ktvideo, KaxVideoPixelCropTop);
             if (kv_pctop != NULL) {
-              track->v_pctop = uint16(*kv_pctop);
+              track->v_pctop = uint64(*kv_pctop);
               if (verbose > 1)
-                mxinfo(PFX "|   + Pixel crop top: %u\n", track->v_pctop);
+                mxinfo(PFX "|   + Pixel crop top: %llu\n", track->v_pctop);
             }
 
             kv_pcright = FINDFIRST(ktvideo, KaxVideoPixelCropRight);
             if (kv_pcright != NULL) {
-              track->v_pcright = uint16(*kv_pcright);
+              track->v_pcright = uint64(*kv_pcright);
               if (verbose > 1)
-                mxinfo(PFX "|   + Pixel crop right: %u\n", track->v_pcright);
+                mxinfo(PFX "|   + Pixel crop right: %llu\n", track->v_pcright);
             }
 
             kv_pcbottom = FINDFIRST(ktvideo, KaxVideoPixelCropBottom);
             if (kv_pcbottom != NULL) {
-              track->v_pcbottom = uint16(*kv_pcbottom);
+              track->v_pcbottom = uint64(*kv_pcbottom);
               if (verbose > 1)
-                mxinfo(PFX "|   + Pixel crop bottom: %u\n", track->v_pcbottom);
+                mxinfo(PFX "|   + Pixel crop bottom: %llu\n",
+                       track->v_pcbottom);
             }
 
           }
@@ -1137,37 +1130,37 @@ kax_reader_c::read_headers() {
           ktmincache = FINDFIRST(ktentry, KaxTrackMinCache);
           if (ktmincache != NULL) {
             if (verbose > 1)
-              mxinfo(PFX "|  + MinCache: %u\n", uint32(*ktmincache));
-            if (uint32(*ktmincache) > 1)
+              mxinfo(PFX "|  + MinCache: %llu\n", uint64(*ktmincache));
+            if (uint64(*ktmincache) > 1)
               track->v_bframes = true;
-            track->min_cache = uint32(*ktmincache);
+            track->min_cache = uint64(*ktmincache);
           }
 
           ktmaxcache = FINDFIRST(ktentry, KaxTrackMaxCache);
           if (ktmaxcache != NULL) {
-            mxverb(2, PFX "|  + MaxCache: %u\n", uint32(*ktmaxcache));
-            track->max_cache = uint32(*ktmaxcache);
+            mxverb(2, PFX "|  + MaxCache: %llu\n", uint64(*ktmaxcache));
+            track->max_cache = uint64(*ktmaxcache);
           }
 
           ktfdefault = FINDFIRST(ktentry, KaxTrackFlagDefault);
           if (ktfdefault != NULL) {
             if (verbose > 1)
-              mxinfo(PFX "|  + Default flag: %d\n", uint32(*ktfdefault));
-            track->default_track = uint32(*ktfdefault);
+              mxinfo(PFX "|  + Default flag: %llu\n", uint64(*ktfdefault));
+            track->default_track = uint64(*ktfdefault);
           }
 
           ktflacing = FINDFIRST(ktentry, KaxTrackFlagLacing);
           if (ktflacing != NULL) {
-            mxverb(2, PFX "|  + Lacing flag: %d\n", uint32(*ktflacing));
-            track->lacing_flag = uint32(*ktflacing);
+            mxverb(2, PFX "|  + Lacing flag: %llu\n", uint64(*ktflacing));
+            track->lacing_flag = uint64(*ktflacing);
           } else
             track->lacing_flag = true;
 
           ktmax_blockadd_id = FINDFIRST(ktentry, KaxMaxBlockAdditionID);
           if (ktmax_blockadd_id != NULL) {
-            mxverb(2, PFX "|  + Max Block Addition ID: %u\n",
-                   uint32(*ktmax_blockadd_id));
-            track->max_blockadd_id = uint32(*ktmax_blockadd_id);
+            mxverb(2, PFX "|  + Max Block Addition ID: %llu\n",
+                   uint64(*ktmax_blockadd_id));
+            track->max_blockadd_id = uint64(*ktmax_blockadd_id);
           } else
             track->max_blockadd_id = 0;
 
@@ -1281,8 +1274,9 @@ kax_reader_c::read_headers() {
 
     } // while (l1 != NULL)
 
-    for (i = 0; i < deferred_attachments.size(); i++)
-      handle_attachments(in, l0, deferred_attachments[i]);
+    if (!ti.no_attachments)
+      for (i = 0; i < deferred_attachments.size(); i++)
+        handle_attachments(in, l0, deferred_attachments[i]);
     for (i = 0; i < deferred_chapters.size(); i++)
       handle_chapters(in, l0, deferred_chapters[i]);
     for (i = 0; i < deferred_tags.size(); i++)
@@ -1375,7 +1369,7 @@ kax_reader_c::set_packetizer_headers(kax_track_t *t) {
                                         DEFAULT_TRACK_PRIORITY_FROM_SOURCE);
   if (t->tuid != 0)
     if (!PTZR(t->ptzr)->set_uid(t->tuid))
-      mxwarn(PFX "Could not keep the track UID %u because it is already "
+      mxwarn(PFX "Could not keep the track UID %llu because it is already "
              "allocated for the new file.\n", t->tuid);
 }
 
@@ -1765,8 +1759,10 @@ kax_reader_c::read(generic_packetizer_c *,
 
   l0 = segment;
 
-  if (saved_l1 == NULL)         // We're done.
+  if (saved_l1 == NULL) {       // We're done.
+    flush_packetizers();
     return FILE_STATUS_DONE;
+  }
 
   if (!force && (get_queued_bytes() > 20 * 1024 * 1024))
     return FILE_STATUS_HOLDING;
@@ -1893,12 +1889,14 @@ kax_reader_c::read(generic_packetizer_c *,
 
             for (i = 0; i < (int)block->NumberFrames(); i++) {
               DataBuffer &data = block->GetBuffer(i);
-              memory_c mem((unsigned char *)data.Buffer(), data.Size(),
-                           false);
+              packet_t *packet =
+                new packet_t(new memory_c((unsigned char *)data.Buffer(),
+                                          data.Size(), false),
+                             last_timecode + i * frame_duration,
+                             block_duration, block_bref, block_fref);
+              packet->duration_mandatory = duration != NULL;
               ((passthrough_packetizer_c *)PTZR(block_track->ptzr))->
-                process(mem, last_timecode + i * frame_duration,
-                        block_duration, block_bref, block_fref,
-                        duration != NULL);
+                process(packet_cptr(packet));
             }
 
           } else if ((block_track != NULL) &&
@@ -1932,21 +1930,22 @@ kax_reader_c::read(generic_packetizer_c *,
                   lines = (char *)safemalloc(re_size + 1);
                   lines[re_size] = 0;
                   memcpy(lines, re_buffer, re_size);
-                  memory_c mem((unsigned char *)lines, 0, true);
                   PTZR(block_track->ptzr)->
-                    process(mem, (int64_t)last_timecode, block_duration,
-                            block_bref, block_fref);
+                    process(new packet_t(new memory_c((unsigned char *)lines,
+                                                      0, true),
+                                         last_timecode, block_duration,
+                                         block_bref, block_fref));
                   if (re_modified)
                     safefree(re_buffer);
                 }
               } else {
-                memory_c mem(re_buffer, re_size, re_modified);
-                if (blockadd) {
-                  memories_c mems;
-                  mems.resize(blockadd->ListSize() + 1);
-                  mems[0] = &mem;
+                memory_c *mem = new memory_c(re_buffer, re_size, re_modified);
+                packet_cptr packet(new packet_t(mem, last_timecode + i *
+                                                frame_duration, block_duration,
+                                                block_bref, block_fref));
 
-                  for (i=0; i<blockadd->ListSize(); i++) {
+                if (blockadd) {
+                  for (i = 0; i < blockadd->ListSize(); i++) {
                     if (!(is_id((*blockadd)[i], KaxBlockMore)))
                       continue;
                     blockmore = static_cast<KaxBlockMore *>((*blockadd)[i]);
@@ -1957,20 +1956,13 @@ kax_reader_c::read(generic_packetizer_c *,
                     memory_c *blockadded =
                       new memory_c(blockadd_data->GetBuffer(),
                                    blockadd_data->GetSize(), false);
-                    mems[blockadd_id] = blockadded;
+                    packet->memory_adds.push_back(memory_cptr(blockadded));
                   }
 
-                  PTZR(block_track->ptzr)->
-                    process(mems, (int64_t)last_timecode + i * frame_duration,
-                            block_duration, block_bref, block_fref);
+                  PTZR(block_track->ptzr)->process(packet);
 
-                  for (i = 1; i<mems.size(); i++)
-                    delete mems[i];
-                } else {
-                  PTZR(block_track->ptzr)->
-                    process(mem, (int64_t)last_timecode + i * frame_duration,
-                            block_duration, block_bref, block_fref);
-                }
+                } else
+                  PTZR(block_track->ptzr)->process(packet);
               }
             }
 
@@ -2085,14 +2077,14 @@ kax_reader_c::identify() {
         if ((tracks[i]->v_dwidth != 0) && (tracks[i]->v_dheight != 0) &&
             ((tracks[i]->v_dwidth != tracks[i]->v_width) ||
              (tracks[i]->v_dheight != tracks[i]->v_height)))
-          info += mxsprintf("display_dimensions:%ux%u ",
+          info += mxsprintf("display_dimensions:%llux%llu ",
                             tracks[i]->v_dwidth, tracks[i]->v_dheight);
-        info += mxsprintf("default_track:%d ",
+        info += mxsprintf("default_track:%u ",
                           tracks[i]->default_track ? 1 : 0);
         info += "]";
       } else
         info = "";
-      mxinfo("Track ID %d: %s (%s%s%s)%s\n", tracks[i]->tnum,
+      mxinfo("Track ID %llu: %s (%s%s%s)%s\n", tracks[i]->tnum,
              tracks[i]->type == 'v' ? "video" :
              tracks[i]->type == 'a' ? "audio" :
              tracks[i]->type == 'b' ? "buttons" :
@@ -2108,66 +2100,20 @@ kax_reader_c::identify() {
     }
 
   for (i = 0; i < attachments.size(); i++) {
-    mxinfo("Attachment ID %lld: type '%s', size %lld bytes, ",
+    mxinfo("Attachment ID %lld: type '%s', size %d bytes, ",
            attachments[i].id, attachments[i].mime_type.c_str(),
-           attachments[i].size);
+           attachments[i].data->size);
     if (attachments[i].description.length() > 0)
-      mxinfo("description '%s', ",
-             UTFstring_to_cstrutf8(attachments[i].description).c_str());
+      mxinfo("description '%s', ", attachments[i].description.c_str());
     if (attachments[i].name.length() == 0)
       mxinfo("no file name given\n");
     else
-      mxinfo("file name '%s'\n",
-             UTFstring_to_cstrutf8(attachments[i].name).c_str());
+      mxinfo("file name '%s'\n", attachments[i].name.c_str());
   }
 }
 
 // }}}
 
-void
-kax_reader_c::add_attachments(KaxAttachments *a) {
-  uint32_t i;
-  KaxAttached *attached;
-  KaxFileData *fdata;
-  binary *buffer;
-
-  if (ti.no_attachments)
-    return;
-
-  for (i = 0; i < attachments.size(); i++) {
-    attached = new KaxAttached;
-    if (attachments[i].description.length() > 0)
-      *static_cast<EbmlUnicodeString *>
-        (&GetChild<KaxFileDescription>(*attached)) =
-        attachments[i].description;
-
-    *static_cast<EbmlString *>(&GetChild<KaxMimeType>(*attached)) =
-      attachments[i].mime_type;
-
-    *static_cast<EbmlUnicodeString *>(&GetChild<KaxFileName>(*attached)) =
-      attachments[i].name;
-
-    *static_cast<EbmlUInteger *>(&GetChild<KaxFileUID>(*attached)) =
-      attachments[i].id;
-
-    fdata = &GetChild<KaxFileData>(*attached);
-    buffer = new binary[attachments[i].size];
-    memcpy(buffer, attachments[i].data, attachments[i].size);
-    fdata->SetBuffer(buffer, attachments[i].size);
-
-    a->PushElement(*attached);
-  }
-}
-
-void
-kax_reader_c::flush_packetizers() {
-  uint32_t i;
-
-  for (i = 0; i < tracks.size(); i++)
-    if (tracks[i]->ptzr != -1)
-      PTZR(tracks[i]->ptzr)->flush();
-}
-
 void
 kax_reader_c::add_available_track_ids() {
   int i;
