@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_matroska.h 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_matroska.h 3022 2005-08-15 20:47:56Z mosu $
 
    class definitions for the Matroska reader
 
@@ -39,15 +39,8 @@ using namespace libebml;
 using namespace libmatroska;
 using namespace std;
 
-typedef struct {
-  string mime_type;
-  UTFstring name, description;
-  unsigned char *data;
-  int64 size, id;
-} kax_attachment_t;
-
 struct kax_track_t {
-  uint32_t tnum, tuid;
+  uint64_t tnum, tuid;
 
   string codec_id;
   bool ms_compat;
@@ -60,14 +53,14 @@ struct kax_track_t {
   bool lacing_flag;
 
   // Parameters for video tracks
-  uint32_t v_width, v_height, v_dwidth, v_dheight;
-  uint32_t v_pcleft, v_pctop, v_pcright, v_pcbottom;
+  uint64_t v_width, v_height, v_dwidth, v_dheight;
+  uint64_t v_pcleft, v_pctop, v_pcright, v_pcbottom;
   float v_frate;
   char v_fourcc[5];
   bool v_bframes;
 
   // Parameters for audio tracks
-  uint32_t a_channels, a_bps, a_formattag;
+  uint64_t a_channels, a_bps, a_formattag;
   float a_sfreq, a_osfreq;
 
   void *private_data;
@@ -110,6 +103,7 @@ struct kax_track_t {
                  a_sfreq(0.0), a_osfreq(0.0),
                  private_data(NULL), private_size(0),
                  default_track(false),
+                 language("eng"),
                  units_processed(0),
                  ok(false),
                  previous_timecode(0),
@@ -148,7 +142,6 @@ private:
   int64_t last_timecode, first_timecode;
   string title;
 
-  vector<kax_attachment_t> attachments;
   vector<int64_t> handled_tags, handled_attachments, handled_chapters;
 
   string writing_app;
@@ -165,7 +158,6 @@ public:
   virtual void identify();
   virtual void create_packetizers();
   virtual void create_packetizer(int64_t tid);
-  virtual void add_attachments(KaxAttachments *a);
   virtual void add_available_track_ids();
 
   static int probe_file(mm_io_c *io, int64_t size);
@@ -175,14 +167,13 @@ protected:
   virtual void init_passthrough_packetizer(kax_track_t *t);
   virtual void set_packetizer_headers(kax_track_t *t);
   virtual kax_track_t *new_kax_track();
-  virtual kax_track_t *find_track_by_num(uint32_t num, kax_track_t *c = NULL);
-  virtual kax_track_t *find_track_by_uid(uint32_t uid, kax_track_t *c = NULL);
+  virtual kax_track_t *find_track_by_num(uint64_t num, kax_track_t *c = NULL);
+  virtual kax_track_t *find_track_by_uid(uint64_t uid, kax_track_t *c = NULL);
   virtual void verify_tracks();
   virtual int packets_available();
   virtual void handle_attachments(mm_io_c *io, EbmlElement *l0, int64_t pos);
   virtual void handle_chapters(mm_io_c *io, EbmlElement *l0, int64_t pos);
   virtual void handle_tags(mm_io_c *io, EbmlElement *l0, int64_t pos);
-  virtual void flush_packetizers();
 };
 
 
