@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_mpeg.cpp 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_mpeg.cpp 2951 2005-06-10 07:12:50Z mosu $
 
    MPEG ES demultiplexer module
 
@@ -170,10 +170,8 @@ mpeg_es_reader_c::read(generic_packetizer_c *,
     return FILE_STATUS_DONE;
   }
 
-  if (num_read > 0) {
-    memory_c mem(chunk, num_read, true);
-    PTZR0->process(mem);
-  }
+  if (num_read > 0)
+    PTZR0->process(new packet_t(new memory_c(chunk, num_read, true)));
   if (num_read < 20000)
     PTZR0->flush();
 
@@ -888,8 +886,8 @@ mpeg_ps_reader_c::read(generic_packetizer_c *,
         return FILE_STATUS_DONE;
       }
 
-      memory_c mem(buf, length, true);
-      PTZR(tracks[id2idx[new_id]]->ptzr)->process(mem);
+      PTZR(tracks[id2idx[new_id]]->ptzr)->
+        process(new packet_t(new memory_c(buf, length, true)));
 
       return FILE_STATUS_MOREDATA;
     }
@@ -901,14 +899,6 @@ mpeg_ps_reader_c::read(generic_packetizer_c *,
   return FILE_STATUS_DONE;
 }
 
-void
-mpeg_ps_reader_c::flush_packetizers() {
-  vector<generic_packetizer_c *>::iterator i;
-
-  foreach(i, reader_packetizers)
-    (*i)->flush();
-}
-
 int
 mpeg_ps_reader_c::get_progress() {
   return 100 * io->getFilePointer() / size;
