@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ogm.cpp 2851 2005-03-14 13:15:04Z mosu $
+   $Id: r_ogm.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    OGG media stream reader
 
@@ -395,6 +395,8 @@ ogm_reader_c::create_packetizer(int64_t tid) {
   bool sbr, aac_info_extracted;
   ogm_demuxer_t *dmx;
   generic_packetizer_c *ptzr;
+  double fps;
+  int width, height;
 
   if ((tid < 0) || (tid >= sdemuxers.size()))
     return;
@@ -425,14 +427,20 @@ ogm_reader_c::create_packetizer(int64_t tid) {
         ti.private_data = (unsigned char *)&bih;
         ti.private_size = sizeof(alBITMAPINFOHEADER);
 
-        ptzr = new video_packetizer_c(this, NULL, (double)10000000.0 /
-                                      get_uint64_le(&sth->time_unit),
-                                      get_uint32_le(&sth->sh.video.width),
-                                      get_uint32_le(&sth->sh.video.height),
-                                      ti);
+        fps = (double)10000000.0 / get_uint64_le(&sth->time_unit);
+        width = get_uint32_le(&sth->sh.video.width);
+        height = get_uint32_le(&sth->sh.video.height);
+        if (is_mpeg4_p2_fourcc(sth->subtype)) {
+          ptzr = new mpeg4_p2_video_packetizer_c(this, fps, width, height,
+                                                 false, ti);
+          mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module.\n",
+                 ti.fname.c_str(), (int64_t)tid);
+        } else {
+          ptzr = new video_packetizer_c(this, NULL, fps, width, height, ti);
+          mxinfo(FMT_TID "Using the video output module.\n", ti.fname.c_str(),
+                 (int64_t)tid);
+        }
 
-        mxinfo(FMT_TID "Using the video output module.\n", ti.fname.c_str(),
-               (int64_t)tid);
         ti.private_data = NULL;
 
         break;
@@ -618,8 +626,8 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
   sdemuxers.push_back(dmx);
   if (ogg_stream_init(&dmx->os, ogg_page_serialno(og))) {
     mxwarn("ogm_reader: ogg_stream_init for stream number "
-           "%d failed. Will try to continue and ignore this stream.",
-           sdemuxers.size());
+           "%u failed. Will try to continue and ignore this stream.",
+           (unsigned int)sdemuxers.size());
     return;
   }
 
@@ -660,7 +668,7 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
     dmx->fhe = new flac_header_extractor_c(ti.fname, dmx->serialno);
     if (!dmx->fhe->extract())
       mxerror("ogm_reader: Could not read the FLAC header packets for "
-              "stream id %d.\n", sdemuxers.size() - 1);
+              "stream id %u.\n", (unsigned int)sdemuxers.size() - 1);
     dmx->flac_header_packets = dmx->fhe->num_header_packets;
     dmx->vorbis_rate = dmx->fhe->sample_rate;
     dmx->channels = dmx->fhe->channels;
@@ -709,7 +717,8 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
         dmx->stype = OGM_STREAM_TYPE_AAC;
       else {
         mxwarn("ogm_reader: Unknown audio stream type %u. "
-               "Ignoring stream id %d.\n", codec_id, sdemuxers.size() - 1);
+               "Ignoring stream id %u.\n", codec_id,
+               (unsigned int)sdemuxers.size() - 1);
         return;
       }
 
@@ -769,17 +778,19 @@ ogm_reader_c::process_page(ogg_page *og) {
       if (dmx->units_processed <= dmx->flac_header_packets)
         continue;
       for (i = 0; i < (int)dmx->nh_packet_data.size(); i++) {
-        memory_c mem(dmx->nh_packet_data[i], dmx->nh_packet_sizes[i], true);
-        PTZR(dmx->ptzr)->process(mem, 0);
+        memory_c *mem = new memory_c(dmx->nh_packet_data[i],
+                                     dmx->nh_packet_sizes[i], true);
+        PTZR(dmx->ptzr)->process(new packet_t(mem, 0));
       }
       dmx->nh_packet_data.clear();
-      if (dmx->last_granulepos == -1) {
-        memory_c mem(op.packet, op.bytes, false);
-        PTZR(dmx->ptzr)->process(mem, -1);
-      } else {
-        memory_c mem(op.packet, op.bytes, false);
-        PTZR(dmx->ptzr)->process(mem, dmx->last_granulepos * 1000000000 /
-                                 dmx->vorbis_rate);
+      if (dmx->last_granulepos == -1)
+        PTZR(dmx->ptzr)->process(new packet_t(new memory_c(op.packet, op.bytes,
+                                                           false), -1));
+      else {
+        PTZR(dmx->ptzr)->process(new packet_t(new memory_c(op.packet, op.bytes,
+                                                           false),
+                                              dmx->last_granulepos *
+                                              1000000000 / dmx->vorbis_rate));
         dmx->last_granulepos = ogg_page_granulepos(og);
       }
 
@@ -805,11 +816,12 @@ ogm_reader_c::process_page(ogg_page *og) {
         ((*op.packet & 3) != PACKET_TYPE_COMMENT)) {
 
       if (dmx->stype == OGM_STREAM_TYPE_VIDEO) {
-        memory_c mem(&op.packet[duration_len + 1], op.bytes - 1 -
-                     duration_len, false);
-        PTZR(dmx->ptzr)->process(mem, -1, -1,
-                                 (*op.packet & PACKET_IS_SYNCPOINT ?
-                                  VFT_IFRAME : VFT_PFRAMEAUTOMATIC));
+        memory_c *mem = new memory_c(&op.packet[duration_len + 1],
+                                     op.bytes - 1 - duration_len, false);
+        PTZR(dmx->ptzr)->
+          process(new packet_t(mem, -1, -1,
+                               (*op.packet & PACKET_IS_SYNCPOINT ?
+                                VFT_IFRAME : VFT_PFRAMEAUTOMATIC)));
         dmx->units_processed += (duration_len > 0 ? duration : 1);
 
       } else if (dmx->stype == OGM_STREAM_TYPE_TEXT) {
@@ -818,20 +830,21 @@ ogm_reader_c::process_page(ogg_page *og) {
             ((op.packet[duration_len + 1] != ' ') &&
              (op.packet[duration_len + 1] != 0) &&
              !iscr(op.packet[duration_len + 1]))) {
-          memory_c mem(&op.packet[duration_len + 1], op.bytes - 1 -
-                       duration_len, false);
-          PTZR(dmx->ptzr)->process(mem, ogg_page_granulepos(og) * 1000000,
-                                   (int64_t)duration * 1000000);
+          memory_c *mem = new memory_c(&op.packet[duration_len + 1],
+                                       op.bytes - 1 - duration_len, false);
+          PTZR(dmx->ptzr)->
+            process(new packet_t(mem, ogg_page_granulepos(og) * 1000000,
+                                 (int64_t)duration * 1000000));
         }
 
-      } else if (dmx->stype == OGM_STREAM_TYPE_VORBIS) {
-        memory_c mem(op.packet, op.bytes, false);
-        PTZR(dmx->ptzr)->process(mem);
+      } else if (dmx->stype == OGM_STREAM_TYPE_VORBIS)
+        PTZR(dmx->ptzr)->process(new packet_t(new memory_c(op.packet, op.bytes,
+                                                           false)));
 
-      } else {
-        memory_c mem(&op.packet[duration_len + 1], op.bytes - 1 -
-                     duration_len, false);
-        PTZR(dmx->ptzr)->process(mem);
+      else {
+        memory_c *mem = new memory_c(&op.packet[duration_len + 1],
+                                     op.bytes - 1 - duration_len, false);
+        PTZR(dmx->ptzr)->process(new packet_t(mem));
         dmx->units_processed += op.bytes - 1;
       }
     }
@@ -1056,15 +1069,6 @@ ogm_reader_c::identify() {
   }
 }
 
-void
-ogm_reader_c::flush_packetizers() {
-  int i;
-
-  for (i = 0; i < sdemuxers.size(); i++)
-    if (sdemuxers[i]->ptzr != -1)
-      PTZR(sdemuxers[i]->ptzr)->flush();
-}
-
 void
 ogm_reader_c::handle_stream_comments() {
   int i, j, cch;
@@ -1170,10 +1174,8 @@ ogm_reader_c::handle_stream_comments() {
       try {
         out = new mm_mem_io_c(NULL, 0, 1000);
         out->write_bom("UTF-8");
-        for (j = 0; j < chapters.size(); j++) {
-          out->puts_unl(to_utf8(cch, chapters[j]));
-          out->puts_unl("\n");
-        }
+        for (j = 0; j < chapters.size(); j++)
+          out->puts(to_utf8(cch, chapters[j]) + string("\n"));
         out->set_file_name(ti.fname);
         kax_chapters = parse_chapters(new mm_text_io_c(out));
       } catch (...) {
