@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_qtmp4.cpp 2846 2005-03-06 19:07:17Z mosu $
+   $Id: r_qtmp4.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    Quicktime and MP4 reader
 
@@ -75,7 +75,8 @@ qtmp4_reader_c::probe_file(mm_io_c *in,
 
     if ((atom == FOURCC('m', 'o', 'o', 'v')) ||
         (atom == FOURCC('f', 't', 'y', 'p')) ||
-        (atom == FOURCC('m', 'd', 'a', 't')))
+        (atom == FOURCC('m', 'd', 'a', 't')) ||
+        (atom == FOURCC('p', 'n', 'o', 't')))
         return 1;
 
   } catch (...) {
@@ -172,15 +173,9 @@ qtmp4_reader_c::parse_headers() {
       mdat_size = atom.size;
       skip_atom();
 
-    } else if ((atom.fourcc == FOURCC('f', 'r', 'e', 'e')) ||
-               (atom.fourcc == FOURCC('s', 'k', 'i', 'p')) ||
-               (atom.fourcc == FOURCC('j', 'u', 'n', 'k')) ||
-               (atom.fourcc == FOURCC('p', 'n', 'o', 't')) ||
-               (atom.fourcc == FOURCC('P', 'I', 'C', 'T'))) {
+    } else
       skip_atom();
 
-    }
-
   } while (!io->eof() && (!headers_parsed || (mdat_pos == -1)));
 
   if (!headers_parsed)
@@ -310,6 +305,8 @@ qtmp4_reader_c::parse_headers() {
         dmx->chunk_table[j].size = dmx->chunkmap_table[i].samples_per_chunk;
       }
       last = dmx->chunkmap_table[i].first_chunk;
+      if (dmx->chunk_table.size() <= last)
+        break;
     }
 
     // calc pts of chunks:
@@ -375,10 +372,11 @@ qtmp4_reader_c::parse_headers() {
           push_back(dmx->raw_frame_offset_table[j].offset);
     }
     mxverb(3, PFX "Frame offset table: %u entries\n",
-           dmx->frame_offset_table.size());
+           (unsigned int)dmx->frame_offset_table.size());
   }
 
-  mxverb(2, PFX "Number of valid tracks found: %u\n", demuxers.size());
+  mxverb(2, PFX "Number of valid tracks found: %u\n",
+         (unsigned int)demuxers.size());
 }
 
 void
@@ -570,8 +568,8 @@ qtmp4_reader_c::handle_hdlr_atom(qtmp4_demuxer_ptr &new_dmx,
   hdlr_atom_t hdlr;
 
   if (atom.size < sizeof(hdlr_atom_t))
-    mxerror(PFX "'hdlr' atom is too small. Expected size: >= %d. Actual "
-            "size: %lld.\n", sizeof(hdlr_atom_t), atom.size);
+    mxerror(PFX "'hdlr' atom is too small. Expected size: >= %u. Actual "
+            "size: %lld.\n", (unsigned int)sizeof(hdlr_atom_t), atom.size);
   if (io->read(&hdlr, sizeof(hdlr_atom_t)) != sizeof(hdlr_atom_t))
     throw error_c("end-of-file");
   mxverb(2, PFX "%*s Component type: %.4s subtype: %.4s\n",
@@ -593,8 +591,8 @@ qtmp4_reader_c::handle_mdhd_atom(qtmp4_demuxer_ptr &new_dmx,
   mdhd_atom_t mdhd;
 
   if (atom.size < sizeof(mdhd_atom_t))
-    mxerror(PFX "'mdhd' atom is too small. Expected size: >= %d. Actual "
-            "size: %lld.\n", sizeof(mdhd_atom_t), atom.size);
+    mxerror(PFX "'mdhd' atom is too small. Expected size: >= %u. Actual "
+            "size: %lld.\n", (unsigned int)sizeof(mdhd_atom_t), atom.size);
   if (io->read(&mdhd, sizeof(mdhd_atom_t)) != sizeof(mdhd_atom_t))
     throw error_c("end-of-file");
   mxverb(2, PFX "%*s Time scale: %u, duration: %u\n", level * 2, "",
@@ -689,8 +687,8 @@ qtmp4_reader_c::handle_mvhd_atom(qt_atom_t atom,
   mvhd_atom_t mvhd;
 
   if ((atom.size - atom.hsize) < sizeof(mvhd_atom_t))
-    mxerror(PFX "'mvhd' atom is too small. Expected size: >= %d. Actual "
-            "size: %lld.\n", sizeof(mvhd_atom_t), atom.size - atom.hsize);
+    mxerror(PFX "'mvhd' atom is too small. Expected size: >= %u. Actual "
+            "size: %lld.\n", (unsigned int)sizeof(mvhd_atom_t), atom.size);
   if (io->read(&mvhd, sizeof(mvhd_atom_t)) != sizeof(mvhd_atom_t))
     throw error_c("end-of-file");
   mxverb(2, PFX "%*s Time scale: %u\n", level * 2, "",
@@ -839,8 +837,10 @@ qtmp4_reader_c::handle_stsd_atom(qtmp4_demuxer_ptr &new_dmx,
 
       if (get_uint16_be(&sv1_stsd.v0.version) == 1)
         stsd_size = sizeof(sound_v1_stsd_atom_t);
-      else
+      else if (get_uint16_be(&sv1_stsd.v0.version) == 0)
         stsd_size = sizeof(sound_v0_stsd_atom_t);
+      else if (get_uint16_be(&sv1_stsd.v0.version) == 2)
+        stsd_size = 68;
 
       memcpy(&new_dmx->a_stsd, &sv1_stsd, sizeof(sound_v1_stsd_atom_t));
 
@@ -970,8 +970,8 @@ qtmp4_reader_c::handle_tkhd_atom(qtmp4_demuxer_ptr &new_dmx,
   tkhd_atom_t tkhd;
 
   if (atom.size < sizeof(tkhd_atom_t))
-    mxerror(PFX "'tkhd' atom is too small. Expected size: >= %d. Actual "
-            "size: %lld.\n", sizeof(tkhd_atom_t), atom.size);
+    mxerror(PFX "'tkhd' atom is too small. Expected size: >= %u. Actual "
+            "size: %lld.\n", (unsigned int)sizeof(tkhd_atom_t), atom.size);
   if (io->read(&tkhd, sizeof(tkhd_atom_t)) != sizeof(tkhd_atom_t))
     throw error_c("end-of-file");
   mxverb(2, PFX "%*s Track ID: %u\n", level * 2, "",
@@ -1006,7 +1006,7 @@ qtmp4_reader_c::handle_video_with_bframes(qtmp4_demuxer_ptr &dmx,
                                           int64_t timecode,
                                           int64_t duration,
                                           bool is_keyframe,
-                                          memory_c &mem) {
+                                          memory_cptr mem) {
   int64_t bref, fref, old_timecode;
   bool is_bframe;
 
@@ -1073,7 +1073,7 @@ qtmp4_reader_c::handle_video_with_bframes(qtmp4_demuxer_ptr &dmx,
           1000000000 / dmx->timescale) / 1000000,
          timecode / 1000000, bref, fref);
 
-  PTZR(dmx->ptzr)->process(mem, timecode, duration, bref, fref);
+  PTZR(dmx->ptzr)->process(new packet_t(mem, timecode, duration, bref, fref));
 
   if (timecode > dmx->max_timecode)
     dmx->max_timecode = timecode;
@@ -1138,8 +1138,9 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
       buffer = (unsigned char *)safemalloc(frame_size);
       if (io->read(buffer, frame_size) != frame_size) {
         mxwarn(PFX "Could not read chunk number %u/%u with size %u from "
-               "position %lld. Aborting.\n", frame, dmx->chunk_table.size(),
-               frame_size, dmx->chunk_table[dmx->pos].pos);
+               "position %lld. Aborting.\n", frame,
+               (unsigned int)dmx->chunk_table.size(), frame_size,
+               dmx->chunk_table[dmx->pos].pos);
         safefree(buffer);
         flush_packetizers();
 
@@ -1156,9 +1157,11 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
       dmx->avg_duration = (dmx->avg_duration * dmx->pos + duration) /
         (dmx->pos + 1);
 
-      memory_c mem(buffer, frame_size, true);
-      PTZR(dmx->ptzr)->process(mem, timecode, duration, is_keyframe ?
-                               VFT_IFRAME : VFT_PFRAMEAUTOMATIC);
+      PTZR(dmx->ptzr)->process(new packet_t(new memory_c(buffer, frame_size,
+                                                         true),
+                                            timecode, duration, is_keyframe ?
+                                            VFT_IFRAME : VFT_PFRAMEAUTOMATIC,
+                                            VFT_NOBFRAME));
       dmx->pos++;
 
       if (dmx->pos < dmx->chunk_table.size())
@@ -1202,7 +1205,8 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
         if (io->read(buffer + dmx->esds.decoder_config_len, frame_size) !=
             frame_size) {
           mxwarn(PFX "Could not read chunk number %u/%u with size %u from "
-                 "position %lld. Aborting.\n", frame, dmx->chunk_table.size(),
+                 "position %lld. Aborting.\n", frame,
+                 (unsigned int)dmx->chunk_table.size(),
                  frame_size, dmx->chunk_table[dmx->pos].pos);
           safefree(buffer);
           flush_packetizers();
@@ -1215,7 +1219,8 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
         io->setFilePointer(dmx->sample_table[frame].pos);
         if (io->read(buffer, frame_size) != frame_size) {
           mxwarn(PFX "Could not read chunk number %u/%u with size %u from "
-                 "position %lld. Aborting.\n", frame, dmx->sample_table.size(),
+                 "position %lld. Aborting.\n", frame,
+                 (unsigned int)dmx->sample_table.size(),
                  frame_size, dmx->sample_table[frame].pos);
           safefree(buffer);
           flush_packetizers();
@@ -1224,14 +1229,16 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
         }
       }
 
-      memory_c mem(buffer, frame_size, true);
+      memory_cptr mem(new memory_c(buffer, frame_size, true));
       if ((dmx->type == 'v') && (dmx->pos < dmx->frame_offset_table.size()) &&
           !strncasecmp(dmx->fourcc, "avc1", 4))
         handle_video_with_bframes(dmx, timecode, duration, is_keyframe, mem);
 
       else
-        PTZR(dmx->ptzr)->process(mem, timecode, duration, is_keyframe ?
-                                 VFT_IFRAME : VFT_PFRAMEAUTOMATIC);
+        PTZR(dmx->ptzr)->process(new packet_t(mem, timecode, duration,
+                                              is_keyframe ? VFT_IFRAME :
+                                              VFT_PFRAMEAUTOMATIC,
+                                              VFT_NOBFRAME));
 
       dmx->pos++;
       if (dmx->pos < dmx->sample_table.size())
@@ -1475,9 +1482,12 @@ qtmp4_reader_c::create_packetizer(int64_t tid) {
         bool sbraac;
 
         if (dmx->esds.decoder_config_len >= 2) {
-          parse_aac_data(dmx->esds.decoder_config,
-                         dmx->esds.decoder_config_len, profile, channels,
-                         sample_rate, output_sample_rate, sbraac);
+          if (!parse_aac_data(dmx->esds.decoder_config,
+                              dmx->esds.decoder_config_len, profile, channels,
+                              sample_rate, output_sample_rate, sbraac))
+            mxerror(FMT_TID "This AAC track does not contain valid headers. "
+                    "Could not parse the AAC information.\n", ti.fname.c_str(),
+                    (int64_t)dmx->id);
           mxverb(2, PFX "AAC: profile: %d, sample_rate: %d, channels: "
                  "%d, output_sample_rate: %d, sbr: %d\n", profile, sample_rate,
                  channels, output_sample_rate, (int)sbraac);
@@ -1566,15 +1576,6 @@ qtmp4_reader_c::identify() {
   }
 }
 
-void
-qtmp4_reader_c::flush_packetizers() {
-  uint32_t i;
-
-  for (i = 0; i < demuxers.size(); i++)
-    if (demuxers[i]->ptzr != -1)
-      PTZR(demuxers[i]->ptzr)->flush();
-}
-
 void
 qtmp4_reader_c::add_available_track_ids() {
   int i;
@@ -1620,7 +1621,7 @@ qtmp4_demuxer_t::calculate_fps() {
       fps = (double)sample_table.size() * 1000000000.0 / (double)max_tc;
 
     mxverb(3, PFX "calculate_fps: case 2: max_tc %lld s_t.s %u fps %f\n",
-           max_tc, sample_table.size(), fps);
+           max_tc, (unsigned int)sample_table.size(), fps);
     return fps;
   }
 
@@ -1637,7 +1638,7 @@ qtmp4_demuxer_t::calculate_fps() {
       fps = (double)sample_table.size() * 1000000000.0 / (double)max_tc;
 
     mxverb(3, PFX "calculate_fps: case 3: max_tc %lld s_t.s %u fps %f\n",
-           max_tc, sample_table.size(), fps);
+           max_tc, (unsigned int)sample_table.size(), fps);
     return fps;
   }
 
