@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_qtmp4.h 2846 2005-03-06 19:07:17Z mosu $
+   $Id: r_qtmp4.h 2951 2005-06-10 07:12:50Z mosu $
 
    class definitions for the Quicktime & MP4 reader
 
@@ -210,11 +210,11 @@ protected:
   virtual bool parse_esds_atom(mm_mem_io_c &memio, qtmp4_demuxer_ptr &dmx,
                                int level);
   virtual uint32_t read_esds_descr_len(mm_mem_io_c &memio);
-  virtual void flush_packetizers();
 
   virtual void handle_video_with_bframes(qtmp4_demuxer_ptr &dmx,
                                          int64_t timecode, int64_t duration,
-                                         bool is_keyframe, memory_c &mem);
+                                         bool is_keyframe,
+                                         memory_cptr mem);
 
   virtual void handle_cmov_atom(qt_atom_t parent, int level);
   virtual void handle_cmvd_atom(qt_atom_t parent, int level);
