@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_real.cpp 2876 2005-04-07 15:17:17Z mosu $
+   $Id: r_real.cpp 2951 2005-06-10 07:12:50Z mosu $
 
    RealMedia demultiplexer module
 
@@ -361,9 +361,12 @@ real_reader_c::create_packetizer(int64_t tid) {
           mxverb(2, PFX "extra_len: %u\n", extra_len);
           if (dmx->extra_data_size >= (4 + extra_len)) {
             extra_data_parsed = true;
-            parse_aac_data(&dmx->extra_data[4 + 1], extra_len - 1,
-                           profile, channels, sample_rate, output_sample_rate,
-                           sbr);
+            if (!parse_aac_data(&dmx->extra_data[4 + 1], extra_len - 1,
+                                profile, channels, sample_rate,
+                                output_sample_rate, sbr))
+              mxerror(FMT_TID "This AAC track does not contain valid headers. "
+                      "Could not parse the AAC information.\n",
+                      ti.fname.c_str(), (int64_t)track->id);
             mxverb(2, PFX "1. profile: %d, channels: %d, "
                    "sample_rate: %d, output_sample_rate: %d, sbr: %d\n",
                    profile, channels, sample_rate, output_sample_rate,
@@ -588,10 +591,11 @@ real_reader_c::deliver_audio_frames(real_demuxer_t *dmx,
            "0x%08x duration %llu\n", dmx->track->id, ti.fname.c_str(),
            segment.size, dmx->last_timecode, (uint32_t)segment.flags,
            duration);
-    memory_c mem(segment.data, segment.size, true);
-    PTZR(dmx->ptzr)->process(mem, dmx->last_timecode, duration,
-                             (segment.flags & 2) == 2 ? -1 :
-                             dmx->ref_timecode);
+    PTZR(dmx->ptzr)->process(new packet_t(new memory_c(segment.data,
+                                                       segment.size, true),
+                                          dmx->last_timecode, duration,
+                                          (segment.flags & 2) == 2 ? -1 :
+                                          dmx->ref_timecode));
     if ((segment.flags & 2) == 2)
       dmx->ref_timecode = dmx->last_timecode;
   }
@@ -637,8 +641,8 @@ real_reader_c::deliver_aac_frames(real_demuxer_t *dmx,
   data_idx = 2 + num_sub_packets * 2;
   for (i = 0; i < num_sub_packets; i++) {
     sub_length = get_uint16_be(&chunk[2 + i * 2]);
-    memory_c mem(&chunk[data_idx], sub_length, false);
-    PTZR(dmx->ptzr)->process(mem);
+    PTZR(dmx->ptzr)->process(new packet_t(new memory_c(&chunk[data_idx],
+                                                       sub_length, false)));
     data_idx += sub_length;
   }
 }
@@ -680,13 +684,15 @@ real_reader_c::assemble_video_packet(real_demuxer_t *dmx,
   }
   assembled = rmff_get_packed_video_frame(dmx->track);
   while (assembled != NULL) {
-    memory_c mem(assembled->data, assembled->size, true);
     if (!dmx->rv_dimensions)
       set_dimensions(dmx, assembled->data, assembled->size);
-    PTZR(dmx->ptzr)->process(mem, (int64_t)assembled->timecode * 1000000, -1,
-                             (assembled->flags & RMFF_FRAME_FLAG_KEYFRAME) ==
-                             RMFF_FRAME_FLAG_KEYFRAME ? VFT_IFRAME :
-                             VFT_PFRAMEAUTOMATIC);
+    packet_t *packet =
+      new packet_t(new memory_c(assembled->data, assembled->size, true),
+                   (int64_t)assembled->timecode * 1000000, -1,
+                   (assembled->flags & RMFF_FRAME_FLAG_KEYFRAME) ==
+                   RMFF_FRAME_FLAG_KEYFRAME ? VFT_IFRAME :
+                   VFT_PFRAMEAUTOMATIC, VFT_NOBFRAME);
+    PTZR(dmx->ptzr)->process(packet);
     assembled->allocated_by_rmff = 0;
     rmff_release_frame(assembled);
     assembled = rmff_get_packed_video_frame(dmx->track);
@@ -851,15 +857,6 @@ real_reader_c::get_information_from_data() {
   file->num_packets_read = 0;
 }
 
-void
-real_reader_c::flush_packetizers() {
-  uint32_t i;
-
-  for (i = 0; i < demuxers.size(); i++)
-    if (demuxers[i]->ptzr != -1)
-      PTZR(demuxers[i]->ptzr)->flush();
-}
-
 void
 real_reader_c::add_available_track_ids() {
   int i;
