@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ssa.cpp 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_ssa.cpp 2951 2005-06-10 07:12:50Z mosu $
 
    SSA/ASS subtitle parser
 
@@ -264,7 +264,11 @@ ssa_reader_c::read(generic_packetizer_c *,
 
   subs.process((textsubs_packetizer_c *)PTZR0);
 
-  return subs.empty() ? FILE_STATUS_DONE : FILE_STATUS_MOREDATA;
+  if (subs.empty()) {
+    flush_packetizers();
+    return FILE_STATUS_DONE;
+  }
+  return FILE_STATUS_MOREDATA;
 }
 
 int
