@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_vobsub.cpp 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_vobsub.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    VobSub stream reader
 
@@ -251,7 +251,7 @@ vobsub_reader_c::parse_headers() {
         else {
           tracks.push_back(track);
           if (sort_required) {
-            mxverb(2, PFX "Sorting track %u\n", tracks.size());
+            mxverb(2, PFX "Sorting track %u\n", (unsigned int)tracks.size());
             stable_sort(track->entries.begin(), track->entries.end());
           }
         }
@@ -313,7 +313,7 @@ vobsub_reader_c::parse_headers() {
     else {
       tracks.push_back(track);
       if (sort_required) {
-        mxverb(2, PFX "Sorting track %u\n", tracks.size());
+        mxverb(2, PFX "Sorting track %u\n", (unsigned int)tracks.size());
         stable_sort(track->entries.begin(), track->entries.end());
       }
     }
@@ -357,10 +357,10 @@ vobsub_reader_c::deliver_packet(unsigned char *buf,
            ti.fname.c_str(), ARG_TIMECODE(timecode));
     duration = default_duration;
   }
-  if (duration != -2) {
-    memory_c mem(buf, size, true);
-    ptzr->process(mem, timecode, duration);
-  } else
+  if (duration != -2)
+    ptzr->process(new packet_t(new memory_c(buf, size, true), timecode,
+                               duration));
+  else
     safefree(buf);
 
   return -1;
