@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_wavpack.cpp 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_wavpack.cpp 2951 2005-06-10 07:12:50Z mosu $
 
    WAVPACK demultiplexer module
 
@@ -130,14 +130,18 @@ wavpack_reader_c::read(generic_packetizer_c *ptzr,
   while (dummy_meta.channel_count < meta.channel_count) {
     extra_frames_number++;
     block_size = wv_parse_frame(io, dummy_header, dummy_meta, false, false);
-    if (block_size == -1)
+    if (block_size == -1) {
+      PTZR0->flush();
       return FILE_STATUS_DONE;
+    }
     data_size += block_size;
     io->skip(block_size);
   }
 
-  if (data_size < 0)
+  if (data_size < 0) {
+    PTZR0->flush();
     return FILE_STATUS_DONE;
+  }
 
   data_size += 3 * sizeof(uint32_t);
   if (extra_frames_number)
@@ -162,17 +166,16 @@ wavpack_reader_c::read(generic_packetizer_c *ptzr,
       put_uint32_le(databuffer, block_size);
       databuffer += 4;
     }
-    if (io->read(databuffer, block_size) < 0)
+    if (io->read(databuffer, block_size) < 0) {
+      PTZR0->flush();
       return FILE_STATUS_DONE;
+    }
     databuffer += block_size;
   }
 
-  memory_c mem(chunk, data_size, true);
+  packet_cptr packet(new packet_t(new memory_c(chunk, data_size, true)));
 
   // find the if there is a correction file data corresponding
-  memories_c mems;
-  mems.push_back(&mem);
-
   if (io_correc) {
     do {
       initial_position = io_correc->getFilePointer();
@@ -184,8 +187,10 @@ wavpack_reader_c::read(generic_packetizer_c *ptzr,
         extra_frames_number++;
         block_size = wv_parse_frame(io_correc, dummy_header_correc,
                                     dummy_meta, false, false);
-        if (block_size == -1)
+        if (block_size == -1) {
+          PTZR0->flush();
           return FILE_STATUS_DONE;
+        }
         data_size += block_size;
         io_correc->skip(block_size);
       }
@@ -230,14 +235,13 @@ wavpack_reader_c::read(generic_packetizer_c *ptzr,
         databuffer += block_size;
       }
 
-      memory_c mem_correc(chunk_correc, data_size, true);
-      mems.push_back(&mem_correc);
-      PTZR0->process(mems);
+      packet->memory_adds.push_back(memory_cptr(new memory_c(chunk_correc,
+                                                             data_size,
+                                                             true)));
     }
   }
 
-  if (mems.size() == 1)
-    PTZR0->process(mem);
+  PTZR0->process(packet);
 
   return FILE_STATUS_MOREDATA;
 }
