@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: mkvmerge.cpp 2865 2005-03-27 10:32:03Z mosu $
+   \version $Id: mkvmerge.cpp 3022 2005-08-15 20:47:56Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -43,6 +43,7 @@
 #include <matroska/KaxTags.h>
 
 #include "chapters.h"
+#include "cluster_helper.h"
 #include "common.h"
 #include "commonebml.h"
 #include "extern_data.h"
@@ -110,8 +111,8 @@ file_type_t file_types[] =
 /** \brief Outputs usage information
 */
 static void
-usage() {
-  mxinfo(_(
+set_usage() {
+  usage_text = _(
     "mkvmerge -o out [global options] [options1] <file1> [@optionsfile ...]"
     "\n\n Global options:\n"
     "  -v, --verbose            verbose status\n"
@@ -154,6 +155,9 @@ usage() {
     "  --split <d[K,M,G]|HH:MM:SS|s>\n"
     "                           Create a new file after d bytes (KB, MB, GB)\n"
     "                           or after a specific time.\n"
+    "  --split timecodes:A[,B...]\n"
+    "                           Create a new file after each timecode A, B\n"
+    "                           etc.\n"
     "  --split-max-files <n>    Create at most n files.\n"
     "  --link                   Link splitted files.\n"
     "  --link-to-previous <SID> Link the first file to the given SID.\n"
@@ -240,7 +244,9 @@ usage() {
     "                           ISO639-2 codes.\n"
     "  --priority <priority>    Set the priority mkvmerge runs with.\n"
     "  --command-line-charset   Charset for strings on the command line\n"
-    "  --output-charset         Output messages in this charset\n"
+    "  --output-charset <cset>  Output messages in this charset\n"
+    "  -r, --redirect-output <file>\n"
+    "                           Redirects all messages into this file.\n"
     "  @optionsfile             Reads additional command line options from\n"
     "                           the specified file (see man page).\n"
     "  -h, --help               Show this help.\n"
@@ -248,7 +254,9 @@ usage() {
     "\n\nPlease read the man page/the HTML documentation to mkvmerge. It\n"
     "explains several details in great length which are not obvious from\n"
     "this listing.\n"
-  ));
+  );
+
+  version_info = "mkvmerge v" VERSION " ('" VERSIONNAME "')";
 }
 
 static bool print_malloc_report = false;
@@ -257,7 +265,7 @@ static bool print_malloc_report = false;
 */
 static void
 print_capabilities() {
-  mxinfo("VERSION=" VERSIONINFO "\n");
+  mxinfo("VERSION=%s\n", version_info.c_str());
 #if defined(HAVE_AVICLASSES)
   mxinfo("AVICLASSES\n");
 #endif
@@ -641,70 +649,73 @@ parse_cropping(const string &s,
   ti.pixel_crop_list.push_back(crop);
 }
 
-/** \brief Parse the \c --split argument
+/** \brief Parse the duration formats to \c --split
 
-   The \c --split option takes several formats.
-
-   \arg size based: If only a number is given or the number is
-   postfixed with '<tt>K</tt>', '<tt>M</tt>' or '<tt>G</tt>' this is
-   interpreted as the size after which to split.
-
-   \arg time based: If a number postfixed with '<tt>s</tt>' or in a
-   format matching '<tt>HH:MM:SS</tt>' or '<tt>HH:MM:SS.mmm</tt>' is
-   given then this is interpreted as the time after which to split.
+  This function is called by ::parse_split if the format specifies
+  a duration after which a new file should be started.
 */
 static void
-parse_split(const string &arg) {
-  int64_t modifier;
+parse_split_duration(const string &arg) {
+  int64_t split_after;
+  string s = arg;
+
+  if (starts_with_case(s, "duration:"))
+    s.erase(0, strlen("duration:"));
+
+  if (!parse_timecode(s, split_after))
+      mxerror(_("Invalid time for '--split' in '--split %s'. Additional error "
+                "message: %s\n"), arg.c_str(), timecode_parser_error.c_str());
+
+  cluster_helper->add_split_point(split_point_t(split_after,
+                                                split_point_t::SPT_DURATION,
+                                                false));
+}
+
+/** \brief Parse the timecode format to \c --split
+
+  This function is called by ::parse_split if the format specifies
+  a timecodes after which a new file should be started.
+*/
+static void
+parse_split_timecodes(const string &arg) {
+  int64_t split_after;
+  string s = arg;
+  vector<string> timecodes;
+  vector<string>::const_iterator timecode;
+
+  if (starts_with_case(s, "timecodes:"))
+    s.erase(0, 10);
+
+  timecodes = split(s, ",");
+  foreach(timecode, timecodes) {
+    if (!parse_timecode(*timecode, split_after))
+      mxerror(_("Invalid time for '--split' in '--split %s'. Additional error "
+                "message: %s.\n"), arg.c_str(), timecode_parser_error.c_str());
+    cluster_helper->add_split_point(split_point_t(split_after,
+                                                  split_point_t::SPT_TIMECODE,
+                                                  true));
+  }
+}
+
+/** \brief Parse the size format to \c --split
+
+  This function is called by ::parse_split if the format specifies
+  a size after which a new file should be started.
+*/
+static void
+parse_split_size(const string &arg) {
+  int64_t modifier, split_after;
+  string s = arg;
   char mod;
-  string s;
 
-  if (arg.size() < 2)
-    mxerror(_("Invalid format for '--split' in '--split %s'.\n"), arg.c_str());
+  if (starts_with_case(s, "size:"))
+    s.erase(0, strlen("size:"));
 
-  s = arg;
-
-  // HH:MM:SS
-  if (((s.size() == 8) || (s.size() == 12)) &&
-      (s[2] == ':') && (s[5] == ':') &&
-      isdigit(s[0]) && isdigit(s[1]) && isdigit(s[3]) &&
-      isdigit(s[4]) && isdigit(s[6]) && isdigit(s[7])) {
-    int secs, mins, hours, msecs;
-
-    if (s.size() == 12) {
-      if ((s[8] != '.') || !parse_int(s.substr(9), msecs))
-        mxerror(_("Invalid time for '--split' in '--split %s'.\n"),
-                arg.c_str());
-      s.erase(8);
-    } else
-      msecs = 0;
-
-    mxsscanf(s.c_str(), "%d:%d:%d", &hours, &mins, &secs);
-    split_after = secs + mins * 60 + hours * 3600;
-    if ((hours < 0) || (mins < 0) || (mins > 59) || (secs < 0) || (secs > 59))
-      mxerror(_("Invalid time for '--split' in '--split %s'.\n"),
-              arg.c_str());
-
-    split_after = split_after * 1000 + msecs;
-    split_by_time = true;
-    return;
-  }
-
-  mod = tolower(s[s.size() - 1]);
-
-  // Number of seconds: e.g. 1000s or 4254S
-  if (mod == 's') {
-    s.erase(s.size() - 1);
-    if (!parse_int(s, split_after))
-      mxerror(_("Invalid time for '--split' in '--split %s'.\n"),
-              arg.c_str());
-
-    split_after *= 1000;
-    split_by_time = true;
-    return;
-  }
+  if (s.empty())
+    mxerror(_("Invalid split size in '--split %s'.\n"), arg.c_str());
 
   // Size in bytes/KB/MB/GB
+  mod = tolower(s[s.length() - 1]);
   modifier = 1;
   if (mod == 'k')
     modifier = 1024;
@@ -714,13 +725,70 @@ parse_split(const string &arg) {
     modifier = 1024 * 1024 * 1024;
   else if (!isdigit(mod))
     mxerror(_("Invalid split size in '--split %s'.\n"), arg.c_str());
+
   if (modifier != 1)
     s.erase(s.size() - 1);
   if (!parse_int(s, split_after))
     mxerror(_("Invalid split size in '--split %s'.\n"), arg.c_str());
 
-  split_after *= modifier;
-  split_by_time = false;
+  cluster_helper->add_split_point(split_point_t(split_after * modifier,
+                                                split_point_t::SPT_SIZE,
+                                                false));
+}
+
+/** \brief Parse the \c --split argument
+
+   The \c --split option takes several formats.
+
+   \arg size based: If only a number is given or the number is
+   postfixed with '<tt>K</tt>', '<tt>M</tt>' or '<tt>G</tt>' this is
+   interpreted as the size after which to split.
+   This format is parsed by ::parse_split_size
+
+   \arg time based: If a number postfixed with '<tt>s</tt>' or in a
+   format matching '<tt>HH:MM:SS</tt>' or '<tt>HH:MM:SS.mmm</tt>' is
+   given then this is interpreted as the time after which to split.
+   This format is parsed by ::parse_split_duration
+*/
+static void
+parse_split(const string &arg) {
+  string s;
+
+  if (arg.size() < 2)
+    mxerror(_("Invalid format for '--split' in '--split %s'.\n"), arg.c_str());
+
+  s = arg;
+
+  // HH:MM:SS
+  if (starts_with_case(s, "duration:"))
+    parse_split_duration(arg);
+
+  else if (starts_with_case(s, "size:"))
+    parse_split_size(arg);
+
+  else if (starts_with_case(s, "timecodes:"))
+    parse_split_timecodes(arg);
+
+  else if (((s.size() == 8) || (s.size() == 12)) &&
+           (s[2] == ':') && (s[5] == ':') &&
+           isdigit(s[0]) && isdigit(s[1]) && isdigit(s[3]) &&
+           isdigit(s[4]) && isdigit(s[6]) && isdigit(s[7]))
+    // HH:MM:SS
+    parse_split_duration(arg);
+
+  else {
+    char mod = tolower(s[s.size() - 1]);
+
+    if ('s' == mod)
+      parse_split_duration(arg);
+
+    else if (('k' == mod) || ('m' == mod) || ('g' == mod) || isdigit(mod))
+      parse_split_size(arg);
+
+    else
+      mxerror(_("Invalid format for '--split' in '--split %s'.\n"),
+              arg.c_str());
+  }
 }
 
 /** \brief Parse the \c --delay argument
@@ -1226,24 +1294,7 @@ parse_args(vector<string> args) {
 
   ti = new track_info_c;
 
-  // Really important: Which charset does the user want?
-  i = 0;
-  while (i < args.size()) {
-    this_arg = args[i];
-    if ((i + 1) == args.size())
-      next_arg = "";
-    else
-      next_arg = args[i + 1];
-
-    if (this_arg == "--output-charset") {
-      if (next_arg == "")
-        mxerror("Missing argument for '--output-charset'.\n");
-      cc_stdio = utf8_init(next_arg);
-      args.erase(args.begin() + i, args.begin() + i + 2);
-
-    } else
-      i++;
-  }
+  handle_common_cli_args(args, "");
 
   // Check if only information about the file is wanted. In this mode only
   // two parameters are allowed: the --identify switch and the file.
@@ -1267,16 +1318,7 @@ parse_args(vector<string> args) {
       next_arg = *(sit + 1);
     no_next_arg = (sit + 1) == args.end();
 
-    if ((this_arg == "-V") || (this_arg == "--version")) {
-      mxinfo(VERSIONINFO " built on " __DATE__ " " __TIME__ "\n");
-      mxexit(0);
-
-    } else if ((this_arg == "-h") || (this_arg == "-?") ||
-               (this_arg == "--help")) {
-      usage();
-      mxexit(0);
-
-    } else if ((this_arg == "-l") || (this_arg == "--list-types")) {
+    if ((this_arg == "-l") || (this_arg == "--list-types")) {
       mxinfo(_("Known file types:\n  ext  description\n"
                "  ---  --------------------------\n"));
       for (i = 1; file_types[i].ext; i++)
@@ -1300,7 +1342,7 @@ parse_args(vector<string> args) {
 
   }
 
-  mxinfo(VERSIONINFO " built on " __DATE__ " " __TIME__ "\n");
+  mxinfo("%s built on %s %s\n", version_info.c_str(), __DATE__, __TIME__);
 
   // Now parse options that are needed right at the beginning.
   foreach(sit, args) {
@@ -1331,8 +1373,7 @@ parse_args(vector<string> args) {
 
   if (outfile == "") {
     mxinfo(_("Error: no output file name was given.\n\n"));
-    usage();
-    mxexit(2);
+    usage(2);
   }
 
   foreach(sit, args) {
@@ -1510,17 +1551,22 @@ parse_args(vector<string> args) {
       if (this_arg == "--attach-file")
         attachment.to_all_files = true;
       try {
+        attachment.data = counted_ptr<buffer_t>(new buffer_t);
         io = new mm_file_io_c(attachment.name);
-        attachment.size = io->get_size();
+        attachment.data->size = io->get_size();
+        if (0 == attachment.data->size)
+          mxerror("The size of attachment '%s' is 0.\n",
+                  attachment.name.c_str());
+        attachment.data->buffer = (unsigned char *)
+          safemalloc(attachment.data->size);
+        io->read(attachment.data->buffer, attachment.data->size);
         delete io;
-        if (attachment.size == 0)
-          throw error_c("Empty attachment");
       } catch (...) {
-        mxerror(_("The attachment '%s' could not be read, or its "
-                  "size is 0.\n"), attachment.name.c_str());
+        mxerror(_("The attachment '%s' could not be read.\n"),
+                attachment.name.c_str());
       }
 
-      attachments.push_back(attachment);
+      add_attachment(attachment);
       attachment.clear();
 
       sit++;
@@ -1639,9 +1685,9 @@ parse_args(vector<string> args) {
       if (temp == -1)
         timecode_scale_mode = TIMECODE_SCALE_MODE_AUTO;
       else {
-        if ((temp > 10000000) || (temp < 1000))
+        if ((temp > 10000000) || (temp < 1))
           mxerror(_("The given timecode scale factor is outside the valid "
-                    "range (1000...10000000 or -1 for 'sample precision "
+                    "range (1...10000000 or -1 for 'sample precision "
                     "even if a video track is present').\n"));
 
         timecode_scale = temp;
@@ -1927,7 +1973,7 @@ parse_args(vector<string> args) {
     }
   }
 
-  if ((split_after <= 0) && !no_linking)
+  if (!cluster_helper->splitting() && !no_linking)
     mxwarn(_("'--link' is only useful in combination with '--split'.\n"));
 
   delete ti;
@@ -1954,14 +2000,11 @@ main(int argc,
   time_t start, end;
 
   init_globals();
-
+  set_usage();
   setup();
 
   parse_args(command_line_utf8(argc, argv));
 
-  if (split_after > 0)
-    splitting = true;
-
   start = time(NULL);
 
   create_readers();
