@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: output_control.cpp 2872 2005-04-07 08:46:58Z mosu $
+   \version $Id: output_control.cpp 3022 2005-08-15 20:47:56Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -85,6 +85,7 @@
 #include "r_srt.h"
 #include "r_ssa.h"
 #include "r_tta.h"
+#include "r_usf.h"
 #include "r_vobbtn.h"
 #include "r_vobsub.h"
 #include "r_wav.h"
@@ -124,9 +125,6 @@ bool write_cues = true, cue_writing_requested = false;
 generic_packetizer_c *video_packetizer = NULL;
 bool write_meta_seek_for_clusters = true;
 bool no_lacing = false, no_linking = true;
-int64_t split_after = -1;
-bool split_by_time = false;
-int split_max_num_files = 65535;
 bool use_durations = false;
 
 double timecode_scale = TIMECODE_SCALE;
@@ -172,7 +170,8 @@ int64_t tags_size = 0;
 static bool accept_tags = true;
 
 int file_num = 1;
-bool splitting = false;
+
+int split_max_num_files = 65535;
 
 string default_language = "und";
 
@@ -340,8 +339,6 @@ get_file_type(filelist_t &file) {
   else if (mpeg_es_reader_c::probe_file(io, size))
     type = FILE_TYPE_MPEG_ES;
   else {
-    delete io;
-
     try {
       text_io = new mm_text_io_c(new mm_file_io_c(file.name));
       text_io->setFilePointer(0, seek_end);
@@ -359,10 +356,12 @@ get_file_type(filelist_t &file) {
       type = FILE_TYPE_SSA;
     else if (vobsub_reader_c::probe_file(text_io, size))
       type = FILE_TYPE_VOBSUB;
+    else if (usf_reader_c::probe_file(text_io, size))
+      type = FILE_TYPE_USF;
     else
       type = FILE_TYPE_IS_UNKNOWN;
 
-    io = text_io;
+    delete text_io;
   }
 
   delete io;
@@ -418,6 +417,35 @@ add_tags(KaxTag *tags) {
   kax_tags->PushElement(*tags);
 }
 
+/** \brief Add an attachment
+
+   \param attachment The attachment specification to add
+   \return The attachment UID created for this attachment.
+*/
+int64_t
+add_attachment(attachment_t attachment) {
+  vector<attachment_t>::iterator i;
+
+  // If the attachment is coming from an existing file then we should
+  // check if we already have another attachment stored. This can happen
+  // if we're concatenating files.
+  if (0 != attachment.id) {
+    foreach(i, attachments)
+      if (((i->id == attachment.id) && !hack_engaged(ENGAGE_NO_VARIABLE_DATA))
+          ||
+          ((i->name == attachment.name) &&
+           (i->description == attachment.description) &&
+           (i->data->size == attachment.data->size)))
+        return attachment.id;
+    add_unique_uint32(attachment.id, UNIQUE_ATTACHMENT_IDS);
+  } else
+    // No ID yet. Let's assign one.
+    attachment.id = create_unique_uint32(UNIQUE_ATTACHMENT_IDS);
+
+  attachments.push_back(attachment);
+  return attachment.id;
+}
+
 /** \brief Add a packetizer to the list of packetizers
 */
 void
@@ -425,7 +453,6 @@ add_packetizer_globally(generic_packetizer_c *packetizer) {
   vector<filelist_t>::iterator file;
   packetizer_t pack;
 
-  memset(&pack, 0, sizeof(packetizer_t));
   pack.packetizer = packetizer;
   pack.orig_packetizer = packetizer;
   pack.status = FILE_STATUS_MOREDATA;
@@ -446,7 +473,7 @@ static void
 set_timecode_scale() {
   vector<packetizer_t>::const_iterator ptzr;
   bool video_present, audio_present;
-  int64_t highest_sample_rate;
+  int64_t highest_sample_rate, max_ns_with_timecode_scale;
 
   video_present = false;
   audio_present = false;
@@ -467,23 +494,15 @@ set_timecode_scale() {
   if (timecode_scale_mode != TIMECODE_SCALE_MODE_FIXED) {
     if (audio_present && (highest_sample_rate > 0) &&
         (!video_present ||
-         (timecode_scale_mode == TIMECODE_SCALE_MODE_AUTO))) {
-      int64_t max_ns_with_timecode_scale;
-
+         (timecode_scale_mode == TIMECODE_SCALE_MODE_AUTO)))
       timecode_scale = (double)1000000000.0 / (double)highest_sample_rate -
         1.0;
-      max_ns_with_timecode_scale = (int64_t)(32700 * timecode_scale);
-      if (max_ns_with_timecode_scale < max_ns_per_cluster)
-        max_ns_per_cluster = max_ns_with_timecode_scale;
-
-      mxverb(2, "mkvmerge: using sample precision timestamps. highest sample "
-             "rate: %lld, new timecode_scale: %f, "
-             "max_ns_with_timecode_scale: %lld, max_ns_per_cluster: %lld\n",
-             highest_sample_rate, timecode_scale, max_ns_with_timecode_scale,
-             max_ns_per_cluster);
-    }
   }
 
+  max_ns_with_timecode_scale = (int64_t)(32700 * timecode_scale);
+  if (max_ns_with_timecode_scale < max_ns_per_cluster)
+    max_ns_per_cluster = max_ns_with_timecode_scale;
+
   KaxTimecodeScale &time_scale = GetChild<KaxTimecodeScale>(*kax_infos);
   *(static_cast<EbmlUInteger *>(&time_scale)) = (int64_t)timecode_scale;
 
@@ -601,7 +620,7 @@ render_headers(mm_io_c *rout) {
       KaxNextUID &kax_nextuid = GetChild<KaxNextUID>(*kax_infos);
       kax_nextuid.CopyBuffer(seguid_link_next->data(), 128 / 8);
     }
-    if (!no_linking && splitting) {
+    if (!no_linking && cluster_helper->splitting()) {
       KaxNextUID &kax_nextuid = GetChild<KaxNextUID>(*kax_infos);
       kax_nextuid.CopyBuffer(seguid_next.data(), 128 / 8);
 
@@ -682,78 +701,6 @@ rerender_track_headers() {
   out->restore_pos();
 }
 
-static uint32_t
-kax_attachment_get_uid(const KaxAttached &a) {
-  KaxFileUID *uid;
-
-  uid = FINDFIRST(&a, KaxFileUID);
-  if (uid == NULL)
-    throw false;
-  return uint32(*static_cast<EbmlUInteger *>(uid));
-}
-
-static UTFstring
-kax_attachment_get_name(const KaxAttached &a) {
-  KaxFileName *name;
-
-  name = FINDFIRST(&a, KaxFileName);
-  if (name == NULL)
-    throw false;
-  return UTFstring(*static_cast<EbmlUnicodeString *>(name));
-}
-
-static UTFstring
-kax_attachment_get_description(const KaxAttached &a) {
-  KaxFileDescription *description;
-
-  description = FINDFIRST(&a, KaxFileDescription);
-  if (description == NULL)
-    return L"";
-  return UTFstring(*static_cast<EbmlUnicodeString *>(description));
-}
-
-static string
-kax_attachment_get_mime_type(const KaxAttached &a) {
-  KaxMimeType *mime_type;
-
-  mime_type = FINDFIRST(&a, KaxMimeType);
-  if (mime_type == NULL)
-    throw false;
-  return string(*static_cast<EbmlString *>(mime_type));
-
-}
-
-static int64_t
-kax_attachment_get_size(const KaxAttached &a) {
-  KaxFileData *data;
-
-  data = FINDFIRST(&a, KaxFileData);
-  if (data == NULL)
-    throw false;
-  return data->GetSize();
-}
-
-static bool
-operator ==(const KaxAttached &a1,
-            const KaxAttached &a2) {
-  try {
-    if (kax_attachment_get_uid(a1) == kax_attachment_get_uid(a2))
-      return true;
-    if (kax_attachment_get_name(a1) != kax_attachment_get_name(a2))
-      return false;
-    if (kax_attachment_get_description(a1) !=
-        kax_attachment_get_description(a2))
-      return false;
-    if (kax_attachment_get_size(a1) != kax_attachment_get_size(a2))
-      return false;
-    if (kax_attachment_get_mime_type(a1) != kax_attachment_get_mime_type(a2))
-      return false;
-    return true;
-  } catch(...) {
-    return false;
-  }
-}
-
 /** \brief Render all attachments into the output file at the current position
 
    This function also makes sure that no duplicates are output. This might
@@ -761,114 +708,53 @@ operator ==(const KaxAttached &a1,
 */
 static void
 render_attachments(IOCallback *rout) {
-  KaxAttachments *other_as, *other_as_test;
   KaxAttached *kax_a;
   KaxFileData *fdata;
-  vector<attachment_t>::const_iterator attch;
-  int i, k;
+  vector<attachment_t>::iterator attch;
   int name;
-  binary *buffer;
-  mm_io_c *io;
 
   if (kax_as != NULL)
     delete kax_as;
   kax_as = new KaxAttachments();
   kax_a = NULL;
   foreach(attch, attachments) {
-    if ((file_num == 1) || (*attch).to_all_files) {
+    if ((file_num == 1) || attch->to_all_files) {
       if (kax_a == NULL)
         kax_a = &GetChild<KaxAttached>(*kax_as);
       else
         kax_a = &GetNextChild<KaxAttached>(*kax_as, *kax_a);
 
-      if ((*attch).description != "")
+      if (attch->description != "")
         *static_cast<EbmlUnicodeString *>
           (&GetChild<KaxFileDescription>(*kax_a)) =
-          cstrutf8_to_UTFstring((*attch).description);
+          cstrutf8_to_UTFstring(attch->description);
 
-      if ((*attch).mime_type != "")
+      if (attch->mime_type != "")
         *static_cast<EbmlString *>(&GetChild<KaxMimeType>(*kax_a)) =
-          (*attch).mime_type;
+          attch->mime_type;
 
-      name = (*attch).name.length() - 1;
-      while ((name > 0) && ((*attch).name[name] != PATHSEP))
+      name = attch->name.length() - 1;
+      while ((name > 0) && (attch->name[name] != PATHSEP))
         name--;
-      if ((*attch).name[name] == PATHSEP)
+      if (attch->name[name] == PATHSEP)
         name++;
 
       *static_cast<EbmlUnicodeString *>
         (&GetChild<KaxFileName>(*kax_a)) =
-        cstr_to_UTFstring((*attch).name.substr(name));
+        cstr_to_UTFstring(attch->name.substr(name));
 
       *static_cast<EbmlUInteger *>
-        (&GetChild<KaxFileUID>(*kax_a)) =
-        create_unique_uint32(UNIQUE_ATTACHMENT_IDS);
+        (&GetChild<KaxFileUID>(*kax_a)) = attch->id;
 
-      try {
-        int64_t size;
-
-        io = new mm_file_io_c((*attch).name);
-        size = io->get_size();
-        buffer = new binary[size];
-        io->read(buffer, size);
-        delete io;
-
-        fdata = &GetChild<KaxFileData>(*kax_a);
-        fdata->SetBuffer(buffer, size);
-      } catch (...) {
-        mxerror(_("The attachment '%s' could not be read.\n"),
-                (*attch).name.c_str());
-      }
+      fdata = &GetChild<KaxFileData>(*kax_a);
+      fdata->CopyBuffer(attch->data->buffer, attch->data->size);
     }
   }
 
-  other_as = new KaxAttachments;
-  other_as_test = new KaxAttachments;
-  for (i = 0; i < files.size(); i++)
-    files[i].reader->add_attachments(other_as_test);
-
-  // Test if such an attachment already exists. This may be the case if
-  // mkvmerge is concatenating files.
-  i = 0;
-  while (i < other_as_test->ListSize()) {
-    bool found;
-
-    kax_a = static_cast<KaxAttached *>((*other_as_test)[i]);
-    found = false;
-    for (k = 0; k < kax_as->ListSize(); k++)
-      if (*static_cast<KaxAttached *>((*kax_as)[k]) == *kax_a) {
-        found = true;
-        break;
-      }
-    if (found) {
-      i++;
-      continue;
-    }
-
-    for (k = 0; k < other_as->ListSize(); k++)
-      if (*static_cast<KaxAttached *>((*other_as)[k]) == *kax_a) {
-        found = true;
-        break;
-      }
-    if (found) {
-      i++;
-      continue;
-    }
-
-    other_as->PushElement(*kax_a);
-    other_as_test->Remove(i);
-  }
-  delete other_as_test;
-
-  while (other_as->ListSize() > 0) {
-    kax_as->PushElement(*(*other_as)[0]);
-    other_as->Remove(0);
-  }
-  delete other_as;
-
   if (kax_as->ListSize() != 0)
     kax_as->Render(*rout);
   else {
+    // Delete the kax_as pointer so that it won't be referenced in a seek head.
     delete kax_as;
     kax_as = NULL;
   }
@@ -1151,52 +1037,28 @@ create_readers() {
   foreach(file, files) {
     try {
       switch (file->type) {
-        case FILE_TYPE_MATROSKA:
-          file->reader = new kax_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_OGM:
-          file->reader = new ogm_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_AVI:
-          file->reader = new avi_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_REAL:
-          file->reader = new real_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_WAV:
-          file->reader = new wav_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_SRT:
-          file->reader = new srt_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_MP3:
-          file->reader = new mp3_reader_c(*file->ti);
+        case FILE_TYPE_AAC:
+          file->reader = new aac_reader_c(*file->ti);
           break;
         case FILE_TYPE_AC3:
           file->reader = new ac3_reader_c(*file->ti);
           break;
+        case FILE_TYPE_AVI:
+          file->reader = new avi_reader_c(*file->ti);
+          break;
         case FILE_TYPE_DTS:
           file->reader = new dts_reader_c(*file->ti);
           break;
-        case FILE_TYPE_AAC:
-          file->reader = new aac_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_SSA:
-          file->reader = new ssa_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_VOBSUB:
-          file->reader = new vobsub_reader_c(*file->ti);
-          break;
-        case FILE_TYPE_QTMP4:
-          file->reader = new qtmp4_reader_c(*file->ti);
-          break;
 #if defined(HAVE_FLAC_FORMAT_H)
         case FILE_TYPE_FLAC:
           file->reader = new flac_reader_c(*file->ti);
           break;
 #endif
-        case FILE_TYPE_TTA:
-          file->reader = new tta_reader_c(*file->ti);
+        case FILE_TYPE_MATROSKA:
+          file->reader = new kax_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_MP3:
+          file->reader = new mp3_reader_c(*file->ti);
           break;
         case FILE_TYPE_MPEG_ES:
           file->reader = new mpeg_es_reader_c(*file->ti);
@@ -1204,9 +1066,36 @@ create_readers() {
         case FILE_TYPE_MPEG_PS:
           file->reader = new mpeg_ps_reader_c(*file->ti);
           break;
+        case FILE_TYPE_OGM:
+          file->reader = new ogm_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_QTMP4:
+          file->reader = new qtmp4_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_REAL:
+          file->reader = new real_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_SSA:
+          file->reader = new ssa_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_SRT:
+          file->reader = new srt_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_TTA:
+          file->reader = new tta_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_USF:
+          file->reader = new usf_reader_c(*file->ti);
+          break;
+        case FILE_TYPE_WAV:
+          file->reader = new wav_reader_c(*file->ti);
+          break;
         case FILE_TYPE_VOBBTN:
           file->reader = new vobbtn_reader_c(*file->ti);
           break;
+        case FILE_TYPE_VOBSUB:
+          file->reader = new vobsub_reader_c(*file->ti);
+          break;
         case FILE_TYPE_WAVPACK4:
           file->reader = new wavpack_reader_c(*file->ti);
           break;
@@ -1214,9 +1103,10 @@ create_readers() {
           mxerror(_("EVIL internal bug! (unknown file type). %s\n"), BUGMSG);
           break;
       }
-    } catch (error_c error) {
-      mxerror(_("The demultiplexer failed to initialize:\n%s\n)"),
-              error.get_error());
+    } catch (error_c &error) {
+      mxerror(_("The demultiplexer for the file '%s' failed to initialize:"
+                "\n%s\n"),
+              file->ti->fname.c_str(), error.get_error().c_str());
     }
   }
 
@@ -1242,9 +1132,9 @@ create_readers() {
 
     // Calculate the size of all attachments for split control.
     foreach(att, attachments) {
-      attachment_sizes_first += att->size;
+      attachment_sizes_first += att->data->size;
       if (att->to_all_files)
-        attachment_sizes_others += att->size;
+        attachment_sizes_others += att->data->size;
     }
 
     calc_max_chapter_size();
@@ -1376,7 +1266,7 @@ create_next_output_file() {
   kax_cues = new KaxCues();
   kax_cues->SetGlobalTimecodeScale((int64_t)timecode_scale);
 
-  if (splitting)
+  if (cluster_helper->splitting())
     this_outfile = create_output_name();
   else
     this_outfile = outfile;
@@ -1437,6 +1327,14 @@ finish_file(bool last_file) {
 
   mxinfo("\n");
 
+  // Render the track headers a second time if the user has requested that.
+  if (hack_engaged(ENGAGE_WRITE_HEADERS_TWICE)) {
+    EbmlElement *second_tracks = kax_tracks.Clone();
+    second_tracks->Render(*out);
+    kax_sh_main->IndexThis(*second_tracks, *kax_segment);
+    delete second_tracks;
+  }
+
   // Render the cues.
   if (write_cues && cue_writing_requested) {
     if (verbose >= 1)
@@ -1499,6 +1397,12 @@ finish_file(bool last_file) {
   }
   out->restore_pos();
 
+  // Render the segment info a second time if the user has requested that.
+  if (hack_engaged(ENGAGE_WRITE_HEADERS_TWICE)) {
+    kax_infos->Render(*out);
+    kax_sh_main->IndexThis(*kax_infos, *kax_segment);
+  }
+
   // Select the chapters that lie in this file and render them in the space
   // that was resesrved at the beginning.
   if (kax_chapters != NULL) {
@@ -1510,7 +1414,7 @@ finish_file(bool last_file) {
     end = start + cluster_helper->get_duration();
 
     chapters_here = copy_chapters(kax_chapters);
-    if (splitting)
+    if (cluster_helper->splitting())
       chapters_here = select_chapters_in_timeframe(chapters_here, start, end,
                                                    offset);
 
@@ -1539,7 +1443,7 @@ finish_file(bool last_file) {
     if (chapters_in_this_file == NULL) {
       KaxChapters temp_chapters;
       tags_here = select_tags_for_chapters(*kax_tags, temp_chapters);
-   } else
+    } else
       tags_here = select_tags_for_chapters(*kax_tags, *chapters_in_this_file);
     if (tags_here != NULL) {
       fix_mandatory_tag_elements(tags_here);
@@ -1566,7 +1470,7 @@ finish_file(bool last_file) {
     if (!hack_engaged(ENGAGE_NO_CHAPTERS_IN_META_SEEK))
       kax_sh_main->IndexThis(*chapters_here, *kax_segment);
     delete chapters_here;
-  } else if (!splitting && (kax_chapters != NULL))
+  } else if (!cluster_helper->splitting() && (kax_chapters != NULL))
     if (!hack_engaged(ENGAGE_NO_CHAPTERS_IN_META_SEEK))
       kax_sh_main->IndexThis(*kax_chapters, *kax_segment);
 
@@ -1795,8 +1699,7 @@ append_tracks_maybe() {
       continue;
     if (!files[ptzr->orig_file].appended_to)
       continue;
-    if ((ptzr->status == FILE_STATUS_MOREDATA) ||
-        (ptzr->status == FILE_STATUS_HOLDING))
+    if (FILE_STATUS_DONE_AND_DRY != ptzr->status)
       continue;
     foreach(amap, append_mapping)
       if ((amap->dst_file_id == ptzr->file) &&
@@ -1851,7 +1754,7 @@ main_loop() {
   // Let's go!
   while (1) {
     vector<packetizer_t>::iterator ptzr, winner;
-    packet_t *pack;
+    packet_cptr pack;
     bool appended_a_track;
 
     // Step 1: Make sure a packet is available for each output
@@ -1860,19 +1763,22 @@ main_loop() {
       if (ptzr->status == FILE_STATUS_HOLDING)
         ptzr->status = FILE_STATUS_MOREDATA;
       ptzr->old_status = ptzr->status;
-      while ((ptzr->pack == NULL) && (ptzr->status == FILE_STATUS_MOREDATA) &&
-             (ptzr->packetizer->packet_available() < 1))
+      while ((ptzr->pack.get() == NULL) &&
+             (ptzr->status == FILE_STATUS_MOREDATA) &&
+             !ptzr->packetizer->packet_available())
         ptzr->status = ptzr->packetizer->read();
       if ((ptzr->status != FILE_STATUS_MOREDATA) &&
           (ptzr->old_status == FILE_STATUS_MOREDATA))
         ptzr->packetizer->force_duration_on_last_packet();
-      if (ptzr->pack == NULL)
+      if (ptzr->pack.get() == NULL)
         ptzr->pack = ptzr->packetizer->get_packet();
+      if ((NULL == ptzr->pack.get()) && (FILE_STATUS_DONE == ptzr->status))
+        ptzr->status = FILE_STATUS_DONE_AND_DRY;
 
       // Has this packetizer changed its status from "data available" to
       // "file done" during this loop? If so then decrease the number of
       // unfinished packetizers in the corresponding file structure.
-      if ((ptzr->status == FILE_STATUS_DONE) &&
+      if ((ptzr->status == FILE_STATUS_DONE_AND_DRY) &&
           (ptzr->old_status != ptzr->status)) {
         filelist_t &file = files[ptzr->file];
 
@@ -1890,10 +1796,10 @@ main_loop() {
     // stuff it into the Matroska file.
     winner = packetizers.end();
     foreach(ptzr, packetizers) {
-      if (ptzr->pack != NULL) {
-        if ((winner == packetizers.end()) || (winner->pack == NULL))
+      if (ptzr->pack.get() != NULL) {
+        if ((winner == packetizers.end()) || (winner->pack.get() == NULL))
           winner = ptzr;
-        else if (ptzr->pack &&
+        else if ((NULL != ptzr->pack.get()) &&
                  (ptzr->pack->assigned_timecode <
                   winner->pack->assigned_timecode))
           winner = ptzr;
@@ -1903,14 +1809,14 @@ main_loop() {
     // Append the next track if appending is wanted.
     appended_a_track = append_tracks_maybe();
 
-    if ((winner != packetizers.end()) && (winner->pack != NULL)) {
+    if ((winner != packetizers.end()) && (winner->pack.get() != NULL)) {
       pack = winner->pack;
 
       // Step 3: Add the winning packet to a cluster. Full clusters will be
       // rendered automatically.
       cluster_helper->add_packet(pack);
 
-      winner->pack = NULL;
+      winner->pack = packet_cptr(NULL);
 
       // display some progress information
       if (verbose >= 1)
@@ -1935,6 +1841,7 @@ main_loop() {
 */
 void
 setup() {
+  init_stdio();
 #if ! defined(SYS_WINDOWS) && defined(HAVE_LIBINTL_H)
   if (setlocale(LC_MESSAGES, "") == NULL)
     mxerror("The locale could not be set properly. Check the "
@@ -1947,7 +1854,6 @@ setup() {
 #endif
 
   mm_file_io_c::setup();
-  srand(time(NULL));
   cc_local_utf8 = utf8_init("");
   init_cc_stdio();
 
@@ -1976,6 +1882,7 @@ destroy_readers() {
 void
 cleanup() {
   delete cluster_helper;
+  cluster_helper = NULL;
   vector<filelist_t>::const_iterator file;
 
   destroy_readers();
@@ -1987,14 +1894,22 @@ cleanup() {
   attachments.clear();
 
   delete kax_tags;
+  kax_tags = NULL;
   delete tags_from_cue_chapters;
+  tags_from_cue_chapters = NULL;
   delete kax_chapters;
+  kax_chapters = NULL;
   delete kax_as;
+  kax_as = NULL;
   delete kax_info_chap;
+  kax_info_chap = NULL;
 
   delete seguid_link_previous;
+  seguid_link_previous = NULL;
   delete seguid_link_next;
+  seguid_link_next = NULL;
   delete seguid_forced;
+  seguid_forced = NULL;
 
   utf8_done();
 }
