@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: pr_generic.cpp 2888 2005-04-16 10:53:18Z mosu $
+   $Id: pr_generic.cpp 3014 2005-08-06 11:52:14Z mosu $
 
    functions common for all readers/packetizers
 
@@ -36,6 +36,18 @@
 
 using namespace std;
 
+int64_t packet_t::packet_number_counter = 0;
+
+packet_t::~packet_t() {
+  vector<unsigned char *>::iterator i;
+
+  safefree(data);
+  foreach(i, data_adds)
+    safefree(*i);
+}
+
+// ---------------------------------------------------------------------
+
 vector<generic_packetizer_c *> ptzrs_in_header_order;
 
 generic_packetizer_c::generic_packetizer_c(generic_reader_c *nreader,
@@ -62,6 +74,8 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *nreader,
   max_timecode_seen = 0;
   default_track_warning_printed = false;
   connected_to = 0;
+  has_been_flushed = false;
+  timecode_factory_application_mode = TFA_AUTOMATIC;
 
   // Let's see if the user specified audio sync for this track.
   found = false;
@@ -265,6 +279,10 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *nreader,
 
 generic_packetizer_c::~generic_packetizer_c() {
   safefree(hcodec_private);
+  if (!packet_queue.empty())
+    mxerror("Packet queue not empty for new track ID %lld (flushed: %d). "
+            "Frames have been lost during remux. %s\n", ti.id,
+            has_been_flushed, BUGMSG);
 }
 
 void
@@ -325,7 +343,8 @@ generic_packetizer_c::set_uid(uint32_t uid) {
 }
 
 void
-generic_packetizer_c::set_track_type(int type) {
+generic_packetizer_c::set_track_type(int type,
+                                     timecode_factory_application_e tfa_mode) {
   htrack_type = type;
 
   if ((type == track_audio) && (ti.cues == CUE_STRATEGY_UNSPECIFIED))
@@ -338,6 +357,16 @@ generic_packetizer_c::set_track_type(int type) {
       video_packetizer = this;
   } else
     reader->num_subtitle_tracks++;
+
+  if ((TFA_AUTOMATIC == tfa_mode) &&
+      (TFA_AUTOMATIC == timecode_factory_application_mode))
+    timecode_factory_application_mode =
+      (track_video == type) ? TFA_FULL_QUEUEING :
+      (track_subtitle == type) ? TFA_IMMEDIATE :
+      (track_buttons == type) ? TFA_IMMEDIATE :
+      TFA_FULL_QUEUEING;
+  else if (TFA_AUTOMATIC != tfa_mode)
+    timecode_factory_application_mode = tfa_mode;
 }
 
 void
@@ -612,8 +641,9 @@ generic_packetizer_c::set_headers() {
       (&GetChild<KaxMaxBlockAdditionID>(*track_entry))) =
       htrack_max_add_block_ids;
 
-  htrack_default_duration =
-    (int64_t)timecode_factory->get_default_duration(htrack_default_duration);
+  if (NULL != timecode_factory)
+    htrack_default_duration =
+      (int64_t)timecode_factory->get_default_duration(htrack_default_duration);
   if (htrack_default_duration != -1.0)
     *(static_cast<EbmlUInteger *>
       (&GetChild<KaxTrackDefaultDuration>(*track_entry))) =
@@ -808,92 +838,39 @@ generic_packetizer_c::fix_headers() {
 }
 
 void
-generic_packetizer_c::add_packet(memory_c &mem,
-                                 int64_t timecode,
-                                 int64_t duration,
-                                 bool duration_mandatory,
-                                 int64_t bref,
-                                 int64_t fref,
-                                 int ref_priority) {
-  int length;
-  packet_t *pack;
-
-  if (reader->ptzr_first_packet == NULL)
-    reader->ptzr_first_packet = this;
-
-  pack = new packet_t;
-
-  length = mem.size;
-  if (NULL != compressor.get()) {
-    pack->data = compressor->compress(mem.data, length);
-    mem.release();
-  } else
-    pack->data = mem.grab();
-  pack->length = length;
-  pack->timecode = timecode;
-  pack->bref = bref;
-  pack->fref = fref;
-  pack->ref_priority = ref_priority;
-  pack->duration = duration;
-  pack->duration_mandatory = duration_mandatory;
-  pack->source = this;
-
-  enqueued_bytes += pack->length;
-
-  if (connected_to != 1)
-    add_packet2(pack);
-  else
-    deferred_packets.push_back(pack);
-}
-
-void
-generic_packetizer_c::add_packet(memories_c &mems,
-                                 int64_t timecode,
-                                 int64_t duration,
-                                 bool duration_mandatory,
-                                 int64_t bref,
-                                 int64_t fref,
-                                 int ref_priority) {
+generic_packetizer_c::add_packet(packet_cptr pack) {
   int length, add_length, i;
-  packet_t *pack;
 
   if (reader->ptzr_first_packet == NULL)
     reader->ptzr_first_packet = this;
 
-  pack = new packet_t;
   // strip elements to be removed
   if ((htrack_max_add_block_ids != -1) &&
-      (htrack_max_add_block_ids < (mems.size() - 1)))
-    mems.resize(htrack_max_add_block_ids + 1);
+      (htrack_max_add_block_ids < pack->memory_adds.size()))
+    pack->memory_adds.resize(htrack_max_add_block_ids);
 
-  pack->data_adds.resize(mems.size() - 1);
-  pack->data_adds_lengths.resize(mems.size() - 1);
+  pack->data_adds.resize(pack->memory_adds.size());
+  pack->data_adds_lengths.resize(pack->memory_adds.size());
 
-  length = mems[0]->size;
+  length = pack->memory->size;
   if (NULL != compressor.get()) {
-    pack->data = compressor->compress(mems[0]->data, length);
-    mems[0]->release();
-    for (i = 1; i < mems.size(); i++) {
-      add_length = mems[i]->size;
-      pack->data_adds[i - 1] = compressor->compress(mems[i]->data,
-                                                    add_length);
-      pack->data_adds_lengths[i - 1] = add_length;
-      mems[i]->release();
+    pack->data = compressor->compress(pack->memory->data, length);
+    pack->memory->release();
+    for (i = 0; i < pack->memory_adds.size(); i++) {
+      add_length = pack->memory_adds[i]->size;
+      pack->data_adds[i] = compressor->compress(pack->memory_adds[i]->data,
+                                                add_length);
+      pack->data_adds_lengths[i] = add_length;
+      pack->memory_adds[i]->release();
     }
   } else {
-    pack->data = mems[0]->grab();
-    for (i = 1; i < mems.size(); i++) {
-      pack->data_adds[i - 1] = mems[i]->grab();
-      pack->data_adds_lengths[i - 1] = mems[i]->size;
+    pack->data = pack->memory->grab();
+    for (i = 0; i < pack->memory_adds.size(); i++) {
+      pack->data_adds[i] = pack->memory_adds[i]->grab();
+      pack->data_adds_lengths[i] = pack->memory_adds[i]->size;
     }
   }
   pack->length = length;
-  pack->timecode = timecode;
-  pack->bref = bref;
-  pack->fref = fref;
-  pack->ref_priority = ref_priority;
-  pack->duration = duration;
-  pack->duration_mandatory = duration_mandatory;
   pack->source = this;
 
   enqueued_bytes += pack->length;
@@ -905,9 +882,7 @@ generic_packetizer_c::add_packet(memories_c &mems,
 }
 
 void
-generic_packetizer_c::add_packet2(packet_t *pack) {
-  int64_t factory_timecode;
-
+generic_packetizer_c::add_packet2(packet_cptr pack) {
   pack->timecode += correction_timecode_offset + append_timecode_offset;
   if (pack->bref >= 0)
     pack->bref += correction_timecode_offset + append_timecode_offset;
@@ -923,7 +898,6 @@ generic_packetizer_c::add_packet2(packet_t *pack) {
   }
 
   if (pack->timecode < 0) {
-    delete pack;
     return;
   }
 
@@ -967,37 +941,46 @@ generic_packetizer_c::add_packet2(packet_t *pack) {
   }
   safety_last_timecode = pack->timecode;
   safety_last_duration = pack->duration;
-
-  factory_timecode = pack->timecode;
-  pack->gap_following =
-    timecode_factory->get_next(factory_timecode, pack->duration);
-  pack->assigned_timecode = factory_timecode + ti.packet_delay;
-
-  if (max_timecode_seen < (pack->assigned_timecode + pack->duration))
-    max_timecode_seen = pack->assigned_timecode + pack->duration;
-  if (reader->max_timecode_seen < max_timecode_seen)
-    reader->max_timecode_seen = max_timecode_seen;
+  pack->timecode_before_factory = pack->timecode;
 
   packet_queue.push_back(pack);
+  if ((NULL == timecode_factory) ||
+      (TFA_IMMEDIATE == timecode_factory_application_mode))
+    apply_factory_once(pack);
+  else
+    apply_factory();
 }
 
 void
 generic_packetizer_c::process_deferred_packets() {
-  deque<packet_t *>::iterator packet;
+  deque<packet_cptr>::iterator packet;
 
   foreach(packet, deferred_packets)
     add_packet2(*packet);
   deferred_packets.clear();
 }
 
-packet_t *
+bool 
+generic_packetizer_c::has_enough_packets() const {
+  if (packet_queue.size() < 2)
+    return false;
+
+  const packet_cptr &pack = packet_queue.front();
+  deque<packet_cptr>::const_iterator packet = packet_queue.begin();
+
+  ++packet;
+  while ((packet_queue.end() != packet) &&
+         ((*packet)->timecode_before_factory <= pack->timecode_before_factory))
+    ++packet;
+  return (packet_queue.end() != packet);
+}
+
+packet_cptr
 generic_packetizer_c::get_packet() {
-  packet_t *pack;
+  if ((packet_queue.size() == 0) || !packet_queue.front()->factory_applied)
+    return packet_cptr(NULL);
 
-  if (packet_queue.size() == 0)
-    return NULL;
-
-  pack = packet_queue.front();
+  packet_cptr pack = packet_queue.front();
   packet_queue.pop_front();
 
   enqueued_bytes -= pack->length;
@@ -1005,6 +988,136 @@ generic_packetizer_c::get_packet() {
   return pack;
 }
 
+void
+generic_packetizer_c::apply_factory_once(packet_cptr &packet) {
+  if (NULL == timecode_factory) {
+    packet->assigned_timecode = packet->timecode;
+    packet->gap_following = false;
+  } else
+    packet->gap_following = timecode_factory->get_next(packet);
+  packet->factory_applied = true;
+
+  mxverb(4, "apply_factory_once(): source %lld t %lld tbf %lld at %lld\n",
+         packet->source->get_source_track_num(), packet->timecode,
+         packet->timecode_before_factory, packet->assigned_timecode);
+
+  if (max_timecode_seen < (packet->assigned_timecode + packet->duration))
+    max_timecode_seen = packet->assigned_timecode + packet->duration;
+  if (reader->max_timecode_seen < max_timecode_seen)
+    reader->max_timecode_seen = max_timecode_seen;
+}
+
+void
+generic_packetizer_c::apply_factory() {
+  packet_cptr_di p_start;
+
+  if (packet_queue.empty())
+    return;
+
+  // Find the first packet to which the factory hasn't been applied yet.
+  p_start = packet_queue.begin();
+  while ((packet_queue.end() != p_start) && (*p_start)->factory_applied)
+    ++p_start;
+
+  if (packet_queue.end() == p_start)
+    return;
+
+  if (TFA_SHORT_QUEUEING == timecode_factory_application_mode)
+    apply_factory_short_queueing(p_start);
+  else
+    apply_factory_full_queueing(p_start);
+}
+
+void
+generic_packetizer_c::apply_factory_short_queueing(packet_cptr_di &p_start) {
+  packet_cptr_di p_end, p_current;
+
+  while (packet_queue.end() != p_start) {
+    // Find the next packet with a timecode bigger than the start packet's
+    // timecode. All packets between those two including the start packet
+    // and excluding the end packet can be timestamped.
+    p_end = p_start + 1;
+    while ((packet_queue.end() != p_end) &&
+           ((*p_end)->timecode_before_factory <
+            (*p_start)->timecode_before_factory))
+      ++p_end;
+
+    // Abort if no such packet was found, but keep on assigning if the
+    // packetizer has been flushed already.
+    if (!has_been_flushed && (packet_queue.end() == p_end))
+      return;
+  
+    // Now assign timecodes to the ones between p_start and p_end...
+    for (p_current = p_start + 1; p_current != p_end; ++p_current)
+      apply_factory_once(*p_current);
+    // ...and to p_start itself.
+    apply_factory_once(*p_start);
+
+    p_start = p_end;
+  }
+}
+
+struct packet_sorter_t {
+  int m_index;
+  static deque<packet_cptr> *m_packet_queue;
+
+  packet_sorter_t(int index):
+    m_index(index) {}
+
+  bool operator <(const packet_sorter_t &cmp) const {
+    return (*m_packet_queue)[m_index]->timecode <
+      (*m_packet_queue)[cmp.m_index]->timecode;
+  }
+};
+
+deque<packet_cptr> *packet_sorter_t::m_packet_queue = NULL;
+
+void
+generic_packetizer_c::apply_factory_full_queueing(packet_cptr_di &p_start) {
+  packet_cptr_di p_end, p_current;
+  vector<packet_sorter_t> sorter;
+  bool needs_sorting;
+  int64_t previous_timecode;
+  int i;
+
+  packet_sorter_t::m_packet_queue = &packet_queue;
+
+  while (packet_queue.end() != p_start) {
+    // Find the next I frame packet.
+    p_end = p_start + 1;
+    while ((packet_queue.end() != p_end) &&
+           ((0 <= (*p_end)->fref) || (0 <= (*p_end)->bref)))
+      ++p_end;
+
+    // Abort if no such packet was found, but keep on assigning if the
+    // packetizer has been flushed already.
+    if (!has_been_flushed && (packet_queue.end() == p_end))
+      return;
+
+    // Now sort the frames by their timecode as the factory has to be
+    // applied to the packets in the same order as they're timestamped.
+    sorter.clear();
+    needs_sorting = false;
+    previous_timecode = 0;
+    i = distance(packet_queue.begin(), p_start);
+    for (p_current = p_start; p_current != p_end; ++i, ++p_current) {
+      sorter.push_back(packet_sorter_t(i));
+      if (packet_queue[i]->timecode < previous_timecode)
+        needs_sorting = true;
+      previous_timecode = packet_queue[i]->timecode;
+    }
+
+    if (needs_sorting)
+      sort(sorter.begin(), sorter.end());
+
+    // Finally apply the factory.
+    for (i = 0; sorter.size() > i; ++i)
+      apply_factory_once(packet_queue[sorter[i].m_index]);
+
+    p_start = p_end;
+  }
+}
+
 void
 generic_packetizer_c::displace(float by_ns) {
   ti.async.displacement += (int64_t)by_ns;
@@ -1024,14 +1137,12 @@ generic_packetizer_c::displace(float by_ns) {
 
 void
 generic_packetizer_c::force_duration_on_last_packet() {
-  packet_t *packet;
-
   if (packet_queue.empty()) {
     mxverb(2, "force_duration_on_last_packet: packet queue is empty for "
            "'%s'/%lld\n", ti.fname.c_str(), ti.id);
     return;
   }
-  packet = packet_queue.back();
+  packet_cptr &packet = packet_queue.back();
   packet->duration_mandatory = true;
   mxverb(2, "force_duration_on_last_packet: forcing at " FMT_TIMECODE " with "
          "%.3fms for '%s'/%lld\n", ARG_TIMECODE_NS(packet->timecode),
@@ -1110,6 +1221,12 @@ generic_packetizer_c::contains_gap() {
     return false;
 }
 
+void
+generic_packetizer_c::flush() {
+  has_been_flushed = true;
+  apply_factory();
+}
+
 //--------------------------------------------------------------------
 
 #define add_all_requested_track_ids(container) \
@@ -1299,6 +1416,14 @@ generic_reader_c::get_queued_bytes() {
   return bytes;
 }
 
+void
+generic_reader_c::flush_packetizers() {
+  vector<generic_packetizer_c *>::const_iterator it;
+
+  foreach(it, reader_packetizers)
+    (*it)->flush();
+}
+
 //
 //--------------------------------------------------------------------
 
