@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: pr_generic.h 2888 2005-04-16 10:53:18Z mosu $
+   $Id: pr_generic.h 3022 2005-08-15 20:47:56Z mosu $
 
    class definition for the generic reader and packetizer
 
@@ -29,9 +29,11 @@
 #include <matroska/KaxTracks.h>
 #include <matroska/KaxTags.h>
 
+#include "common_memory.h"
 #include "compression.h"
 #include "error.h"
 #include "mm_io.h"
+#include "packet.h"
 #include "smart_pointers.h"
 #include "timecode_factory.h"
 
@@ -63,51 +65,11 @@ class generic_reader_c;
 
 enum file_status_e {
   FILE_STATUS_DONE = 0,
+  FILE_STATUS_DONE_AND_DRY,
   FILE_STATUS_HOLDING,
   FILE_STATUS_MOREDATA
 };
 
-class memory_c {
-public:
-  unsigned char *data;
-  uint32_t size;
-  bool is_free;
-
-public:
-  memory_c(unsigned char *ndata, uint32_t nsize, bool nis_free):
-    data(ndata), size(nsize), is_free(nis_free) {
-    if (data == NULL)
-      die("memory_c::memory_c: data = %p, size = %u\n", data, size);
-  }
-  memory_c(const memory_c &src) {
-    die("memory_c::memory_c(const memory_c &) called\n");
-  }
-  ~memory_c() {
-    release();
-  }
-  int lock() {
-    is_free = false;
-    return 0;
-  }
-  unsigned char *grab() {
-    if (is_free) {
-      is_free = false;
-      return data;
-    }
-    return (unsigned char *)safememdup(data, size);
-  }
-  int release() {
-    if (is_free) {
-      safefree(data);
-      data = NULL;
-      is_free = false;
-    }
-    return 0;
-  }
-};
-
-typedef std::vector<memory_c *> memories_c;
-
 struct audio_sync_t {
   int64_t displacement;
   double linear;
@@ -133,38 +95,6 @@ enum default_track_priority_e {
 #define FMT_FN "'%s': "
 #define FMT_TID "'%s' track %lld: "
 
-struct packet_t {
-  KaxBlockGroup *group;
-  KaxBlock *block;
-  KaxCluster *cluster;
-  unsigned char *data;
-  int length, ref_priority;
-  int64_t timecode, bref, fref, duration, packet_num, assigned_timecode;
-  int64_t unmodified_assigned_timecode, unmodified_duration;
-  bool duration_mandatory, superseeded, gap_following;
-  generic_packetizer_c *source;
-  vector<unsigned char *> data_adds;
-  vector<int> data_adds_lengths;
-
-  packet_t():
-    group(NULL), block(NULL), cluster(NULL), data(NULL), length(0),
-    ref_priority(0),
-    timecode(0), bref(0), fref(0), duration(0),
-    packet_num(0),
-    assigned_timecode(0), unmodified_assigned_timecode(0),
-    unmodified_duration(0),
-    duration_mandatory(false), superseeded(false), gap_following(false),
-    source(NULL) {}
-
-  virtual ~packet_t() {
-    vector<unsigned char *>::iterator i;
-
-    safefree(data);
-    foreach(i, data_adds)
-      safefree(*i);
-  }
-};
-
 struct cue_creation_t {
   cue_strategy_e cues;
   int64_t id;
@@ -380,8 +310,6 @@ public:
     create_packetizer(0);
   }
 
-  virtual void add_attachments(KaxAttachments *a) {
-  }
   virtual int add_packetizer(generic_packetizer_c *ptzr);
   virtual void set_timecode_offset(int64_t offset);
 
@@ -393,6 +321,8 @@ public:
 
   virtual int64_t get_queued_bytes();
 
+  virtual void flush_packetizers();
+
 protected:
   virtual bool demuxing_requested(char type, int64_t id);
 };
@@ -442,9 +372,11 @@ enum connection_result_e {
     return CAN_CONNECT_NO_PARAMETERS; \
   }
 
+typedef deque<packet_cptr>::iterator packet_cptr_di;
+
 class generic_packetizer_c {
 protected:
-  deque<packet_t *> packet_queue, deferred_packets;
+  deque<packet_cptr> packet_queue, deferred_packets;
 
   int64_t initial_displacement;
   int64_t m_free_refs, m_next_free_refs, enqueued_bytes;
@@ -475,9 +407,12 @@ protected:
   compressor_ptr compressor;
 
   timecode_factory_c *timecode_factory;
+  timecode_factory_application_e timecode_factory_application_mode;
 
   int64_t last_cue_timecode;
 
+  bool has_been_flushed;
+
 public:
   track_info_c ti;
   generic_reader_c *reader;
@@ -497,49 +432,38 @@ public:
     return reader->read(this);
   }
 
-  virtual void add_packet(memory_c &mem, int64_t timecode,
-                          int64_t duration, bool duration_mandatory = false,
-                          int64_t bref = -1, int64_t fref = -1,
-                          int ref_priority = -1);
-  virtual void add_packet(memories_c &mems, int64_t timecode,
-                          int64_t duration, bool duration_mandatory = false,
-                          int64_t bref = -1, int64_t fref = -1,
-                          int ref_priority = -1);
-  virtual void add_packet2(packet_t *pack);
+  inline void add_packet(packet_t *packet) {
+    add_packet(packet_cptr(packet));
+  }
+  virtual void add_packet(packet_cptr packet);
+  virtual void add_packet2(packet_cptr pack);
   virtual void process_deferred_packets();
 
-  virtual packet_t *get_packet();
-  virtual int packet_available() {
-    return packet_queue.size();
-  }
-  virtual void flush() {
+  virtual packet_cptr get_packet();
+  inline bool packet_available() {
+    return !packet_queue.empty() && packet_queue.front()->factory_applied;
   }
+  virtual void flush();
   virtual int64_t get_smallest_timecode() {
-    return (packet_queue.size() == 0) ? 0x0FFFFFFF :
-      packet_queue.front()->timecode;
+    return packet_queue.empty() ? 0x0FFFFFFF : packet_queue.front()->timecode;
   }
-  virtual int64_t get_queued_bytes() {
+  inline int64_t get_queued_bytes() {
     return enqueued_bytes;
   }
 
-  virtual void set_free_refs(int64_t free_refs) {
+  inline void set_free_refs(int64_t free_refs) {
     m_free_refs = m_next_free_refs;
     m_next_free_refs = free_refs;
   }
-  virtual int64_t get_free_refs() {
+  inline int64_t get_free_refs() {
     return m_free_refs;
   }
   virtual void set_headers();
   virtual void fix_headers();
-  virtual int process(memory_c &mem,
-                      int64_t timecode = -1, int64_t length = -1,
-                      int64_t bref = -1, int64_t fref = -1) = 0;
-
-  virtual int process(memories_c &mems,
-                      int64_t timecode = -1, int64_t length = -1,
-                      int64_t bref = -1, int64_t fref = -1) {
-    return process(*mems[0], timecode, length, bref, fref);
+  inline int process(packet_t *packet) {
+    return process(packet_cptr(packet));
   }
+  virtual int process(packet_cptr packet) = 0;
 
   virtual void dump_debug_info() = 0;
 
@@ -570,7 +494,9 @@ public:
   virtual int get_uid() {
     return huid;
   }
-  virtual void set_track_type(int type);
+  virtual void set_track_type(int type,
+                              timecode_factory_application_e tfa_mode =
+                              TFA_AUTOMATIC);
   virtual int get_track_type() {
     return htrack_type;
   }
@@ -642,6 +568,14 @@ public:
   }
 
   virtual void set_displacement_maybe(int64_t displacement);
+
+  virtual void apply_factory();
+  virtual void apply_factory_once(packet_cptr &packet);
+  virtual void apply_factory_short_queueing(packet_cptr_di &p_start);
+  virtual void apply_factory_full_queueing(packet_cptr_di &p_start);
+
+protected:
+  inline bool has_enough_packets() const;
 };
 
 extern vector<generic_packetizer_c *> ptzrs_in_header_order;
