@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: jobs.cpp 2711 2005-01-12 15:03:47Z mosu $
+   $Id: jobs.cpp 2849 2005-03-13 17:47:27Z mosu $
 
    job queue management dialog
 
@@ -155,8 +155,10 @@ job_run_dialog::start_next_job() {
   mdlg->load(wxString::Format(wxT("%s/jobs/%d.mmg"), wxGetCwd().c_str(),
                               jobs[ndx].id));
 
-  opt_file_name.Printf(wxT("mmg-mkvmerge-options-%d-%d"),
+  opt_file_name.Printf(wxT("%smmg-mkvmerge-options-%d-%d"),
+                       get_temp_dir().c_str(),
                        (int)wxGetProcessId(), (int)time(NULL));
+  
   try {
     opt_file = new wxFile(opt_file_name, wxFile::write);
   } catch (...) {
@@ -191,8 +193,8 @@ job_run_dialog::start_next_job() {
 
   process = new wxProcess(this, 1);
   process->Redirect();
-  pid = wxExecute((*arg_list)[0] + wxT(" @") + opt_file_name, wxEXEC_ASYNC,
-                  process);
+  pid = wxExecute((*arg_list)[0] + wxT(" \"@") + opt_file_name + wxT("\""),
+                  wxEXEC_ASYNC, process);
   out = process->GetInputStream();
 
   *jobs[ndx].log = wxT("");
@@ -227,9 +229,9 @@ job_run_dialog::process_input() {
 
     if (got_char && ((c == '\n') || (c == '\r') || out->Eof())) {
 #if WXUNICODE
-      wx_line = wxU(to_utf8(cc_local_utf8, line).c_str());
+      wx_line = wxU(line.c_str());
 #else
-      wx_line = line.c_str();
+      wx_line = from_utf8(cc_local_utf8, line).c_str();
 #endif
       if (wx_line.Find(wxT("progress")) == 0) {
         if (wx_line.Find(wxT("%")) != 0) {
@@ -274,7 +276,7 @@ void
 job_run_dialog::on_end_process(wxProcessEvent &evt) {
   int exit_code, ndx;
   wxString s;
-  const char *status;
+  const wxChar *status;
 
   process_input();
 
@@ -282,16 +284,16 @@ job_run_dialog::on_end_process(wxProcessEvent &evt) {
   exit_code = evt.GetExitCode();
   if (abort) {
     jobs[ndx].status = JOBS_ABORTED;
-    status = "aborted";
+    status = wxT("aborted");
   } else if (exit_code == 0) {
     jobs[ndx].status = JOBS_DONE;
-    status = "completed OK";
+    status = wxT("completed OK");
   } else if (exit_code == 1) {
     jobs[ndx].status = JOBS_DONE_WARNINGS;
-    status = "completed with warnings";
+    status = wxT("completed with warnings");
   } else {
     jobs[ndx].status = JOBS_FAILED;
-    status = "failed";
+    status = wxT("failed");
   }
   jobs[ndx].finished_on = time(NULL);
 
