@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.cpp 2843 2005-03-02 21:34:35Z mosu $
+   $Id: mmg.cpp 3031 2005-08-21 13:04:48Z mosu $
 
    main stuff
 
@@ -96,7 +96,7 @@ wxString cli_options[][2] = {
         "will use a value of 1000000 which means that timecodes and "
         "durations will have a precision of 1ms. For files that will not "
         "contain a video track but at least one audio track mkvmerge "
-        "will automatically chose a timecode scale factor so that all "
+        "will automatically choose a timecode scale factor so that all "
         "timecodes and durations have a precision of one sample. This "
         "causes bigger overhead but allows precise seeking and extraction. "
         "If the magical value -1 is used then mkvmerge will use sample "
@@ -131,6 +131,9 @@ wxString cli_options[][2] = {
   { wxT("--engage force_passthrough_packetizer"),
     wxT("Forces the Matroska reader to use the generic passthrough "
         "packetizer even for known and supported track types.") },
+  { wxT("--engage allow_avc_in_vfw_mode"),
+    wxT("Allows storing AVC/h.264 video in Video-for-Windows compatibility "
+        "mode, e.g. when it is read from an AVI") },
   { wxT("--engage cow"),
     wxT("No help available.") },
   { wxT(""), wxT("") }
@@ -780,7 +783,9 @@ mmg_dialog::mmg_dialog():
   cfg = wxConfigBase::Get();
   cfg->SetPath(wxT("/GUI"));
   if (cfg->Read(wxT("window_position_x"), &window_pos_x) &&
-      cfg->Read(wxT("window_position_y"), &window_pos_y))
+      cfg->Read(wxT("window_position_y"), &window_pos_y) &&
+      (0 < window_pos_x) && (0xffff > window_pos_x) &&
+      (0 < window_pos_y) && (0xffff > window_pos_y))
     Move(window_pos_x, window_pos_y);
   cfg->Read(wxT("warned_chapterditor_not_empty"),
             &warned_chapter_editor_not_empty, false);
@@ -871,10 +876,10 @@ mmg_dialog::on_file_new(wxCommandEvent &evt) {
   cfg = new wxFileConfig(wxT("mkvmerge GUI"), wxT("Moritz Bunkus"), tmp_name);
   tc_output->SetValue(wxT(""));
 
-  input_page->load(cfg);
-  attachments_page->load(cfg);
-  global_page->load(cfg);
-  settings_page->load(cfg);
+  input_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  attachments_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  global_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  settings_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
 
   delete cfg;
   wxRemoveFile(tmp_name);
@@ -908,7 +913,8 @@ mmg_dialog::load(wxString file_name,
 
   cfg = new wxFileConfig(wxT("mkvmerge GUI"), wxT("Moritz Bunkus"), file_name);
   cfg->SetPath(wxT("/mkvmergeGUI"));
-  if (!cfg->Read(wxT("file_version"), &version) || (version != 1)) {
+  if (!cfg->Read(wxT("file_version"), &version) || (1 > version) ||
+      (MMG_CONFIG_FILE_VERSION_MAX < version)) {
     if (used_for_jobs)
       return;
     wxMessageBox(wxT("The file does not seem to be a valid mkvmerge GUI "
@@ -920,10 +926,10 @@ mmg_dialog::load(wxString file_name,
   tc_output->SetValue(s);
   cfg->Read(wxT("cli_options"), &cli_options, wxT(""));
 
-  input_page->load(cfg);
-  attachments_page->load(cfg);
-  global_page->load(cfg);
-  settings_page->load(cfg);
+  input_page->load(cfg, version);
+  attachments_page->load(cfg, version);
+  global_page->load(cfg, version);
+  settings_page->load(cfg, version);
 
   delete cfg;
 
@@ -953,7 +959,7 @@ mmg_dialog::save(wxString file_name,
 
   cfg = new wxFileConfig(wxT("mkvmerge GUI"), wxT("Moritz Bunkus"), file_name);
   cfg->SetPath(wxT("/mkvmergeGUI"));
-  cfg->Write(wxT("file_version"), 1);
+  cfg->Write(wxT("file_version"), MMG_CONFIG_FILE_VERSION_MAX);
   cfg->Write(wxT("gui_version"), wxT(VERSION));
   cfg->Write(wxT("output_file_name"), tc_output->GetValue());
   cfg->Write(wxT("cli_options"), cli_options);
@@ -1147,7 +1153,7 @@ mmg_dialog::muxing_has_finished() {
 void
 mmg_dialog::on_help(wxCommandEvent &evt) {
   if (help == NULL) {
-    wxDirDialog dlg(this, wxT("Chose the location of the mkvmerge GUI "
+    wxDirDialog dlg(this, wxT("Chosoe the location of the mkvmerge GUI "
                               "help files"));
     wxString help_path;
     wxConfigBase *cfg;
@@ -1500,9 +1506,13 @@ mmg_dialog::update_command_line() {
   if (global_page->cb_split->IsChecked()) {
     clargs.Add(wxT("--split"));
     if (global_page->rb_split_by_size->GetValue())
-      clargs.Add(global_page->cob_split_by_size->GetValue());
+      clargs.Add(wxT("size:") + global_page->cob_split_by_size->GetValue());
+    else if (global_page->rb_split_by_time->GetValue())
+      clargs.Add(wxT("duration:") +
+                 global_page->cob_split_by_time->GetValue());
     else
-      clargs.Add(global_page->cob_split_by_time->GetValue());
+      clargs.Add(wxT("timecodes:") +
+                 global_page->tc_split_after_timecodes->GetValue());
 
     if (global_page->tc_split_max_files->GetValue().Length() > 0) {
       clargs.Add(wxT("--split-max-files"));
@@ -1984,6 +1994,7 @@ mmg_app::OnInit() {
   uint32_t i;
   wxString k, v;
 
+  init_stdio();
   mm_file_io_c::setup();
   cc_local_utf8 = utf8_init("");
   xml_element_map_init();
