@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mux_dialog.cpp 2711 2005-01-12 15:03:47Z mosu $
+   $Id: mux_dialog.cpp 2849 2005-03-13 17:47:27Z mosu $
 
    muxing dialog
 
@@ -144,8 +144,8 @@ mux_dialog::mux_dialog(wxWindow *parent):
   }
   delete opt_file;
 
-  pid = wxExecute((*arg_list)[0] + wxT(" @") + opt_file_name, wxEXEC_ASYNC,
-                  process);
+  pid = wxExecute((*arg_list)[0] + wxT(" \"@") + opt_file_name + wxT("\""),
+                  wxEXEC_ASYNC, process);
   out = process->GetInputStream();
 
   line = "";
@@ -158,9 +158,9 @@ mux_dialog::mux_dialog(wxWindow *parent):
 
     if ((c == '\n') || (c == '\r') || out->Eof()) {
 #if WXUNICODE
-      wx_line = wxU(to_utf8(cc_local_utf8, line).c_str());
+      wx_line = wxU(line.c_str());
 #else
-      wx_line = line.c_str();
+      wx_line = from_utf8(cc_local_utf8, line).c_str();
 #endif
       log += wx_line + wxT("\n");
       if (wx_line.Find(wxT("Warning:")) == 0)
