@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_chapters.cpp 2795 2005-02-19 15:42:31Z mosu $
+   $Id: tab_chapters.cpp 2985 2005-07-16 14:51:18Z mosu $
 
    "chapter editor" tab
 
@@ -663,7 +663,7 @@ tab_chapters::load(wxString name) {
     }
   } catch (error_c ex) {
     analyzer = NULL;
-    s = wxU((const char *)ex);
+    s = wxU(ex.get_error().c_str());
     break_line(s);
     while (s[s.Length() - 1] == wxT('\n'))
       s.Remove(s.Length() - 1);
@@ -2016,7 +2016,7 @@ tab_chapters::parse_time(wxString s) {
   c = utf8s.c_str();
   while (*c != 0) {
     if (!isdigit(*c)) {
-      if (parse_timecode(utf8s.c_str(), &nsecs))
+      if (parse_timecode(utf8s, nsecs))
         return nsecs;
       return -1;
     }
