@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_input.cpp 2858 2005-03-19 11:02:04Z mosu $
+   $Id: tab_input.cpp 2995 2005-07-19 20:01:07Z mosu $
 
    "input" tab
 
@@ -660,6 +660,7 @@ tab_input::select_file(bool append) {
     { wxT("SRT text subtitles"), wxT("srt") },
     { wxT("SSA/ASS text subtitles"), wxT("ass ssa") },
     { wxT("TTA (The lossless True Audio codec)"), wxT("tta") },
+    { wxT("USF text subtitles"), wxT("usf xml") },
     { wxT("VobSub subtitles"), wxT("idx") },
     { wxT("WAVE (uncompressed PCM audio)"), wxT("wav") },
     { wxT("WAVPACK v4 audio"), wxT("wv") },
@@ -708,11 +709,13 @@ tab_input::select_file(bool append) {
     a_exts += wxString::Format(wxT("*.%s"), all_extensions[ae].c_str());
   }
 
-  media_files.Printf(wxT("Media files (%s)|%s%s|%s"), a_exts.c_str(),
-                     a_exts.c_str(), rest.c_str(), wxT(ALLFILES));
+  media_files = wxT("All supported media files|");
+  media_files += a_exts + rest;
+  media_files += wxT("|");
+  media_files += wxT(ALLFILES);
   wxFileDialog dlg(NULL,
                    append ? wxT("Choose an input file to append") :
-                   wxT("Chose an input file to add"),
+                   wxT("Choose an input file to add"),
                    last_open_dir, wxT(""), media_files, wxOPEN | wxMULTIPLE);
 
   if(dlg.ShowModal() == wxID_OK) {
@@ -856,8 +859,8 @@ tab_input::add_file(const wxString &file_name,
             dims = split(pair[1], wxU("x"));
             if ((dims.size() == 2) && parse_int(wxMB(dims[0]), width) &&
                 parse_int(wxMB(dims[1]), height)) {
-              track->dwidth.Printf(wxT("%d"), (int)width);
-              track->dheight.Printf(wxT("%d"), (int)height);
+              track->dwidth.Printf(wxT(LLD), width);
+              track->dheight.Printf(wxT(LLD), height);
               track->display_dimensions_selected = true;
             }
 
@@ -1240,6 +1243,9 @@ tab_input::on_track_selected(wxCommandEvent &evt) {
 
   selected_track = -1;
   new_sel = clb_tracks->GetSelection();
+  if (0 > new_sel)
+    return;
+
   t = tracks[new_sel];
   f = &files[t->source];
 
@@ -1549,7 +1555,8 @@ tab_input::save(wxConfigBase *cfg) {
 }
 
 void
-tab_input::load(wxConfigBase *cfg) {
+tab_input::load(wxConfigBase *cfg,
+                int version) {
   long fidx, tidx, i;
   wxString s, c, id, track_order;
   string format;
