@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_aac.cpp 2842 2005-03-02 15:53:34Z mosu $
+   $Id: p_aac.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    AAC output module
 
@@ -156,18 +156,15 @@ aac_packetizer_c::set_headers() {
 }
 
 int
-aac_packetizer_c::process(memory_c &mem,
-                          int64_t timecode,
-                          int64_t,
-                          int64_t,
-                          int64_t) {
-  unsigned char *packet;
+aac_packetizer_c::process(packet_cptr packet) {
+  unsigned char *aac_packet;
   unsigned long header;
   aac_header_t aacheader;
-  int64_t my_timecode, duration;
+  int64_t my_timecode, duration, timecode;
 
   debug_enter("aac_packetizer_c::process");
 
+  timecode = packet->timecode;
   my_timecode = 0;
   if (headerless) {
     if (timecode != -1) {
@@ -193,32 +190,34 @@ aac_packetizer_c::process(memory_c &mem,
       return FILE_STATUS_MOREDATA;
     }
     while (needs_positive_displacement(duration)) {
-      add_packet(mem, my_timecode + ti.async.displacement, duration);
+      add_packet(new packet_t(packet->memory->clone(), my_timecode +
+                              ti.async.displacement, duration));
       displace(duration);
     }
 
-    my_timecode = (int64_t)((my_timecode + ti.async.displacement) *
-                            ti.async.linear);
-    mxverb(2, "aac: my_tc = %lld\n", my_timecode);
-    add_packet(mem, my_timecode, duration);
+    packet->duration = duration;
+    packet->timecode = (int64_t)((my_timecode + ti.async.displacement) *
+                                 ti.async.linear);
+    mxverb(2, "aac: my_tc = %lld\n", packet->timecode);
+    add_packet(packet);
 
     debug_leave("aac_packetizer_c::process");
 
     return FILE_STATUS_MOREDATA;
   }
 
-  byte_buffer.add(mem.data, mem.size);
-  while ((packet = get_aac_packet(&header, &aacheader)) != NULL) {
+  byte_buffer.add(packet->memory->data, packet->memory->size);
+  while ((aac_packet = get_aac_packet(&header, &aacheader)) != NULL) {
     if (timecode == -1)
       my_timecode = (int64_t)(1024 * 1000000000.0 * packetno /
                               samples_per_sec);
     else
       my_timecode = timecode + ti.async.displacement;
-    my_timecode = (int64_t)(my_timecode * ti.async.linear);
-    memory_c mem(packet, aacheader.data_byte_size, true);
-    add_packet(mem, my_timecode,
-               (int64_t)(1024 * 1000000000.0 *
-                         ti.async.linear / samples_per_sec));
+    add_packet(new packet_t(new memory_c(aac_packet,
+                                         aacheader.data_byte_size, true),
+                            (int64_t)(my_timecode * ti.async.linear),
+                            (int64_t)(1024 * 1000000000.0 * ti.async.linear /
+                                      samples_per_sec)));
     packetno++;
   }
 
@@ -229,8 +228,8 @@ aac_packetizer_c::process(memory_c &mem,
 
 void
 aac_packetizer_c::dump_debug_info() {
-  mxdebug("aac_packetizer_c: queue: %d; buffer size: %d\n",
-          packet_queue.size(), byte_buffer.get_size());
+  mxdebug("aac_packetizer_c: queue: %u; buffer size: %d\n",
+          (unsigned int)packet_queue.size(), byte_buffer.get_size());
 }
 
 connection_result_e
