@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_ac3.cpp 2842 2005-03-02 15:53:34Z mosu $
+   $Id: p_ac3.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    AC3 output module
 
@@ -143,30 +143,25 @@ ac3_packetizer_c::set_headers() {
 }
 
 int
-ac3_packetizer_c::process(memory_c &mem,
-                          int64_t timecode,
-                          int64_t,
-                          int64_t,
-                          int64_t) {
-  unsigned char *packet;
+ac3_packetizer_c::process(packet_cptr packet) {
+  unsigned char *ac3_packet;
   unsigned long header;
   ac3_header_t ac3header;
   int64_t my_timecode;
 
   debug_enter("ac3_packetizer_c::process");
 
-  add_to_buffer(mem.data, mem.size);
-  while ((packet = get_ac3_packet(&header, &ac3header)) != NULL) {
-    if (timecode == -1)
+  add_to_buffer(packet->memory->data, packet->memory->size);
+  while ((ac3_packet = get_ac3_packet(&header, &ac3header)) != NULL) {
+    if (packet->timecode == -1)
       my_timecode = (int64_t)(1000000000.0 * packetno * 1536 /
                               samples_per_sec);
     else
-      my_timecode = timecode + ti.async.displacement;
-    my_timecode = (int64_t)(my_timecode * ti.async.linear);
-    memory_c mem(packet, ac3header.bytes, true);
-    add_packet(mem, my_timecode,
-               (int64_t)(1000000000.0 * 1536 * ti.async.linear /
-                         samples_per_sec));
+      my_timecode = packet->timecode + ti.async.displacement;
+    add_packet(new packet_t(new memory_c(ac3_packet, ac3header.bytes, true),
+                            (int64_t)(my_timecode * ti.async.linear),
+                            (int64_t)(1000000000.0 * 1536 * ti.async.linear /
+                                      samples_per_sec)));
     packetno++;
   }
 
@@ -177,8 +172,8 @@ ac3_packetizer_c::process(memory_c &mem,
 
 void
 ac3_packetizer_c::dump_debug_info() {
-  mxdebug("ac3_packetizer_c: queue: %d; buffer size: %d\n",
-          packet_queue.size(), byte_buffer.get_size());
+  mxdebug("ac3_packetizer_c: queue: %u; buffer size: %d\n",
+          (unsigned int)packet_queue.size(), byte_buffer.get_size());
 }
 
 connection_result_e
