@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_mp3.cpp 2842 2005-03-02 15:53:34Z mosu $
+   $Id: p_mp3.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    MP3 output module
 
@@ -186,28 +186,24 @@ mp3_packetizer_c::set_headers() {
 }
 
 int
-mp3_packetizer_c::process(memory_c &mem,
-                          int64_t timecode,
-                          int64_t,
-                          int64_t,
-                          int64_t) {
-  unsigned char *packet;
+mp3_packetizer_c::process(packet_cptr packet) {
+  unsigned char *mp3_packet;
   mp3_header_t mp3header;
   int64_t my_timecode;
 
   debug_enter("mp3_packetizer_c::process");
 
-  byte_buffer.add(mem.data, mem.size);
-  while ((packet = get_mp3_packet(&mp3header)) != NULL) {
-    if (timecode == -1)
+  byte_buffer.add(packet->memory->data, packet->memory->size);
+  while ((mp3_packet = get_mp3_packet(&mp3header)) != NULL) {
+    if (packet->timecode == -1)
       my_timecode = (int64_t)(1000000000.0 * packetno * spf / samples_per_sec);
     else
-      my_timecode = timecode + ti.async.displacement;
-    my_timecode = (int64_t)(my_timecode * ti.async.linear);
-    memory_c mem(packet, mp3header.framesize, true);
-    add_packet(mem, my_timecode,
-               (int64_t)(1000000000.0 * spf * ti.async.linear /
-                         samples_per_sec));
+      my_timecode = packet->timecode + ti.async.displacement;
+    add_packet(new packet_t(new memory_c(mp3_packet, mp3header.framesize,
+                                         true),
+                            (int64_t)(my_timecode * ti.async.linear),
+                            (int64_t)(1000000000.0 * spf * ti.async.linear /
+                                      samples_per_sec)));
     packetno++;
   }
 
@@ -218,8 +214,8 @@ mp3_packetizer_c::process(memory_c &mem,
 
 void
 mp3_packetizer_c::dump_debug_info() {
-  mxdebug("mp3_packetizer_c: queue: %d; buffer_size: %d\n",
-          packet_queue.size(), byte_buffer.get_size());
+  mxdebug("mp3_packetizer_c: queue: %u; buffer_size: %d\n",
+          (unsigned int)packet_queue.size(), byte_buffer.get_size());
 }
 
 connection_result_e
