@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_passthrough.cpp 2842 2005-03-02 15:53:34Z mosu $
+   $Id: p_passthrough.cpp 3024 2005-08-17 11:46:09Z mosu $
 
    PASSTHROUGH output module
 
@@ -30,6 +30,8 @@ passthrough_packetizer_c::passthrough_packetizer_c(generic_reader_c *_reader,
   throw (error_c):
   generic_packetizer_c(_reader, _ti),
   packets_processed(0), bytes_processed(0), sync_to_keyframe(false) {
+
+  timecode_factory_application_mode = TFA_FULL_QUEUEING;
 }
 
 void
@@ -38,52 +40,50 @@ passthrough_packetizer_c::set_headers() {
 }
 
 int
-passthrough_packetizer_c::process(memory_c &mem,
-                                  int64_t timecode,
-                                  int64_t duration,
-                                  int64_t bref,
-                                  int64_t fref) {
-  return process(mem, timecode, duration, bref, fref, false);
-}
+passthrough_packetizer_c::process(packet_cptr packet) {
+  int64_t timecode;
 
-int
-passthrough_packetizer_c::process(memory_c &mem,
-                                  int64_t timecode,
-                                  int64_t duration,
-                                  int64_t bref,
-                                  int64_t fref,
-                                  bool duration_mandatory) {
   debug_enter("passthrough_packetizer_c::process");
 
+  timecode = packet->timecode;
   packets_processed++;
-  bytes_processed += mem.size;
-  if ((duration > 0) && needs_negative_displacement(duration)) {
-    displace(-duration);
+  bytes_processed += packet->memory->size;
+  if ((packet->duration > 0) &&
+      needs_negative_displacement(packet->duration)) {
+    displace(-packet->duration);
     sync_to_keyframe = true;
     return FILE_STATUS_MOREDATA;
   }
-  if ((duration > 0) && needs_positive_displacement(duration) &&
+  if ((packet->duration > 0) &&
+      needs_positive_displacement(packet->duration) &&
       sync_complete_group) {
     // Not implemented yet.
   }
-  while ((duration > 0) && needs_positive_displacement(duration)) {
-    add_packet(mem,
-               (int64_t)((timecode + ti.async.displacement) *
-                         ti.async.linear),
-               duration, duration_mandatory, bref, fref);
-    displace(duration);
+  while ((packet->duration > 0) &&
+         needs_positive_displacement(packet->duration)) {
+    packet_t *new_packet =
+      new packet_t(packet->memory->clone(),
+                   (int64_t)((timecode + ti.async.displacement) *
+                             ti.async.linear),
+                   packet->duration, packet->bref, packet->fref);
+    new_packet->duration_mandatory = packet->duration_mandatory;
+    add_packet(new_packet);
+    displace(packet->duration);
   }
 
-  if (sync_to_keyframe && (bref != -1))
+  if (sync_to_keyframe && (packet->bref != -1))
     return FILE_STATUS_MOREDATA;
   sync_to_keyframe = false;
-  timecode = (int64_t)((timecode + ti.async.displacement) * ti.async.linear);
-  duration = (int64_t)(duration * ti.async.linear);
-  if (bref >= 0)
-    bref = (int64_t)((bref + ti.async.displacement) * ti.async.linear);
-  if (fref >= 0)
-    fref = (int64_t)((fref + ti.async.displacement) * ti.async.linear);
-  add_packet(mem, timecode, duration, duration_mandatory, bref, fref);
+  packet->timecode = (int64_t)((timecode + ti.async.displacement) *
+                               ti.async.linear);
+  packet->duration = (int64_t)(packet->duration * ti.async.linear);
+  if (packet->bref >= 0)
+    packet->bref = (int64_t)((packet->bref + ti.async.displacement) *
+                             ti.async.linear);
+  if (packet->fref >= 0)
+    packet->fref = (int64_t)((packet->fref + ti.async.displacement) *
+                             ti.async.linear);
+  add_packet(packet);
 
   debug_leave("passthrough_packetizer_c::process");
   return FILE_STATUS_MOREDATA;
@@ -97,8 +97,9 @@ passthrough_packetizer_c::always_sync_complete_group(bool sync) {
 void
 passthrough_packetizer_c::dump_debug_info() {
   mxdebug("passthrough_packetizer_c: packets processed: %lld, "
-          "bytes processed: %lld, packets in queue: %d\n",
-          packets_processed, bytes_processed, packet_queue.size());
+          "bytes processed: %lld, packets in queue: %u\n",
+          packets_processed, bytes_processed,
+          (unsigned int)packet_queue.size());
 }
 
 connection_result_e
