@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_vobsub.h 2842 2005-03-02 15:53:34Z mosu $
+   $Id: p_vobsub.h 2932 2005-05-25 21:31:26Z mosu $
 
    class definition for the VobSub output module
 
@@ -35,9 +35,7 @@ public:
     throw (error_c);
   virtual ~vobsub_packetizer_c();
 
-  virtual int process(memory_c &mem,
-                      int64_t old_timecode = -1, int64_t duration = -1,
-                      int64_t bref = -1, int64_t fref = -1);
+  virtual int process(packet_cptr packet);
   virtual void set_headers();
 
   virtual void dump_debug_info();
